Package: SYNCSA
Type: Package
Title: Analysis of Functional and Phylogenetic Patterns in
        Metacommunities
Version: 1.3.5
Date: 2025-08-21
Authors@R: person(given = c("Vanderlei", "Julio"),
                      family = "Debastiani",
                      role = c("aut", "cre"),
                      email = "vanderleidebastiani@yahoo.com.br")
Maintainer: Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
Imports: vegan, FD, permute, RcppArmadillo, utils, stats, graphics,
        parallel
Description: Analysis of metacommunities based on functional traits and
    phylogeny of the community components. The functions that are offered here
    implement for the R environment methods that have been available in the
    'SYNCSA' application written in C++ (by Valerio Pillar, available at 
    <http://ecoqua.ecologia.ufrgs.br/SYNCSA.html>).
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.2
Collate: 'CollectNames.R' 'ProgressBAR.R' 'belonging.R' 'cent.norm.R'
        'coinertia.partial.syncsa.R' 'coinertia.syncsa.R'
        'cor.matrix.R' 'cor.matrix.bf.R' 'cor.matrix.partial.R'
        'cor.matrix.rao.R' 'matmult.syncsa.R' 'matrix.p.R' 'matrix.t.R'
        'matrix.w.transformation.R' 'matrix.x.R' 'optimal.R'
        'organize.syncsa.R' 'part.cor.R' 'pca.R' 'permut.row.matrix.R'
        'permut.vector.R' 'plot.pcasyncsa.R' 'print.optimal.R'
        'syncsa.R' 'print.syncsa.R' 'pro.matrix.R' 'pro.matrix.bf.R'
        'pro.matrix.partial.R' 'pro.matrix.rao.R' 'procrustes.syncsa.R'
        'procrustes.partial.syncsa.R' 'rao.diversity.R' 'rv.matrix.R'
        'rv.matrix.bf.R' 'rv.matrix.partial.R' 'rv.matrix.rao.R'
        'startup.R' 'syncsa.obs.R' 'var.dummy.R' 'var.type.R'
NeedsCompilation: no
Packaged: 2025-08-22 01:13:04 UTC; vanderleidebastiani
Author: Vanderlei Julio Debastiani [aut, cre]
Repository: CRAN
Date/Publication: 2025-08-22 09:30:23 UTC
Built: R 4.6.0; ; 2026-04-26 04:27:55 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,pcasyncsa)
S3method(print,optimal)
S3method(print,syncsa)
export(CollectNames)
export(ProgressBAR)
export(belonging)
export(cent.norm)
export(coinertia.partial.syncsa)
export(coinertia.syncsa)
export(cor.matrix)
export(cor.matrix.bf)
export(cor.matrix.partial)
export(cor.matrix.rao)
export(matmult.syncsa)
export(matrix.p)
export(matrix.t)
export(matrix.w.transformation)
export(matrix.x)
export(optimal)
export(organize.syncsa)
export(part.cor)
export(pca)
export(permut.row.matrix)
export(permut.vector)
export(pro.matrix)
export(pro.matrix.bf)
export(pro.matrix.partial)
export(pro.matrix.rao)
export(procrustes.partial.syncsa)
export(procrustes.syncsa)
export(rao.diversity)
export(rv.matrix)
export(rv.matrix.bf)
export(rv.matrix.partial)
export(rv.matrix.rao)
export(syncsa)
export(syncsa.obs)
export(var.dummy)
export(var.type)
importFrom(FD,gowdis)
importFrom(RcppArmadillo,fastLmPure)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(parallel,makeCluster)
importFrom(parallel,parRapply)
importFrom(parallel,stopCluster)
importFrom(permute,Plots)
importFrom(permute,Within)
importFrom(permute,how)
importFrom(permute,shuffleSet)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,prcomp)
importFrom(stats,residuals)
importFrom(utils,combn)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(vegan,beals)
importFrom(vegan,protest)
importFrom(vegan,vegdist)
importFrom(vegan,wcmdscale)

#' @title Principal Components Analysis (PCA) with NA (missing data)
#'
#' @description The function use the option "pairwise.complete.obs" (in function \code{\link{cor}}) for
#' calculate the correlation. The correlation between each pair of variables is computed
#' using all complete pairs of observations on those variables.
#'
#' @encoding UTF-8
#' @importFrom graphics plot abline text points arrows
#' @aliases pca print.pcasyncsa
#' @param data A data frame  or matrix with individuals in rows and variables in columns.
#' @param x A object of class pcasyncsa.
#' @param show Draw "variables" or "individuals".
#' @param axis Axis for draw, must have length equal to two (Default axis = c(1, 2)).
#' @param xlab Text for x label (Default xlab = axis[1]).
#' @param ylab Text for y label (Default ylab = axis[2]).
#' @param arrows Logical argument (TRUE or FALSE) to specify if arrows are showed for variables (Default arrows = TRUE).
#' @param text Logical argument (TRUE or FALSE) to specify if text are showed for individuals (Default text = TRUE).
#' @param points Logical argument (TRUE or FALSE) to specify if points are showed for individuals (Default points = FALSE).
#' @param ... Parameters for \code{\link{plot}} function.
#' @return \item{decomposition}{list with the results of decomposition of correlation matrix.}
#' \item{eigenvalues}{Data frame containing all the eigenvalues, the
#' percentage of inertia and the cumulative percentage of inertia.}
#' \item{individuals}{Coordinates for the individuals.}
#' \item{variables}{Correlation between original variables and axes.}
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}
#' \code{\link{syncsa}}
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' traits<-ADRS$traits
#' # Some NA
#' traits[c(1,5),1]<-NA
#' traits[3,2]<-NA
#' traits
#' res<-pca(traits)
#' res
#' plot(res, show = "variables", arrows = TRUE)
#' plot(res, show = "individuals", axis = c(1, 2), text = TRUE)
#' plot(res, show = "individuals", text = FALSE, points = TRUE)
#' @export
pca <- function(data)
{
  res <- list(call = match.call())
  colnames(data) <- colnames(data, do.NULL = FALSE, prefix = "Var.")
  rownames(data) <- rownames(data, do.NULL = FALSE, prefix = "Ind.")
  COR <- cor(data, use = "pairwise.complete.obs")
  data.norm <- as.matrix(cent.norm(data, na.rm = TRUE))
  data.NA <- apply(data.norm, 2, is.na)
  data.norm[data.NA] <- 0
  RES_eig <- eigen(COR)
  res$decomposition <- RES_eig
  eig <- RES_eig$values[RES_eig$values>=0]
  res.eig <- data.frame(Eigenvalues = eig, Perc.Inertia = eig/sum(eig), Cum.Inertia = cumsum(eig/sum(eig)))
  rownames(res.eig) <- paste("Axis.", 1:length(eig), sep = "")
  res$eigenvalues <- res.eig
  ind.coord <- data.norm%*%RES_eig$vectors[, RES_eig$values>=0, drop = FALSE]
  colnames(ind.coord) <- paste("Axis.", 1:ncol(ind.coord), sep = "")
  # var.coord <- cor(data, ind.coord, use = "pairwise.complete.obs")
  var.coord <- RES_eig$vectors[, RES_eig$values>=0, drop = FALSE]%*%diag(sqrt(eig))
  colnames(var.coord) <- paste("Axis.", 1:ncol(var.coord), sep = "")
  rownames(var.coord) <- colnames(data)
  res$individuals <- ind.coord
  res$variables <- var.coord
  class(res) <- "pcasyncsa"
  return(res)
}

#' @title Generate dummy variable
#'
#' @description Function to expand factor variables in dummy variables in a data.frame. See details.
#'
#' @details Variables in the data.frame of class factor is expanded in dummy variables. Each
#' level of factor produce a new column in the dataframe, with presence (1) or
#' absense (0) of level. The name of columns is a combination of orginal variable name
#' plus the level separate by underscore ( _ ). Ordered factor and character class are
#' not expanded.
#'
#' @encoding UTF-8
#' @param data A data.frame.
#' @return \item{data}{The data with all variables of class factor expanded.} \item{together}{A list
#' with sugestion to group of traits that are analysed together.}
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso  \code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{var.type}}
#' @keywords Auxiliary
#' @export
var.dummy<-function(data)
{
  if(!inherits(data, "data.frame")){
    stop("data must be a data.frame")
  }
  colnames(data) <- colnames(data, do.NULL = FALSE, prefix = "var")
  type <- var.type(data)
  n <- nrow(data)
  RES <- as.data.frame(rep(NA, n))
  rownames(RES) <- rownames(data)
  m <- 1
  l <- 1
  together <- list()
  for(i in 1:ncol(data)){
    if(type[i] == "f"){
      var_temp <- table(1:n, as.factor(data[,i]))
      colnames(var_temp) <- paste(colnames(data)[i], colnames(var_temp), sep = "_")
      together[[l]] <- colnames(var_temp)
      l <- l+1
      for(j in 1:ncol(var_temp)){
        m <- m+1
        RES[,m] <- as.numeric(var_temp[, j, drop = FALSE])
        colnames(RES)[m] <- colnames(var_temp)[j]
      }
    }else{
      m <- m+1
      RES[, m] <- data[, i, drop = FALSE]
    }
  }
  RES <- RES[, -1, drop = FALSE]
  return(list(data = RES, together = together))
}

#' @title Function to obtain the correlation between two matrices and partial matrix
#' correlation between three matrices.
#'
#' @description The functions cor.matrix and cor.matrix.partial are similar the function
#' \code{\link[vegan]{mantel}} and \code{\link[vegan]{mantel.partial}}, although the significance
#' of the statistics is evaluated differently from Mantel. The functions pro.matrix
#' and pro.matrix.partial use symmetric Procrustes as a measure of concordance between
#' data sets. The function cor.mantel is similar to the function \code{\link[vegan]{mantel}}, but
#' allows the use of a set of predefined permutation. For more details, see
#' \code{\link{syncsa}}.
#'
#' @details The null model is based on permutations in the matrix mx2, typically the
#' matrices B, U and Q, except in the function cor.mantel when the permutations
#' are done in one of distance matrix.
#'
#' Null model described by Pillar et al. (2009) and Pillar & Duarte (2010). For
#' more details on the matrices and the null model, see \code{\link{syncsa}}.
#'
#' @encoding UTF-8
#' @importFrom vegan vegdist
#' @importFrom parallel makeCluster parRapply stopCluster
#' @importFrom stats as.dist cor
#' @aliases cor.matrix cor.matrix.partial cor.matrix.rao cor.matrix.bf pro.matrix pro.matrix.partial pro.matrix.rao pro.matrix.bf rv.matrix rv.matrix.partial rv.matrix.rao rv.matrix.bf
#' @param mx1 Matrix that multiplied by mx2 results in the matrix x.
#' @param mx2 Matrix that when multiplied by mx1 results in the matrix x. See
#' `details` below.
#' @param x Matrix that will be correlated with the matrix y.
#' @param my1 Matrix that multiplied by my2 results in the matrix y.
#' @param my2 Matrix that when multiplied by my1 results in the matrix y. See
#' `details` below.
#' @param y Matrix that will be correlated with the matrix x.
#' @param mz1 Matrix that multiplied by mz2 results in the matrix z.
#' @param mz2 Matrix that when multiplied by mz1 results in the matrix z. See
#' `details` below.
#' @param z Matrix whose effect will be removed from the correlation between x
#' and y.
#' @param dist.x Dissimilarity matrices of class dist.
#' @param dist.y Dissimilarity matrices of class dist.
#' @param method Correlation method, as accepted by cor: "pearson", "spearman"
#' or "kendall".
#' @param dist Dissimilarity index, as accepted by vegdist: "manhattan",
#' "euclidean", "canberra", "bray", "kulczynski", "jaccard", "gower",
#' "altGower", "morisita", "horn", "mountford", "raup" , "binomial" or "chao".
#' @param permutations Number of permutations in assessing significance.
#' @param norm Logical argument (TRUE or FALSE) to specify if x is standardized
#' within variables (Default norm = FALSE).
#' @param norm.y Logical argument (TRUE or FALSE) to specify if y is standardized
#' within variables (Default norm = FALSE).
#' @param norm.z Logical argument (TRUE or FALSE) to specify if z is standardized
#' within variables (Default norm = FALSE).
#' @param permute.my2 Logical argument (TRUE or FALSE) to specify if realize
#' parallel permutation in matrix my2.
#' @param permute.mz2 Logical argument (TRUE or FALSE) to specify if realize
#' parallel permutation in matrix mz2.
#' @param strata Argument to specify restricting permutations within species
#' groups (Default strata = NULL).
#' @param na.rm Logical argument (TRUE or FALSE) to specify if pairwise
#' deletion of missing observations when computing dissimilarities (Default
#' na.rm = FALSE).
#' @param seqpermutation A set of predefined permutation, with the same dimensions of
#' permutations (Default seqpermutation = NULL).
#' @param parallel Number of parallel processes.  Tip: use parallel::detectCores() (Default parallel = NULL).
#' @param newClusters Logical argument (TRUE or FALSE) to specify if make new parallel
#' processes or use predefined socket cluster. Only if parallel is different of NULL (Default newClusters = TRUE).
#' @param CL A predefined socket cluster done with parallel package.
#' @param put.together List to specify group of traits. Each group specify receive the
#' same weight that one trait outside any group, in the way each group is considered
#' as unique trait (Default put.together = NULL). This argument must be a list, see
#' examples in \code{\link{syncsa}}.
#' @return \item{Obs}{Correlation between matrices.} \item{p}{Significance
#' level based on permutations.}
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link[vegan]{mantel}},
#' \code{\link[vegan]{procrustes}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#'
#' Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
#' Discriminating trait-convergence and trait-divergence assembly patterns in
#' ecological community gradients. Journal of Vegetation Science, 20, 334:348.
#' @keywords SYNCSA
#' @export
cor.matrix <- function (mx1, mx2, x, my1 = NULL, my2 = NULL, y, permute.my2 = FALSE, method = "pearson",
                        dist = "euclidean", permutations = 999, norm = FALSE, norm.y = FALSE,
                        strata = NULL, na.rm = FALSE, seqpermutation = NULL, parallel = NULL,
                        newClusters = TRUE, CL =  NULL)
{
  if(!is.null(seqpermutation)){
    if(dim(seqpermutation)[1]!=permutations){
      stop("\n The seqpermutation must be the same dimension of permutations\n")
    }
  }
  mx1 <- as.matrix(mx1)
  mx2 <- as.matrix(mx2)
  x <- as.matrix(x)
  if(permute.my2){
    my1 <- as.matrix(my1)
    my2 <- as.matrix(my2)
  }
  y <- as.matrix(y)
  dist.y <- vegan::vegdist(y, method = dist, na.rm = na.rm)
  dist.x <- vegan::vegdist(x, method = dist, na.rm = na.rm)
  correlation <- stats::cor(dist.x, dist.y, method = method)
  N <- nrow(mx2)
  if(is.null(seqpermutation)){
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if(!is.null(CL)){
    parallel <- length(CL)
  }
  ptest <- function(samp, mx1, mx2, my1, my2, dist.y, permute.my2, norm, norm.y, dist, na.rm, method){
    # x.permut <- mx1 %*% mx2[samp, ,drop=FALSE]
    x.permut <- matmult.syncsa(mx1, mx2[samp, , drop = FALSE])
    if (norm) {
      matrix.permut <- apply(x.permut^2, 2, sum)
      x.permut <- sweep(x.permut, 2, sqrt(matrix.permut), "/")
    }
    if(permute.my2){
      # y.permut <- my1 %*% my2[samp, ,drop=FALSE]
      y.permut <- matmult.syncsa(my1, my2[samp, , drop = FALSE])
      if (norm.y) {
        matrix.permut <- apply(y.permut^2, 2, sum)
        y.permut <- sweep(y.permut, 2, sqrt(matrix.permut), "/")
      }
      dist.y.permut <- vegan::vegdist(y.permut, method = dist, na.rm = na.rm)
    }
    dist.x.permut <- vegan::vegdist(x.permut, method = dist, na.rm = na.rm)
    if(!permute.my2){
      cor.x.permut <- stats::cor(dist.x.permut, dist.y, method = method)
    }
    if(permute.my2){
      cor.x.permut <- stats::cor(dist.x.permut, dist.y.permut, method = method)
    }
    return(cor.x.permut)
  }
  if(is.null(parallel)){
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1: permutations) {
      value[i,] <- ptest(samp = seqpermutation[i,], mx1 = mx1, mx2 = mx2, my1 = my1, my2 = my2, dist.y = dist.y, permute.my2 = permute.my2, norm = norm, norm.y = norm.y, dist = dist, na.rm = na.rm, method = method)
    }
  } else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation, ptest, mx1 = mx1, mx2 = mx2, my1 = my1, my2 = my2, dist.y = dist.y, permute.my2 = permute.my2, norm = norm, norm.y = norm.y, dist = dist, na.rm = na.rm, method = method))
    if (newClusters){
      parallel::stopCluster(CL)
    }
  }
  signific <- (sum(abs(value) >= abs(correlation)) + 1)/(permutations + 1)
  res <- list(Obs = correlation, p = signific)
  return(res)
}

#' @title Rao's quadratic entropy
#'
#' @description Calculates Rao's quadratic entropy, functional and phylogenetic redundancy.
#'
#' @details Rao's quadratic entropy is a measure of diversity of ecological communities
#' defined by Rao (1982) and is based on the proportion of the abundance of
#' species present in a community and some measure of dissimilarity among them.
#' The dissimilarity range from 0 to 1 and is based on a set of specified
#' functional traits or in the phylogenetic dissimilarity.
#'
#' For the trait data, the function calculates the square root of the
#' one-complement of Gower`s similarity index, in order to have a dissimilarity
#' matrix with Euclidean metric properties. Gower`s index ranges from 0 to 1
#' and can handle traits measured indifferent scales. When the species are
#' completely different in terms of their traits, Rao quadratic entropy is
#' equivalent to the Gini-Simpson index. Traits data can be numeric, factor or
#' ordered factor. For this be considered traits data must be of data.frame
#' class and containing each variable type determined. For additional details and
#' requirements of function please see \code{\link[FD]{gowdis}}.
#'
#' Functional redundancy is defined purely as the difference between species
#' diversity and Rao`s quadratic entropy based on their functional
#' dissimilarity (de Bello et al. 2007). The same definition is used for
#' phylogenetic redundancy.
#'
#' By default, the community data is standardization to row totals will be 1.
#' The options are: "none" no transformation is applied; "standardized" the
#' community data is standardized to row totals will be 1; and "weights" community
#' data is first "standardized" and when, individual species weights are multiplied
#' to each species entries. The argument transformation also allow an additional
#' method to weights individual species, called "max.weights". In this method
#' community data is standardization as default, however dissimilarity matrix is
#' weighted indeed. The argument spp.weights specify the weights to target species
#' and the dissimilarity weights are 1 if at least one species in the pair belongs
#' to the target community, otherwise weights to pair are 0.
#'
#' Package \strong{SYNCSA} requires that the species and community sequence in
#' the data.frame or matrix must be the same for all dataframe/matrices.
#' The function \code{\link{organize.syncsa}} organizes the data for the functions
#' of the package, placing the matrices of community, traits, phylogenetic distance
#' in the same order. The function use of function organize.syncsa is not requered
#' for run the functions, but is recommended. In this way the arguments comm, traits,
#' phylodist, as well as the argument put.together and spp.weights, can be specified them as normal
#' arguments or by passing them with the object returned by the function
#' \code{\link{organize.syncsa}} using, in this case only the argument comm.
#' Using the object returned by organize.syncsa, the comm argument is used as an alternative
#' way of entering to set all data.frames/matrices, and therefore the other arguments
#' (traits, phylodist, put.together and spp.weights) must be null.
#'
#'
#' @encoding UTF-8
#' @importFrom FD gowdis
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' Alternatively comm can be an object of class metacommunity.data, an alternative
#' way to set all data.frames/matrices. When you use the class metacommunity.data the arguments
#' traits, phylodist and put.together must be null. See details.
#' @param traits Data frame or matrix data of species described by traits, with traits as
#' columns and species as rows (Default traits = NULL).
#' @param phylodist Matrix containing phylogenetic distance between species
#' (Default phylodist = NULL).
#' @param checkdata Logical argument (TRUE or FALSE) to check if species
#' sequence in the community data follows the same order as the one in the
#' trait and in the phylodist matrices (Default checkdata = TRUE).
#' @param ord Method to be used for ordinal variables, see \code{\link[FD]{gowdis}}
#' (Default ord = "metric").
#' @param put.together List to specify group of traits. Each group specify receive the
#' same weight that one trait outside any group, in the way each group is considered
#' as unique trait (Default put.together = NULL). This argument must be a list, see
#' examples in \code{\link{syncsa}}.
#' @param standardize Logical argument (TRUE or FALSE) to specify if standardize phylogenetic
#' distance to range into range 0 to 1. (Default standardize = TRUE).
#' @param transformation Method to transformation, "none", "standardized", "weights"
#' or "max.weights" (Default transformation = "standardized").
#' @param spp.weights Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).
#' @param ... Parameters for \code{\link[FD]{gowdis}} function.
#' @return \item{Simpson}{Gini-Simpson index within each community (equivalent
#' to Rao quadratic entropy with null, crisp, similarities).}
#' \item{FunRao}{Rao quadratic entropy within each community, considering trait distance.}
#' \item{FunRedundancy}{Functional redundancy in each community.}
#' \item{PhyRao}{Rao quadratic entropy within each community, considering phylogenetic distance.}
#' \item{PhyRedundancy}{Phylogenetic redundancy in each community.}
#' @note \strong{IMPORTANT}: The sequence species show up in community data
#' matrix MUST be the same as they show up in traits and phylodist matrices as well as
#' in the species weights vector. See details and \code{\link{organize.syncsa}}.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{organize.syncsa}}, \code{\link[FD]{gowdis}},
#' \code{\link{syncsa}}
#' @references de Bello, F.; Leps, J.; Lavorel, S. & Moretti, M. (2007).
#' Importance of species abundance for assessment of trait composition: an
#' example based on pollinator communities. Community Ecology, 8, 163:170.
#'
#' Pillar, V.D.; Blanco, C.C.; Muler, S.C.; Sosinski, E.E.; Joner, F. & Duarte,
#' L.d.S. (2013). Functional redundancy and stability in plant communities.
#' Journal of Vegetation Science, 24, 963:974.
#'
#' Rao, C.R. (1982). Diversity and dissimilarity coefficients: a unified
#' approach. Theoretical Population Biology, 21, 24:43.
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' rao.diversity(ADRS$community)
#' rao.diversity(ADRS$community, traits = ADRS$traits)
#' @export
rao.diversity <- function(comm, traits = NULL, phylodist = NULL, checkdata = TRUE, ord = "metric",
                          put.together = NULL, standardize = TRUE, transformation = "standardized",
                          spp.weights = NULL, ...)
{
  diver.internal <- function(community, distance){
    if(any(is.na(distance))){
      distance.na <- ifelse(is.na(distance), 0, 1)
      inter.na <- community%*%distance.na
      adjustment <- rowSums(sweep(community, 1, inter.na, "*", check.margin = FALSE))
      distance[is.na(distance)] <- 0
      inter <- community%*%distance
      res <- rowSums(sweep(community, 1, inter, "*", check.margin = FALSE))
      res <- ifelse(adjustment>0, res/adjustment, res)
    } else{
      inter <- community%*%distance
      res <- rowSums(sweep(community, 1, inter, "*", check.margin = FALSE))
    }
    return(res)
  }
  res <- list(call = match.call())
  if (inherits(comm, "metacommunity.data")) {
    if (!is.null(traits) | !is.null(phylodist) | !is.null(put.together) | !is.null(spp.weights)) {
      stop("\n When you use an object of class metacommunity.data the arguments traits, phylodist, spp.weights and put.together must be null. \n")
    }
    traits <- comm$traits
    phylodist <- comm$phylodist
    put.together <- comm$put.together
    spp.weights <- comm$spp.weights
    comm <- comm$community
  }
  list.warning <- list()
  if(checkdata){
    organize.temp <- organize.syncsa(comm, traits = traits, phylodist = phylodist, spp.weights = spp.weights, check.comm = TRUE)
    if(!is.null(organize.temp$stop)){
      organize.temp$call <- match.call()
      return(organize.temp)
    }
    list.warning <- organize.temp$list.warning
    comm <- organize.temp$community
    traits <- organize.temp$traits
    phylodist <- organize.temp$phylodist
    spp.weights <- organize.temp$spp.weights
  }
  if(length(list.warning)>0){
    res$list.warning <- list.warning
  }
  if(any(is.na(comm))){
    stop("\n community data with NA\n")
  }
  TRANS <- c("none", "standardized", "weights", "beals", "max.weights")
  trans <- pmatch(transformation, TRANS)
  if (length(trans) > 1) {
    stop("\n Only one argument is accepted in transformation \n")
  }
  if (is.na(trans) | trans == 4) {
    stop("\n Invalid transformation \n")
  }
  if(trans == 5){
    is.bin.weights <- all(spp.weights %in% c(0, 1))
    if(!is.bin.weights | is.null(spp.weights)){
      stop("\n spp.weights must be 0 or 1\n")
    }
    comm <- matrix.w.transformation(comm, transformation = "standardized", notification = FALSE)
    VecMax <- Vectorize(function(x, y) max(x,y))
    dist.weights <- outer(spp.weights, spp.weights, FUN = VecMax)
  } else{
    comm <- matrix.w.transformation(comm, transformation = transformation, spp.weights = spp.weights, notification = FALSE)
  }
  S <- ncol(comm)
  dist.1 <- 1 - diag(x = rep(1, S))
  if (!is.null(traits)) {
    traits <- as.data.frame(traits)
    m <- ncol(traits)
    weights <- rep(1, m)
    make.names <- is.null(colnames(traits))
    colnames(traits) <- colnames(traits, do.NULL = FALSE, prefix = "T")
    names(weights) <- colnames(traits)
    if(!is.null(put.together)){
      if(!inherits(put.together, "list")){
        stop("\n put.together must be a object of class list\n")
      }
      if(make.names){
        for(k in 1:length(put.together)){
          put.together[[k]] <- paste("T", put.together[[k]], sep = "")
        }
      }
      if(max(table(unlist(put.together)))>1){
        stop("\n The same trait appears more than once in put.together\n")
      }
      if(length(setdiff(unlist(put.together), colnames(traits)))>0){
        stop("\n Check traits names in put.together\n")
      }
      for(k in 1:length(put.together)){
        weights[put.together[[k]]] <- 1/length(put.together[[k]])
      }
    }
    dist.functional <- sqrt(as.matrix(FD::gowdis(x=traits, asym.bin = NULL, ord = ord, w = weights, ...)))
    if (checkdata) {
      if(any(is.na(dist.functional))){
        # stop("\n traits with too much NA \n")
        warning("Warning: NA in distance between species", call. = FALSE)
      }
    }
  }
  if (!is.null(phylodist)) {
    dist.phylogenetic <- as.matrix(phylodist)
    if (checkdata) {
      if(any(is.na(dist.phylogenetic))){
        # stop("\n phylodist with NA \n")
        warning("Warning: NA in phylodist", call. = FALSE)
      }
    }
    if(standardize){
      dist.phylogenetic <- dist.phylogenetic/max(dist.phylogenetic, na.rm = TRUE)
    }
  }
  if(trans == 5){
    dist.1 <- dist.1*dist.weights
  }
  SD <- diver.internal(comm, dist.1)
  res$Simpson <- SD
  if (!is.null(traits)){
    if(trans == 5){
      dist.functional <- dist.functional*dist.weights
    }
    FD <- diver.internal(comm, dist.functional)
    res$FunRao <- FD
    res$FunRedundancy <- SD-FD
  }
  if (!is.null(phylodist)){
    if(trans == 5){
      dist.phylogenetic <- dist.phylogenetic*dist.weights
    }
    PD <- diver.internal(comm, dist.phylogenetic)
    res$PhyRao <- PD
    res$PhyRedundancy <- SD-PD
  }
  return(res)
}

#' @rdname cor.matrix
#' @encoding UTF-8
#' @export
cor.matrix.rao<-function (mx1, mx2, x, y, method = "pearson", dist = "euclidean", put.together = NULL,
                      permutations = 999, strata = NULL, na.rm = FALSE,
                      seqpermutation = NULL, parallel = NULL, newClusters = TRUE, CL = NULL)
{
  if (!is.null(seqpermutation)) {
    if (dim(seqpermutation)[1] != permutations) {
      stop("\n The seqpermutation must be the same dimension of permutations\n")
    }
  }
  mx1 <- as.matrix(mx1)
  mx2 <- as.matrix(mx2)
  x <- as.matrix(x)
  y <- as.matrix(y)
  dist.y <- vegan::vegdist(y, method = dist, na.rm = na.rm)
  dist.x <- vegan::vegdist(x, method = dist, na.rm = na.rm)
  correlation <- stats::cor(dist.x, dist.y, method = method)
  N <- nrow(mx2)
  if (is.null(seqpermutation)) {
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if (!is.null(CL)) {
    parallel <- length(CL)
  }
  ptest <- function(samp, mx1, mx2, dist.y, dist, na.rm, method, put.together) {
    x.permut <- cbind(rao.diversity(mx1, traits = mx2[samp, , drop = FALSE], checkdata = FALSE, put.together = put.together)$FunRao)
    dist.x.permut <- vegan::vegdist(x.permut, method = dist, na.rm = na.rm)
    cor.x.permut <- stats::cor(dist.x.permut, dist.y, method = method)
    return(cor.x.permut)
  }
  if (is.null(parallel)) {
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1:permutations) {
      value[i, ] <- ptest(samp = seqpermutation[i, ], mx1 = mx1,
                          mx2 = mx2, dist.y = dist.y, dist = dist,
                          na.rm = na.rm, method = method, put.together = put.together)
    }
  }
  else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation,
                                       ptest, mx1 = mx1, mx2 = mx2, dist.y = dist.y,
                                       dist = dist, na.rm = na.rm, method = method, put.together = put.together))
    if (newClusters) {
      parallel::stopCluster(CL)
    }
  }
  signific <- (sum(abs(value) >= abs(correlation)) + 1)/(permutations + 1)
  res <- list(Obs = correlation, p = signific)
  return(res)
}

#' @rdname cor.matrix
#' @encoding UTF-8
#' @export
cor.matrix.partial <- function (mx1, mx2, x, my1 = NULL, my2 = NULL, y, mz1 = NULL, mz2 = NULL, z,
                                method = "pearson", dist = "euclidean", permute.my2 = FALSE,
                                permute.mz2 = FALSE, permutations = 999, norm = FALSE, norm.y = FALSE,
                                norm.z = FALSE, strata = NULL, na.rm = FALSE, seqpermutation = NULL,
                                parallel = NULL, newClusters = TRUE, CL =  NULL)
{
  if(!is.null(seqpermutation)){
    if(dim(seqpermutation)[1]!=permutations){
      stop("\n seqpermutation must be the same dimension of permutations.\n")
    }
  }
  mx1 <- as.matrix(mx1)
  mx2 <- as.matrix(mx2)
  x <- as.matrix(x)
  if(permute.my2){
    my1 <- as.matrix(my1)
    my2 <- as.matrix(my2)
  }
  y <- as.matrix(y)
  if(permute.mz2){
    mz1 <- as.matrix(mz1)
    mz2 <- as.matrix(mz2)
  }
  z <- as.matrix(z)
  dist.x <- vegan::vegdist(x, method = dist, na.rm = na.rm)
  dist.y <- vegan::vegdist(y, method = dist, na.rm = na.rm)
  dist.z <- vegan::vegdist(z, method = dist, na.rm = na.rm)
  rxy <- stats::cor(dist.x, dist.y, method = method)
  rxz <- stats::cor(dist.x, dist.z, method = method)
  ryz <- stats::cor(dist.y, dist.z, method = method)
  statistic <- part.cor(rxy, rxz, ryz)
  N <- nrow(mx2)
  if(is.null(seqpermutation)){
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if(!is.null(CL)){
    parallel <- length(CL)
  }
  ptest <- function(samp, mx1, mx2, my1, my2, dist.y, mz1, mz2, dist.z, permute.my2, permute.mz2, norm, norm.y, norm.z, dist, na.rm, method){
    # x.permut <- mx1 %*% mx2[samp, , drop = FALSE]
    x.permut <- matmult.syncsa(mx1, mx2[samp, , drop = FALSE])
    if (norm) {
      matrix.permut <- apply(x.permut^2, 2, sum)
      x.permut <- sweep(x.permut, 2, sqrt(matrix.permut), "/")
    }
    dist.x.permut <- vegan::vegdist(x.permut, method = dist, na.rm = na.rm)
    if(permute.my2){
      # y.permut <- my1 %*% my2[samp, , drop = FALSE]
      y.permut <- matmult.syncsa(my1, my2[samp, , drop = FALSE])
      if (norm.y) {
        matrix.permut <- apply(y.permut^2, 2, sum)
        y.permut <- sweep(y.permut, 2, sqrt(matrix.permut), "/")
      }
      dist.y.permut <- vegan::vegdist(y.permut, method = dist, na.rm = na.rm)
    }
    if(permute.mz2){
      # z.permut <- mz1 %*% mz2[samp, , drop = FALSE]
      z.permut <- matmult.syncsa(mz1, mz2[samp, , drop = FALSE])
      if (norm.z) {
        matrix.permut <- apply(z.permut^2, 2, sum)
        z.permut <- sweep(z.permut, 2, sqrt(matrix.permut), "/")
      }
      dist.z.permut <- vegan::vegdist(z.permut, method = dist, na.rm = na.rm)
    }
    if(!permute.my2 & !permute.mz2){
      rxy.temp <- stats::cor(dist.x.permut, dist.y, method = method)
      rxz.temp <- stats::cor(dist.x.permut, dist.z, method = method)
      ryz.temp <- ryz
    }
    if(permute.my2 & !permute.mz2){
      rxy.temp <- stats::cor(dist.x.permut, dist.y.permut, method = method)
      rxz.temp <- stats::cor(dist.x.permut, dist.z, method = method)
      ryz.temp <- stats::cor(dist.y.permut, dist.z, method = method)
    }
    if(!permute.my2 & permute.mz2){
      rxy.temp <- stats::cor(dist.x.permut, dist.y, method = method)
      rxz.temp <- stats::cor(dist.x.permut, dist.z.permut, method = method)
      ryz.temp <- stats::cor(dist.y, dist.z.permut, method = method)
    }
    if(permute.my2 & permute.mz2){
      rxy.temp <- stats::cor(dist.x.permut, dist.y.permut, method = method)
      rxz.temp <- stats::cor(dist.x.permut, dist.z.permut, method = method)
      ryz.temp <- stats::cor(dist.y.permut, dist.z.permut, method = method)
    }
    res<- SYNCSA::part.cor(rxy.temp, rxz.temp, ryz.temp)
    return(res)
  }
  if(is.null(parallel)){
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1: permutations) {
      value[i,] <- ptest(samp = seqpermutation[i,], mx1 = mx1, mx2 = mx2, my1 = my1, my2 = my2, dist.y = dist.y, mz1 = mz1, mz2 = mz2, dist.z = dist.z, permute.my2 = permute.my2, permute.mz2 = permute.mz2, norm = norm, norm.y = norm.y, norm.z = norm.z, dist = dist, na.rm = na.rm, method = method)
    }
  } else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation, ptest, mx1 = mx1, mx2 = mx2, my1 = my1, my2 = my2, dist.y = dist.y, mz1 = mz1, mz2 = mz2, dist.z = dist.z, permute.my2 = permute.my2, permute.mz2 = permute.mz2, norm = norm, norm.y = norm.y, norm.z = norm.z, dist = dist, na.rm = na.rm, method = method))
    if (newClusters){
      parallel::stopCluster(CL)
    }
  }
  signif <- (sum(abs(value) >= abs(statistic)) + 1)/(permutations + 1)
  res <- list(Obs = statistic, p = signif)
  return(res)
}

#' @rdname cor.matrix
#' @encoding UTF-8
#' @export
pro.matrix.bf <- function (x, y, permutations = 999, strata = NULL, na.rm = FALSE, seqpermutation = NULL,
                            parallel = NULL, newClusters = TRUE, CL =  NULL)
{
  if(!is.null(seqpermutation)){
    if(dim(seqpermutation)[1] != permutations){
      stop("\nThe seqpermutation must be the same dimension of permutations\n")
    }
  }
  correlation <- procrustes.syncsa(x, y)
  N <- nrow(x)
  if(is.null(seqpermutation)){
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if(!is.null(CL)){
    parallel <- length(CL)
  }
  ptest <- function(samp, x, y){
    res <- SYNCSA::procrustes.syncsa(x, y[samp, ,drop=FALSE])
    return(res)
  }
  if(is.null(parallel)){
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1: permutations) {
      value[i,] <- ptest(samp = seqpermutation[i,], x = x, y = y)
    }
  } else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation, ptest, x = x,  y = y))
    if (newClusters){
      parallel::stopCluster(CL)
    }
  }
  signific <- (sum(abs(value) >= abs(correlation)) + 1)/(permutations + 1)
  res <- list(Obs = correlation, p = signific)
  return(res)
}

#' @rdname cor.matrix
#' @encoding UTF-8
#' @export
pro.matrix<-function (mx1, mx2, x, my1 = NULL, my2 = NULL, y, permute.my2 = FALSE, permutations = 999,
                      norm = FALSE, norm.y = FALSE, strata = NULL, seqpermutation = NULL,
                      parallel = NULL, newClusters = TRUE, CL =  NULL)
{
  if(!is.null(seqpermutation)){
    if(dim(seqpermutation)[1] != permutations){
      stop("\n seqpermutation must be the dimension of permutations\n")
    }
  }
  x <- cbind(x)
  if(permute.my2){
    my1 <- as.matrix(my1)
    my2 <- as.matrix(my2)
  }
  y <- cbind(y)
  correlation <- procrustes.syncsa(x,y)
  N <- nrow(mx2)
  if(is.null(seqpermutation)){
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if(!is.null(CL)){
    parallel <- length(CL)
  }
  ptest <- function(samp, mx1, mx2, my1, my2, y, permute.my2, norm, norm.y){
    # x.permut <- mx1 %*% mx2[samp, , drop = FALSE]
    x.permut <- matmult.syncsa(mx1, mx2[samp, , drop = FALSE])
    if (norm) {
      matrix.permut <- apply(x.permut^2, 2, sum)
      x.permut <- sweep(x.permut, 2, sqrt(matrix.permut), "/")
    }
    if(permute.my2){
      # y.permut <- my1 %*% my2[samp, , drop = FALSE]
      y.permut <- matmult.syncsa(my1, my2[samp, , drop = FALSE])
      if (norm.y) {
        matrix.permut <- apply(y.permut^2, 2, sum)
        y.permut <- sweep(y.permut, 2, sqrt(matrix.permut), "/")
      }
    }
    if(!permute.my2){
      res <- SYNCSA::procrustes.syncsa(x.permut, y)
    }
    if(permute.my2){
      res <- SYNCSA::procrustes.syncsa(x.permut, y.permut)
    }
    return(res)
  }
  if(is.null(parallel)){
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1: permutations) {
      value[i,] <- ptest(samp = seqpermutation[i,], mx1 = mx1, mx2 = mx2, my1 = my1, my2 = my2, y = y, permute.my2 = permute.my2, norm = norm, norm.y = norm.y)
    }
  } else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation, ptest, mx1 = mx1, mx2 = mx2, my1 = my1, my2 = my2, y = y, permute.my2 = permute.my2, norm = norm, norm.y = norm.y))
    if (newClusters){
      parallel::stopCluster(CL)
    }
  }
  sig <- (sum(value >= correlation) + 1)/(permutations + 1)
  return(list(Obs = correlation, p = sig))
}

#' @rdname syncsa
#' @include syncsa.R
#' @encoding UTF-8
#' @export
print.syncsa <- function(x, ...)
{
  cat("Call:\n")
  cat(deparse(x$call), "\n\n")
  if(length(x$list.warning)>0){
    cat("List of warning:\n")
    namestemp <- CollectNames(x$list.warning, prefix = "$list.warning$")
    cat(unlist(namestemp, use.names = FALSE), sep = "\n")
  }
  cat("\nList of matrices:\n")
  namestemp <- CollectNames(x$matrices, prefix = "$matrices$")
  cat(unlist(namestemp[!sapply(x$matrices, is.null)], use.names = FALSE), sep = "\n")
  if(length(x$notes)>0){
    cat("\n$notes:\n")
    print(x$notes)
    cat("\n$statistics:\n")
    print(as.matrix(x$statistics), ...)
  }
  if(length(x$matrices.null)>0){
    cat("\nList of null matrices:\n")
    namestemp <- paste("$matrices.null$", names(x$matrices.null), sep = "")
    cat(unlist(namestemp, use.names = FALSE), sep = "\n")
  }
  invisible(x)
}

#' @rdname cor.matrix
#' @encoding UTF-8
#' @export
rv.matrix.rao<-function (mx1, mx2, x, y, put.together = NULL, permutations = 999, strata = NULL,
          seqpermutation = NULL, parallel = NULL, newClusters = TRUE,
          CL = NULL)
{
  if (!is.null(seqpermutation)) {
    if (dim(seqpermutation)[1] != permutations) {
      stop("\n seqpermutation must be the dimension of permutations\n")
    }
  }
  x <- cbind(x)
  y <- cbind(y)
  correlation <- coinertia.syncsa(x, y)
  N <- nrow(mx2)
  if (is.null(seqpermutation)) {
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if (!is.null(CL)) {
    parallel <- length(CL)
  }
  ptest <- function(samp, mx1, mx2, y, put.together) {
    x.permut <- cbind(rao.diversity(mx1, traits = mx2[samp, , drop = FALSE], checkdata = FALSE, put.together = put.together)$FunRao)
    res <- SYNCSA::coinertia.syncsa(x.permut, y)
    return(res)
  }
  if (is.null(parallel)) {
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1:permutations) {
      value[i, ] <- ptest(samp = seqpermutation[i, ], mx1 = mx1, mx2 = mx2, y = y, put.together = put.together)
    }
  }
  else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation, ptest, mx1 = mx1, mx2 = mx2, y = y, put.together = put.together))
    if (newClusters) {
      parallel::stopCluster(CL)
    }
  }
  sig <- (sum(value >= correlation) + 1)/(permutations + 1)
  return(list(Obs = correlation, p = sig))
}

.onLoad <- function(libname, pkgname){
	op <- options()
	op.SYNCSA <- list(SYNCSA.speed = TRUE)
	toset <- !(names(op.SYNCSA) %in% names(op))
	if(any(toset)){
		options(op.SYNCSA[toset])	
	} 
	invisible()
}

#' @rdname coinertia.syncsa
#' @encoding UTF-8
#' @export
coinertia.partial.syncsa <- function(x, y, z, scale = FALSE){
  rxy.temp <- SYNCSA::coinertia.syncsa(x, y, scale = scale)
  rxz.temp <- SYNCSA::coinertia.syncsa(x, z, scale = scale)
  ryz.temp <- SYNCSA::coinertia.syncsa(y, z, scale = scale)
  RV <- SYNCSA::part.cor(rxy.temp, rxz.temp, ryz.temp)
  return(RV)
}

#' @title Check the type of variables
#'
#' @description Function to check the type of variables in a data.frame or matrix. This function was extracted
#' and slightly modified of the function \code{\link[FD]{gowdis}}.
#'
#' @encoding UTF-8
#' @param data A data.frame or matrix.
#' @return A vector with the variable types, where 'c' is continuous/numeric, 'o' is
#' ordinal, 'b' is binary, 'n' is nominal and 'f' is factor.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso  \code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{var.dummy}}
#' @keywords Auxiliary
#' @export
var.type <- function(data)
{
  if(!inherits(data, c("data.frame", "matrix"))){
    stop("data must be a matrix or a data.frame")
  }
  colnames(data) <- colnames(data, do.NULL = FALSE, prefix = "var")
  is.bin <- function(k) all(k[!is.na(k)] %in% c(0, 1))
  nc <- ncol(data)
  if (is.data.frame(data)) {
    type <- sapply(data, data.class)
    type2 <- type
    bin.var <- rep(NA, nc)
    for (i in 1:nc) {
      bin.var[i] <- is.bin(data[, i])
    }
    type[type %in% c("numeric", "integer")] <- "c"
    type[type == "ordered"] <- "o"
    type[type == "character"] <- "n"
    type[type == "factor"] <- "f"
    type[bin.var] <- "b"
    type[type2 == "character"] <- "n"
    type[type2 == "factor"] <- "f"
    names(type) <- NULL
  }
  else {
    if(any(sapply(data, data.class) == "character")){
      stop("\n If data is a matrix class it must be entirely numeric \n")
    }
    type <- rep("c", nc)
  }
  return(type)
}

#' @title Searching for optimal traits
#'
#' @description Maximize trait-convergence assembly patterns (TCAP = roTE), trait-divergence
#' assembly patterns (TDAP = roXE.T), maximize both trait-divergence assembly
#' patterns and trait-convergence assembly patterns (TCAP.TDAP = roXE) or
#' alpha divergence (roRE) For more details, see \code{\link{syncsa}}.
#'
#' @details Package \strong{SYNCSA} requires that the species and community sequence in
#' the data.frame or matrix must be the same for all dataframe/matrices.
#' The function \code{\link{organize.syncsa}} organizes the data for the functions
#' of the package, placing the matrices of community, traits and
#' environmental varibles in the same order. The function
#' use of function organize.syncsa is not requered for run the functions, but
#' is recommended. In this way the arguments comm, traits, envir, as well as the argument
#' put.together, can be specified them as normal arguments or by passing them
#' with the object returned by the function \code{\link{organize.syncsa}} using, in this
#' case only the argument comm. Using the object returned by organize.syncsa, the comm argument
#' is used as an alternative way of entering to set all data.frames/matrices, and therefore
#' the other arguments (traits, envir, and put.together) must be null.
#'
#'
#' @encoding UTF-8
#' @importFrom vegan vegdist
#' @importFrom stats cor
#' @importFrom utils combn
#' @aliases optimal print.optimal
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' Alternatively comm can be an object of class metacommunity.data, an alternative
#' way to set all data.frames/matrices. When you use the class metacommunity.data the arguments
#' traits, envir and put.together must be null. See details.
#' @param traits Matrix data of species described by traits, with traits as
#' columns and species as rows (Default traits = NULL).
#' @param envir Environmental variables for each community, with variables as
#' columns and sampling units as rows (Default envir = NULL).
#' @param checkdata Logical argument (TRUE or FALSE) to check if species
#' sequence in the community data follows the same order as the one in the
#' trait and if sampling units in the community data follows the same order as the one in the
#' environmental matrices (Default checkdata = TRUE).
#' @param subset.min Minimum of traits in each subset (Default subset.min = 1).
#' @param subset.max Maximum of traits in each subset (Default subset.max = ncol(traits)).
#' @param pattern Patterns for maximize correlation, "tcap", "tdap",
#' "tcap.tdap" or "rao" (Default pattern = NULL).
#' @param ro.method Method to obtain the correlation, "mantel" or "procrustes"
#' (Default ro.method = "mantel").
#' @param method Correlation method, as accepted by cor: "pearson", "spearman"
#' or "kendall".
#' @param dist Dissimilarity index, as accepted by vegdist: "manhattan",
#' "euclidean", "canberra", "bray", "kulczynski", "jaccard", "gower",
#' "altGower", "morisita", "horn", "mountford", "raup" , "binomial" or "chao".
#' @param scale Logical argument (TRUE or FALSE) to specify if the traits are
#' measured on different scales (Default Scale = TRUE). When scale = TRUE traits
#' are measured on different scales and the matrix T is subjected to
#' standardization within each trait. When scale = FALSE if traits are measured on
#' the same scale and the matrix T is not subjected to standardization.
#' Furthermore, if scale = TRUE the matrix of traits is subjected to
#' standardization within each trait, and Gower Index is used to calculate the
#' degree of belonging to the species, and if scale = FALSE the matrix of
#' traits is not subjected to standardization, and Euclidean distance is
#' calculated to determine the degree of belonging to the species.
#' @param scale.envir Logical argument (TRUE or FALSE) to specify if the
#' environmental variables are measured on different scales (Default scale =
#' TRUE). If the enviromental variables are measured on different scales, the
#' matrix is subjected to centralization and standardization within each
#' variable.
#' @param ranks Logical argument (TRUE or FALSE) to specify if ordinal variables are
#' convert to ranks (Default ranks = TRUE).
#' @param asym.bin Vector listing the asymmetric binary traits, see \code{\link[FD]{gowdis}} (Default asym.bin = NULL).
#' @param ord Method to be used for ordinal traits, see \code{\link[FD]{gowdis}} (Default ord = "metric").
#' @param na.rm Logical argument (TRUE or FALSE) to specify if pairwise
#' deletion of missing observations when computing dissimilarities (Default
#' na.rm = FALSE).
#' @param notification Logical argument (TRUE or FALSE) to specify if
#' notifications of missing observations are shown (Default notification =
#' TRUE).
#' @param put.together List to specify group traits that are added or removed
#' together (Default put.together = NULL). This argument must be a list, see
#' examples.
#' @param progressbar Logical argument (TRUE or FALSE) to specify if display a
#' progress bar on the R console (Default progressbar = FALSE).
#' @param x An object of class optimal.
#' @param ... Other parameters for the respective functions.
#' @return \item{Subset}{Subset of traits that maximizes the correlation.}
#' \item{ro}{Correlation for the subset of traits.}
#' @note \strong{IMPORTANT}: The sequence species show up in community data
#' matrix MUST be the same as they show up in traits matrix. See details and
#' \code{\link{organize.syncsa}}.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{organize.syncsa}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#'
#' Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
#' Discriminating trait-convergence and trait-divergence assembly patterns in
#' ecological community gradients. Journal of Vegetation Science, 20, 334:348.
#' @keywords SYNCSA
#' @examples
#' data(flona)
#' optimal(flona$community, flona$traits, flona$environment, subset.min = 3,
#'    subset.max = 5, pattern = "tcap")
#' optimal(flona$community, flona$traits, flona$environment, subset.min = 3,
#'    subset.max = 5, pattern = "tdap")
#' optimal(flona$community, flona$traits, flona$environment, subset.min = 3,
#'    subset.max = 5, pattern = "tcap.tdap")
#' put.together <- list(c("fol", "sem"), c("tam", "red"))
#' put.together
#' optimal(flona$community, flona$traits, flona$environment, subset.min = 1,
#'    subset.max = 3, pattern = "tcap", put.together = put.together)
#' @export
optimal <- function (comm, traits = NULL, envir = NULL, checkdata = TRUE,
                   subset.min = 1, subset.max = ncol(traits),
                   pattern = NULL, ro.method = "mantel", dist = "euclidean", method = "pearson",
                   scale = TRUE, scale.envir = TRUE, ranks = TRUE,
                   asym.bin = NULL, ord = "metric", put.together = NULL,
                   na.rm = FALSE, notification = TRUE, progressbar = FALSE)
{
  res <- list(call = match.call())
  roMETHOD <- c("mantel", "procrustes", "coinertia")
  romethod <- pmatch(ro.method, roMETHOD)
  if (length(romethod) > 1) {
    stop("\n Only one argument is accepted in ro.method \n")
  }
  if (is.na(romethod)) {
    stop("\n Invalid ro.method \n")
  }
  PATTERNS <- c("tcap", "tdap", "tcap.tdap", "rao")
  pattern <- pmatch(pattern, PATTERNS)
  if (length(pattern) != 1) {
    stop("\n Only one argument is accepted in pattern \n")
  }
  if (is.na(pattern)) {
    stop("\n Invalid pattern \n")
  }
  if (inherits(comm, "metacommunity.data")) {
    if (!is.null(traits) | !is.null(envir) | !is.null(put.together)) {
      stop("\n When you use an object of class metacommunity.data the arguments traits, envir and put.together must be null. \n")
    }
    traits <- comm$traits
    envir <- comm$environmental
    put.together <- comm$put.together
    comm <- comm$community
  }
  list.warning <- list()
  if(checkdata){
    organize.temp <- organize.syncsa(comm, traits = traits, envir = envir, check.comm = TRUE)
    if(!is.null(organize.temp$stop)){
      organize.temp$call <- match.call()
      return(organize.temp)
    }
    list.warning <- organize.temp$list.warning
    comm <- organize.temp$community
    traits <- organize.temp$traits
    envir <- organize.temp$environmental
  }
  if(length(list.warning)>0){
    res$list.warning <- list.warning
  }
  if (is.null(traits)) {
    stop("\n traits is NULL \n")
  }
  if (is.null(envir)) {
    stop("\n envir is NULL \n")
  }
  if (notification & !checkdata) {
    if (any(is.na(comm))) {
      warning("Warning: NA in community data", call. = FALSE)
    }
    if (any(is.na(traits))) {
      warning("Warning: NA in traits matrix", call. = FALSE)
    }
    if (any(is.na(envir))) {
      warning("Warning: NA in environmental data", call. = FALSE)
    }
  }
  if(!checkdata){
    commvartype <- var.type(comm)
    if(any(commvartype == "n")){
      stop("\n comm must contain only numeric, binary or ordinal variables \n")
    }
    if (!is.null(traits)) {
      traitsvartype <- var.type(traits)
      if(any(traitsvartype == "n")){
        stop("\n trait must contain only numeric, binary or ordinal variables \n")
      }
    }
    if (!is.null(envir)) {
      envirvartype <- var.type(envir)
      if(any(envirvartype == "n")){
        stop("\n envir must contain only numeric, binary or ordinal variables \n")
      }
    }
  }
  make.names <- is.null(colnames(traits))
  colnames(traits) <- colnames(traits, do.NULL = FALSE, prefix = "T")
  if(!is.null(asym.bin)){
    names(asym.bin) <- colnames(traits)[asym.bin]
  }
  if (scale.envir) {
    envir <- cent.norm(envir, na.rm = na.rm)
  }
  if (romethod == 1) {
    dist.y <- vegan::vegdist(envir, method = dist, na.rm = na.rm)
  }
  m <- ncol(traits)
  traits.weights <- rep(1, m)
  names(traits.weights) <- colnames(traits)
  p <- 1:subset.max
  names.traits <- colnames(traits)
  if(!is.null(put.together)){
    if(!inherits(put.together, "list")){
      stop("\n put.together must be a object of class list\n")
    }
    if(make.names){
      for(k in 1:length(put.together)){
        put.together[[k]] <- paste("T", put.together[[k]], sep = "")
      }
    }
    if(max(table(unlist(put.together)))>1){
      stop("\n The same trait appears more than once in put.together\n")
    }
    if(length(setdiff(unlist(put.together), colnames(traits)))>0){
      stop("\n Check traits names in put.together\n")
    }
    for(k in 1:length(put.together)){
      names.traits[which(names.traits == put.together[[k]][1])] <- paste(put.together[[k]], collapse = " ")
      names.traits <- setdiff(names.traits, put.together[[k]][-1])
      traits.weights[put.together[[k]]] <- 1/length(put.together[[k]])
    }
    m <- length(names.traits)
    put.together2 <- list()
    for(k in 1:length(put.together)){
      put.together2[[k]] <- paste(put.together[[k]], collapse = " ")
    }
  }
  if (subset.max > m) {
    stop("\n subset must be lower than the number of traits\n")
  }
  bin <- factorial(m)/(factorial(p) * factorial(m - p))
  nT <- sum(bin[subset.min:subset.max])
  comb <- matrix(NA, nrow = sum(bin[subset.min:subset.max]), ncol = 1)
  n <- 0
  for (i in subset.min:subset.max) {
    combinations <- utils::combn(names.traits, i, simplify = TRUE)
    for (j in 1:bin[i]) {
      n <- n + 1
      comb[n, 1] <- paste(combinations[, j], collapse = " ")
    }
  }
  n <- 0
  correlation <- matrix(NA, nrow = sum(bin[subset.min:subset.max]), ncol = 1)
  for (i in subset.min:subset.max) {
    combinations1 <- utils::combn(names.traits, i, simplify = TRUE)
    for (j in 1:bin[i]) {
      if (pattern == 1) {
        n <- n + 1
        choose.traits <- combinations1[, j]
        if(!is.null(put.together)){
          if(sum(match(choose.traits, unlist(put.together2)), na.rm = TRUE)>0){
            choose.traits2 <- intersect(choose.traits, unlist(put.together2))
            choose.traits3 <- c()
            for(k in 1:length(choose.traits2)){
              for(l in 1:length(put.together)){
                if(choose.traits2[k] == put.together2[[l]]){
                  choose.traits3 <- c(choose.traits3, put.together[[l]])
                }
              }
            }
            choose.traits <- c(choose.traits3, setdiff(choose.traits, unlist(put.together2)))
          }
        }
        T <- matrix.t(comm, traits[, choose.traits, drop = FALSE], scale = scale, ranks = ranks, notification = FALSE)
        if (romethod == 1) {
          correlation[n, 1] <- stats::cor(vegan::vegdist(as.matrix(T$matrix.T), method = dist, na.rm = na.rm), dist.y, method = method)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 2) {
          correlation[n, 1] <- procrustes.syncsa(T$matrix.T, envir)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 3) {
          correlation[n, 1] <- coinertia.syncsa(T$matrix.T, envir)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
      }
      if (pattern == 2) {
        n <- n + 1
        choose.traits <- combinations1[, j]
        if(!is.null(put.together)){
          if(sum(match(choose.traits,unlist(put.together2)), na.rm = TRUE)>0){
            choose.traits2 <- intersect(choose.traits, unlist(put.together2))
            choose.traits3 <- c()
            for(k in 1:length(choose.traits2)){
              for(l in 1:length(put.together)){
                if(choose.traits2[k] == put.together2[[l]]){
                  choose.traits3 <- c(choose.traits3, put.together[[l]])
                }
              }
            }
            choose.traits <- c(choose.traits3, setdiff(choose.traits, unlist(put.together2)))
          }
        }
        traits.position <- which(colnames(traits) %in% choose.traits)
        asym.bin.temp <- which(colnames(traits)[traits.position] %in% names(asym.bin[asym.bin %in% traits.position]))
        if(length(asym.bin.temp)<1){
          asym.bin.temp <- NULL
        }
        T <- matrix.t(comm, traits[, choose.traits, drop = FALSE], scale = scale, ranks = ranks, notification = FALSE)
        X <- matrix.x(comm, traits[, choose.traits, drop = FALSE], scale = scale, ranks = ranks, notification = FALSE,
                      asym.bin = asym.bin.temp, ord = ord, w = traits.weights[choose.traits])
        if (romethod == 1) {
          dist.x <- vegan::vegdist(X$matrix.X, method = dist, na.rm = na.rm)
          dist.z <- vegan::vegdist(T$matrix.T, method = dist, na.rm = na.rm)
          rxy <- stats::cor(dist.x, dist.y, method = method)
          rxz <- stats::cor(dist.x, dist.z, method = method)
          ryz <- stats::cor(dist.y, dist.z, method = method)
          correlation[n, 1] <- part.cor(rxy, rxz, ryz)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 2) {
          correlation[n, 1] <- procrustes.partial.syncsa(X$matrix.X, envir, T$matrix.T)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 3) {
          correlation[n, 1] <- coinertia.partial.syncsa(X$matrix.X, envir, T$matrix.T)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
      }
      if (pattern == 3) {
        n <- n + 1
        choose.traits <- combinations1[, j]
        if(!is.null(put.together)){
          if(sum(match(choose.traits, unlist(put.together2)), na.rm = TRUE)>0){
            choose.traits2 <- intersect(choose.traits, unlist(put.together2))
            choose.traits3 <- c()
            for(k in 1:length(choose.traits2)){
              for(l in 1:length(put.together)){
                if(choose.traits2[k] == put.together2[[l]]){
                  choose.traits3 <- c(choose.traits3, put.together[[l]])
                }
              }
            }
            choose.traits <- c(choose.traits3, setdiff(choose.traits, unlist(put.together2)))
          }
        }
        traits.position <- which(colnames(traits) %in% choose.traits)
        asym.bin.temp <- which(colnames(traits)[traits.position] %in% names(asym.bin[asym.bin %in% traits.position]))
        if(length(asym.bin.temp)<1){
          asym.bin.temp <- NULL
        }
        X <- matrix.x(comm, traits[, choose.traits, drop = FALSE], scale = scale, ranks = ranks, notification = FALSE,
                      asym.bin = asym.bin.temp, ord = ord, w = traits.weights[choose.traits])
        if (romethod == 1) {
          correlation[n, 1] <- stats::cor(vegan::vegdist(as.matrix(X$matrix.X), method = dist, na.rm = na.rm), dist.y, method = method)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 2) {
          correlation[n, 1] <- procrustes.syncsa(X$matrix.X, envir)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 3) {
          correlation[n, 1] <- coinertia.syncsa(X$matrix.X, envir)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
      }
      if (pattern == 4) {
        n <- n + 1
        choose.traits <- combinations1[, j]
        if(!is.null(put.together)){
          put.together.temp <- put.together
          if(sum(match(choose.traits,unlist(put.together2)), na.rm = TRUE)>0){
            choose.traits2 <- intersect(choose.traits, unlist(put.together2))
            choose.traits3 <- c()
            for(k in 1:length(choose.traits2)){
              for(l in 1:length(put.together)){
                if(choose.traits2[k] == put.together2[[l]]){
                  choose.traits3 <- c(choose.traits3, put.together[[l]])
                }
              }
            }
            choose.traits <- c(choose.traits3, setdiff(choose.traits, unlist(put.together2)))
          }
          put.together.temp <- put.together.temp[sapply(put.together.temp, function(x) any(x%in% choose.traits))]
          if(length(put.together.temp)==0){
            put.together.temp <- NULL
          }
        } else{
          put.together.temp <- NULL
        }
        T <- matrix.t(comm, traits[, choose.traits, drop = FALSE], scale = scale, ranks = ranks, notification = FALSE)
        RAO <- cbind(rao.diversity(comm, traits = T$matrix.b, checkdata = FALSE, put.together = put.together.temp)$FunRao)
        colnames(RAO) <- "FunRao"
        if (romethod == 1) {
          correlation[n, 1] <- stats::cor(vegan::vegdist(RAO, method = dist, na.rm = na.rm), dist.y, method = method)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 2) {
          correlation[n, 1] <- procrustes.syncsa(RAO, envir)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
        if (romethod == 3) {
          correlation[n, 1] <- coinertia.syncsa(RAO, envir)
          if (progressbar) {
            ProgressBAR(n, nT, style = 3)
          }
        }
      }
    }
  }
  result <- data.frame(Subset = comb, ro = correlation, stringsAsFactors = FALSE)
  if(pattern == 1 | pattern ==3 | pattern == 4){
    result <- result[order(result[, 2], decreasing = TRUE), ]
  } else{
    result <- result[order(abs(result[, 2]), decreasing = TRUE), ]
  }
  res$N_subset <- nT
  res$optimization <- result
  res$traits.weights <- traits.weights
  class(res) <- "optimal"
  return(res)
}

#' @rdname procrustes.syncsa
#' @include procrustes.syncsa.R
#' @encoding UTF-8
#' @export
procrustes.partial.syncsa <- function(x, y, z)
{
  x <- as.matrix(x)
  y <- as.matrix(y)
  z <- as.matrix(z)
  if(!options("SYNCSA.speed")$SYNCSA.speed){
    pro.residuals <- function(Y, X){
      res <- matrix(NA, dim(Y)[1], dim(Y)[2])
      for(i in 1:ncol(Y)){
        mat <- cbind(1, X)
        fast_mod <- RcppArmadillo::fastLmPure(mat, Y[,i])
        coeffs <- fast_mod$coefficients
        res[,i] <- Y[,i] - mat %*% coeffs
      }
      return(res)
    }
  } else {
    pro.residuals <- function(Y, X){
      X <- sweep(X, 2, colMeans(X, na.rm = TRUE), check.margin = FALSE)
      Y <- sweep(Y, 2, colMeans(Y, na.rm = TRUE), check.margin = FALSE)
      Yfit <- X%*%solve(t(X)%*%X)%*%(t(X))%*%Y
      Yres <- Y-Yfit
      return(Yres)
    }
  }
  scoresofz <- stats::prcomp(z, scale = TRUE)$x
  nm <- round((nrow(x)-2)/2)
  if(nm<ncol(scoresofz)){
    scoresofz <- scoresofz[, 1:nm, drop = FALSE]
  }
  x.r <- pro.residuals(x, scoresofz)
  y.r <- pro.residuals(y, scoresofz)
  statistic <- procrustes.syncsa(x.r, y.r)
  return(statistic)
}

#' @title Procrustes and Partial Procrustes correlations.
#'
#' @description Function to obtain the Procrustes correlation between two matrices and
#' partial Procrustes correlation between three matrices.
#'
#' @details The function procrustes.syncsa is a small change in the function
#' \code{\link[vegan]{procrustes}}. The function uses a correlation-like statistic
#' derived from the symmetric Procrustes sum of squares. Partial Procrustes
#' correlations is obtained by Procrustes correlation of residuals between
#' each variable in z being used as linear predictor of all variables in x and y.
#' For more details, see \code{\link{syncsa}}.
#'
#' @encoding UTF-8
#' @importFrom stats lm residuals prcomp
#' @importFrom RcppArmadillo fastLmPure
#' @aliases procrustes.syncsa procrustes.partial.syncsa
#' @param x,y Matrix that will be correlated.
#' @param z Matrix whose effect will be removed from the correlation between x
#' and y.
#' @return Procrustes correlation between matrices.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @references Peres-Neto, P.R. and Jackson, D.A. (2001). How well do
#' multivariate data sets match? The advantages of a Procrustean
#' superimposition approach over the Mantel test. Oecologia 129: 169-178.
#' @seealso \code{\link{syncsa}}, \code{\link{cor.matrix}}
#' @keywords SYNCSA
#' @export
procrustes.syncsa <- function (x, y)
{
  x <- cbind(x)
  y <- cbind(y)
  if (ncol(x) < ncol(y)){
    addcols <- ncol(y) - ncol(x)
    for (i in 1:addcols) x <- cbind(x, 0)
  }
  ctrace <- function(MAT) sum(diag(crossprod(MAT)))
  x <- scale(x, scale = FALSE)
  y <- scale(y, scale = FALSE)
  x <- x/sqrt(ctrace(x))
  y <- y/sqrt(ctrace(y))
  xy <- crossprod(x, y)
  sol <- svd(xy)
  c <- sum(sol$d)/ctrace(y)
  r2 <- ctrace(x) + c * c * ctrace(y) - 2 * c * sum(sol$d)
  ro <- sqrt(1 - r2)
  return(ro)
}

#' @title Permutate a vector
#'
#' @description Function to permutate a vector of size n using the function \code{\link[permute]{shuffleSet}}.
#'
#' @encoding UTF-8
#' @importFrom permute how Within Plots shuffleSet
#' @param n The length of vector.
#' @param strata A vector to specify restricting permutations.
#' @param nset The number of permutations to generate for the set (Default strata = NULL).
#' @return A matrix of permutations, where each row is a separate permutation.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{permut.row.matrix}}
#' @keywords Auxiliary
#' @export
permut.vector <- function(n, strata = NULL, nset = 999)
{
  if(is.null(strata)){
    CTRL <- permute::how(within = permute::Within(type = "free"), plots = permute::Plots(type = "free"))
    samp <- permute::shuffleSet(n = n, nset = nset, control = CTRL, check = FALSE)
  }
  if(!is.null(strata)){
    if(n != length(strata)){
      stop("\nstrata must be the length of n\n")
    }
    CTRL <- permute::how(within = permute::Within(type = "free"), plots = permute::Plots(strata = strata, type = "none"))
    samp <- permute::shuffleSet(n = n, nset = nset, control = CTRL, check = FALSE)
  }
  res <- as.matrix(samp)
  return(res)
}

#' @title Matrix P
#'
#' @description Function to obtain a matrix containing phylogeny-weighted species
#' composition. For more details, see \code{\link{syncsa}}.
#'
#' @encoding UTF-8
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' @param phylodist Matrix containing phylogenetic distance between species.
#' Must be a complete matrix (not a diagonal resemblance matrix).
#' @param transformation Method to community data transformation, "none", "standardized" or "weights" (Default transformation = "standardized").
#' @param spp.weights Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).
#' @param notification Logical argument (TRUE or FALSE) to specify if
#' notifications for missing observations are to be shown (Default notification =
#' TRUE).
#' @return \item{matrix.w}{Standardized community matrix, where rows are
#' communities and columns species. If default transformation, row totals (communities) = 1.}
#' \item{matrix.q}{Standardized matrix containing the degree of belonging of
#' species in relation to each other. Row totals (species) = 1.}
#' \item{matrix.P}{Phylogeny-weighted species composition matrix. If default transformation, row totals (communities) = 1.}
#' @note \strong{IMPORTANT}: Species sequence in the community data
#' matrix MUST be the same as the one in the phylogenetic distance matrix or in the spp.weights vector. See
#' \code{\link{organize.syncsa}}.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{belonging}},
#' \code{\link{matrix.t}}, \code{\link{matrix.x}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' matrix.p(ADRS$community, ADRS$phylo)
#' @export
matrix.p <- function (comm, phylodist, transformation = "standardized", spp.weights = NULL, notification = TRUE)
{
  phylodist <- as.matrix(phylodist)
  TRANS <- c("none", "standardized", "weights", "beals")
  trans <- pmatch(transformation, TRANS)
  if (length(trans) > 1) {
    stop("\n Only one argument is accepted in transformation \n")
  }
  if (is.na(trans) | trans == 4) {
    stop("\n Invalid transformation \n")
  }
  matrix.w <- matrix.w.transformation(comm, transformation = transformation,
                                      spp.weights = spp.weights, notification = notification)
  if(notification){
    if(any(is.na(phylodist))){
      warning("Warning: NA in phylodist data", call.=FALSE)
    }
  }
  similar.phy <- 1 - (phylodist/max(phylodist))
  matrix.phy <- 1/rowSums(similar.phy)
  matrix.q <- sweep(similar.phy, 1, matrix.phy, "*")
  matrix.P <- matrix.w %*% matrix.q
  # matrix.P <- matmult.syncsa(matrix.w, matrix.q)
  return(list(matrix.w = matrix.w, matrix.q = matrix.q, matrix.P = matrix.P))
}

#' @title Permutate rows in a matrix
#'
#' @description Function to permutate rows in a matrix.
#'
#' @encoding UTF-8
#' @param data A matrix.
#' @param strata Strata vector to specify restricting permutations in rows, must be the same length of number of rows in data matrix (Default strata = NULL).
#' @param seqpermutation A set of predefined permutation vector (Default seqpermutation = NULL).
#' @return \item{permut.matrix}{The matrix permuted.} \item{samp}{The sequence of permutation.}
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{permut.vector}}
#' @keywords Auxiliary
#' @export
permut.row.matrix <- function(data, strata = NULL, seqpermutation = NULL)
{
  N <- nrow(data)
  if(!is.null(strata) & N!=length(strata)){
    stop("\n strata must be the length of number of row in the data\n")
  }
  if(!is.null(seqpermutation) & N!=length(seqpermutation)){
    stop("\n seqpermutation must be the length of number of row in the data\n")
  }
  if(is.null(seqpermutation)){
    samp <- permut.vector(N, strata = strata, nset = 1)
  } else {
    samp <- seqpermutation
  }
  permut.matrix <- data[samp, ,drop = FALSE]
  res <- list(permut.matrix = permut.matrix, samp = samp)
  return(res)
}

#' @title Co-inertia and Partial Co-inertia correlations.
#'
#' @description Function to obtain the RV coefficient between two matrices and
#' partial RV coefficient between three matrices.
#'
#' @encoding UTF-8
#' @aliases coinertia.syncsa coinertia.partial.syncsa
#' @param x,y Matrix that will be correlated.
#' @param z Matrix whose effect will be removed from the correlation between x and y.
#' @param scale Standardized variables to unit variance.
#' @return RV coefficient (generalization of the Pearson correlation coefficient) between matrices.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @references Legendre, P. and Legendre L. (2012). Numerical Ecology. 3nd English edition. Elsevier.
#' @seealso \code{\link{syncsa}}
#' @keywords SYNCSA
#' @export
coinertia.syncsa <- function(x, y, scale = FALSE){
  x <- as.matrix(scale(x, center = TRUE, scale = scale))
  y <- as.matrix(scale(y, center = TRUE, scale = scale))
  RV <- (sum((t(x)%*%y)^2))/sqrt(sum((t(x)%*%x)^2)*sum((t(y)%*%y)^2))
  return(RV)
}

#' @title Community data transformation
#'
#' @description Function to transformation community data and replace missing data. See details.
#'
#' @details The function applies standardization/transformation for community data. The options
#' are: "none" no transformation is applied; "standardized" the community data is standardized
#' to row totals will be 1; "weights" community data is first "standardized" and when, individual
#' species weights are multiplied to each species entries; and "beals" option applies Beals smoothing
#' using the function \code{\link[vegan]{beals}}. The arguments "reference", "type" and "include" are the same
#' used in \code{\link[vegan]{beals}} function. Missing data are replaced by 0 after the standardization/transformation.
#'
#' @encoding UTF-8
#' @importFrom vegan beals
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' @param transformation Method to community data transformation, "none", "standardized", "weights"
#' or "beals" (Default transformation = "standardized").
#' @param spp.weights Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).
#' @param reference Data to compute joint occurrences in \code{\link[vegan]{beals}} function. If NULL, comm is
#' used as reference to compute the joint occurrences (Default reference = NULL).
#' @param type Method to specifies how abundance values are used in \code{\link[vegan]{beals}} function, type = 0
#' presence/absence, type = 1 abundances are used to compute conditioned probabilities, type = 2 abundances
#' are used to compute weighted averages of conditioned probabilities or type = 3 abundances are used to
#' compute both conditioned probabilities and weighted averages (Default type = 0).
#' @param include Logical argument (TRUE or FALSE) to specify if target species are included when computing
#' the mean of the conditioned probabilities in \code{\link[vegan]{beals}} (Default include = TRUE).
#' @param notification Logical argument (TRUE or FALSE) to specify if notifications for missing observations
#' are to be shown (Default notification = TRUE).
#' @return Transformed community matrix, where rows are communities and columns species.
#' @note \strong{IMPORTANT}: Species sequence in the community data
#' matrix MUST be the same as the one in the reference matrix or in spp.weights vector. See
#' \code{\link{organize.syncsa}}.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{matrix.p}},
#' \code{\link{matrix.t}}, \code{\link{matrix.x}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#' De Cáceres, M.; Legendre, P. 2008. Beals smoothing revisited. Oecologia 156: 657–669.
#' @keywords Auxiliary
#' @examples
#' data(ADRS)
#' matrix.w.transformation(ADRS$community)
#' @export
matrix.w.transformation <- function(comm, transformation = "standardized", spp.weights = NULL,
                                     reference = NULL, type = 0, include = TRUE, notification = TRUE)
{
  matrix.w <- as.matrix(comm)
  TRANS <- c("none", "standardized", "weights", "beals")
  trans <- pmatch(transformation, TRANS)
  if (length(trans) > 1) {
    stop("\n Only one argument is accepted in transformation \n")
  }
  if (is.na(trans)) {
    stop("\n Invalid transformation \n")
  }
  w.NA <- apply(matrix.w, 2, is.na)
  if(notification){
    if(any(w.NA)){
      warning("Warning: NA in community data", call.= FALSE)
    }
  }
  if(trans == 2){
    matrix.w <- sweep(matrix.w, 1, rowSums(matrix.w, na.rm = TRUE), "/")
  }
  if(trans == 3){
    is.bin.weights <- all(spp.weights %in% c(0, 1))
    if(!is.bin.weights | is.null(spp.weights)){
      stop("\n spp.weights must be 0 or 1\n")
    }
    if((ncol(matrix.w) != length(spp.weights))){
      stop("\n spp.weights must be the same length of number of species \n")
    }
    matrix.w <- sweep(matrix.w, 1, rowSums(matrix.w, na.rm = TRUE), "/")
    spp.weights <- sapply(spp.weights, function(x, n) rep(x, n), n = nrow(matrix.w))
    matrix.w <- matrix.w*spp.weights
  }
  if(trans == 4){
    if(is.null(reference)){
      reference <- matrix.w
    }
    if((ncol(matrix.w) != ncol(reference)) | (nrow(matrix.w) != nrow(reference))){
      stop("\n comm and reference data must be the same dimensions \n")
    }
    matrix.w <- vegan::beals(matrix.w, reference = reference, type = type, include = include)
  }
  w.NA <- apply(matrix.w, 2, is.na)
  matrix.w[w.NA] <- 0
  return(matrix.w)
}

#' @title Matrix T
#'
#' @description Function to obtain a matrix containing trait averages at community level.
#' For more details, see \code{\link{syncsa}}.
#'
#' @encoding UTF-8
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' @param traits Matrix or data frame of species described by traits, with traits as
#' columns and species as rows.
#' @param scale Logical argument (TRUE or FALSE) to specify if the traits are
#' measured on different scales (Default scale = TRUE). When scale = TRUE traits
#' are measured on different scales and the matrix T is subjected to
#' standardization within each trait. When scale = FALSE traits are measured on
#' the same scale the matrix T is not subjected to standardization.
#' @param ranks Logical argument (TRUE or FALSE) to specify if ordinal variables are
#' convert to ranks (Default ranks = TRUE).
#' @param transformation Method to community data transformation, "none", "standardized" or "weights" (Default transformation = "standardized").
#' @param spp.weights Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).
#' @param notification Logical argument (TRUE or FALSE) to specify if
#' notifications of missing observations are shown (Default notification =
#' TRUE).
#' @return \item{matriz.w}{Standardized community matrix, where rows are
#' communities and columns species. If default transformation, row totals (communities) = 1.}
#' \item{matriz.b}{Matrix of traits, exactly the same data input.}
#' \item{matriz.T}{Matrix containing trait averages at community level. If
#' Scale = TRUE the matrix T is standardized within the traits.}
#' @note \strong{IMPORTANT}: The sequence species show up in community data
#' matrix MUST be the same as they show up in traits matrix or in the spp.weights vector. See
#' \code{\link{organize.syncsa}}.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{organize.syncsa}},
#' \code{\link{matrix.p}}, \code{\link{matrix.x}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#'
#' Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
#' Discriminating trait-convergence and trait-divergence assembly patterns in
#' ecological community gradients. Journal of Vegetation Science, 20, 334:348.
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' matrix.t(ADRS$community, ADRS$traits)
#' @export
matrix.t <- function (comm, traits, scale = TRUE, ranks = TRUE, transformation = "standardized",
                      spp.weights = NULL, notification = TRUE)
{
  vartype <- var.type(traits)
  if(any(vartype == "n")){
    stop("\n trait must contain only numeric, binary or ordinal variables \n")
  }
  for(i in 1:length(vartype)){
    if(ranks & vartype[i] == "o"){
      traits[, i] <- rank(traits[, i], na.last = "keep")
    }
    traits[, i] <- as.numeric(traits[, i])
  }
  traits <- as.matrix(traits)
  TRANS <- c("none", "standardized", "weights", "beals")
  trans <- pmatch(transformation, TRANS)
  if (length(trans) > 1) {
    stop("\n Only one argument is accepted in transformation \n")
  }
  if (is.na(trans) | trans == 4) {
    stop("\n Invalid transformation \n")
  }
  matrix.w <- matrix.w.transformation(comm, transformation = transformation,
                                      spp.weights = spp.weights, notification = notification)
  b.NA <- apply(traits, 2, is.na)
  if(notification){
    if(any(b.NA)){
      warning("Warning: NA in traits matrix", call. = FALSE)
    }
  }
  matrix.b <- traits
  matrix.T <- matmult.syncsa(matrix.w, traits)
  if (scale) {
    matrix.traits <- apply(matrix.T^2, 2, sum)
    matrix.T <- sweep(matrix.T, 2, sqrt(matrix.traits), "/")
  }
  return(list(matrix.w = matrix.w, matrix.b = matrix.b, matrix.T = matrix.T))
}

#' @title Collect names an entire list
#'
#' @description Function to collect names an entire list.
#'
#' @encoding UTF-8
#' @param l A list.
#' @param prefix A prefix to nomes.
#' @return The names.
#' @keywords Auxiliary
#' @export
CollectNames <- function(l, prefix = NULL) {
  if (!is.list(l)) return(NULL)
  if (is.data.frame(l)) return(NULL)
  names <- Map(paste, names(l), lapply(l, CollectNames), sep = "$")
  names <- gsub("\\$$", "", unlist(names, use.names = FALSE))
  names <- paste(prefix, names, sep = "")
  return(names)
}

#' @title Matrix centralization and standardization
#'
#' @description Function to perform centralization and standardization in a matrix.
#'
#' @encoding UTF-8
#' @param x A matrix
#' @param na.rm Logical argument (TRUE or FALSE) to specify if missing
#' observations are removed (Default na.rm = FALSE).
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}
#' @keywords Auxiliary
#' @export
cent.norm <- function (x, na.rm = FALSE)
{
  x.cent <- sweep(x, 2, colMeans(x, na.rm = na.rm), "-")
  x.norm <- apply(x.cent^2, 2, sum, na.rm = na.rm )
  x.cent.norm <- sweep(x.cent, 2, sqrt(x.norm), "/")
  return(x.cent.norm)
}

#' @title First-order partial correlation coefficient
#'
#' @description Function to obtain the first-order partial correlation coefficient.
#'
#' @encoding UTF-8
#' @param rxy Correlation between xy
#' @param rxz Correlation between xz
#' @param ryz Correlation between yz
#' @return The first-order partial correlation coefficient.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{cor.matrix}},
#' @keywords Auxiliary
#' @export
part.cor <- function(rxy, rxz, ryz)
{
  rxy <- round(rxy, 10)
  rxz <- round(rxz, 10)
  ryz <- round(ryz, 10)
  if (rxz == 1 | ryz == 1) {
    res <- 0
  }
  else {
    res <- (rxy - rxz * ryz)/(sqrt(1 - rxz * rxz)*sqrt(1 - ryz * ryz))
  }
  return(res)
}

#' @rdname cor.matrix
#' @include cor.matrix.R
#' @encoding UTF-8
#' @export
cor.matrix.bf <- function (dist.x, dist.y, method = "pearson", permutations = 999, strata = NULL,
                        na.rm = FALSE, seqpermutation = NULL, parallel = NULL, newClusters = TRUE,
                        CL =  NULL)
{
  if(!is.null(seqpermutation)){
    if(dim(seqpermutation)[1] != permutations){
      stop("\n seqpermutation must be the same dimension of permutations.\n")
    }
  }
  correlation <- cor(dist.x, dist.y, method = method)
  mx <- as.matrix(dist.x)
  N <- nrow(mx)
  if(is.null(seqpermutation)){
    seqpermutation <- permut.vector(N, strata = strata, nset = permutations)
  }
  if(!is.null(CL)){
    parallel <- length(CL)
  }
  if (na.rm){
    use <- "complete.obs"
  } else {
    use <- "all.obs"
  }
  ptest <- function(samp,mx,dist.y,method,use) {
    res <- cor(stats::as.dist(mx[samp, samp]), dist.y, method = method, use = use)
  }
  if(is.null(parallel)){
    value <- matrix(NA, nrow = permutations, ncol = 1)
    for (i in 1: permutations) {
      value[i,] <- ptest(samp = seqpermutation[i,], mx = mx, dist.y = dist.y, method = method, use = use)
    }
  } else {
    if (newClusters) {
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    }
    value <- cbind(parallel::parRapply(CL, seqpermutation, ptest, mx = mx, dist.y = dist.y, method = method, use = use))
    if (newClusters){
      parallel::stopCluster(CL)
    }
  }
  signific <- (sum(abs(value) >= abs(correlation)) + 1)/(permutations + 1)
  res <- list(Obs = correlation, p = signific)
  return(res)
}

#' @title Matrix multiplication with missing data
#'
#' @description Function to get the matrix product when missing data (NA) are found in matrix Y.
#' The matrix X must be standardized. See details.
#'
#' @details The function ignore missing data when found in matrix Y. Before multiplication of matrices
#' the missing data in Y are replaced by 0 and multiplication is performed, and then, an adjustment is
#' performed. This adjustment is done by divide each cell of the product matrix by the sum of
#' proportions of nonzero at X with complete data in Y. In SYNCSA context this adjustment is done by
#' divide each cell of the product matrix by the sum of species proportion with trait data in Y.
#' The matrix X must be standardized, in other words, row totals must be equal to 1.
#'
#' @encoding UTF-8
#' @param X A matrix, typically the standardized community matrix (W).
#' @param Y A matrix.
#' @return The matrix product.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{matmult}},
#' @keywords Auxiliary
#' @export
matmult.syncsa <- function(X, Y){
  y.NA <- is.na(Y)
  Y[y.NA] <- 0
  res <- X%*%Y
  if(any(y.NA)){
    X <- sweep(X, 1, rowSums(X, na.rm = TRUE), "/")
    adjustment <- X%*%ifelse(y.NA, 0, 1)
    res <- res/adjustment
    res.NA <- is.na(res)
    res[res.NA] <- 0
  }
  return(res)
}

#' @rdname optimal
#' @include optimal.R
#' @encoding UTF-8
#' @export
print.optimal <- function(x, ...)
{
  n <- 10
  if (nrow(x$optimization) < n) {
    n <- nrow(x$optimization)
  }
  cat("Call:\n")
  cat(deparse(x$call), "\n\n")
  if(length(x$list.warning)>0){
    cat("List of warning:\n")
    namestemp <- CollectNames(x$list.warning, prefix = "$list.warning$")
    cat(unlist(namestemp, use.names = FALSE), sep = "\n")
  }
  cat("\nNumber of subset:\n")
  cat(deparse(x$N_subset), "\n")
  cat("\n$optimization:\n")
  print(x$optimization[1:n,], ...)
  invisible(x)
}

#' @title SYNCSA
#'
#' @description This function integrates several steps for the analysis of phylogenetic
#' assembly patterns and their links to traits and ecological processes in a
#' metacommunity (Pillar et al. 2009, Pillar & Duarte 2010, Debastiani & Pillar 2012). The
#' function implement methods that have been available in the SYNCSA
#' application written in C++ (by Valerio Pillar, available at
#' http://ecoqua.ecologia.ufrgs.br/SYNCSA.html). See details.
#'
#'
#' @details Package \strong{SYNCSA} requires that the species and community sequence in
#' the data.frame or matrix must be the same for all dataframe/matrices.
#' The function \code{\link{organize.syncsa}} organizes the data for the functions
#' of the package, placing the matrices of community, traits, phylogenetic distance,
#' environmental varibles and strata vector in the same order. The function
#' use of function organize.syncsa is not requered for run the functions, but
#' is recommended. It requires data organized into the following matrices: (1) the
#' presences or abundances of species in a set of communities (\strong{W}); (2) the
#' phylogenetic pairwise dissimilarities of these species (\strong{DF}); (3) a set of
#' functional traits describing the species (\strong{B}), which
#' may be a mixture of binary and quantitative traits (continual and ordinal),
#' but not nominal ones (these should be expanded into binary traits); and (4)
#' the ecological gradient of interest, which may be one or more factors to
#' which the communities respond or ecosystem effects of the communities
#' (\strong{E}). In this way the arguments comm, traits, phylodist, envir,
#' as well as the arguments put.together and strata, can be specified them as normal
#' arguments or by passing them with the object returned by the function
#' \code{\link{organize.syncsa}} using, in this case only the argument comm. Using the
#' object returned by organize.syncsa, the comm argument is used as an alternative way
#' of entering to set all data.frames/matrices, and therefore the other arguments
#' (traits, phylodist, envir, put.together and strata) must be null.
#'
#'
#' \strong{Correlations}
#'
#' The function computes several correlations (Mantel or
#' Procrustes) that express trait-convergence assembly patterns (TCAP),
#' trait-divergence assembly patterns (TDAP), and phylogenetic signal in
#' functional traits at the species pool level and at the metacomunity level.
#' This function also generates P-values by permutation testing based on null
#' models (Pillar et al. 2009, Pillar & Duarte 2010).
#'
#' \strong{ro(TE)}
#'
#' This correlation refers to trait-convergence assembly patterns related to
#' the ecological gradient (TCAP, Pillar et al. 2009). For evaluating TCAP, by
#' matrix multiplication we define \strong{T = WB}, which with previous
#' standardization of \strong{W} to unit column totals will contain the trait
#' averages in each community. The elements in \strong{T} are community
#' weighted mean values or community functional parameters (Violle et al.
#' 2007). Standardization of the traits (rows) in \strong{T} is needed if the
#' trait set contains traits measured with different scales. By using matrix
#' correlation, we evaluate how the trait patterns in \strong{T} are associated
#' to ecological gradients in \strong{E}. For relating \strong{T} to
#' \strong{E}, using Mantel correlation we define a distance matrix of the
#' communities (\strong{DT}) using \strong{T}, and another distance matrix of
#' the community sites (\strong{DE}) using \strong{E}. The matrix correlation
#' ro(\strong{TE}) = ro(\strong{DT};\strong{DE}) measures the level of
#' congruence between TCAP and \strong{E}. A strong correlation ro(\strong{TE})
#' indicates the factors directly or indirectly represented in \strong{E} are
#' involved in ecological filtering of species that, at least for the traits
#' considered in the analysis, consistently produce trait-convergence assembly
#' patterns along the gradient comprising the metacommunity.
#'
#' \strong{ro(XE) and ro(XE.T)}
#'
#' These matrix correlations refer to trait-divergence assembly patterns
#' related to the ecological gradient (TDAP, Pillar et al. 2009). For the
#' identification of TDAP, in a first step the species pairwise similarities
#' (in the range 0 to 1) in matrix \strong{SB} based on traits in \strong{B}
#' are used to define matrix \strong{U} with degrees of belonging of species to
#' fuzzy sets. By matrix multiplication \strong{X = WU} will contain the
#' species composition of the communities after fuzzy-weighting by their trait
#' similarities (each row in \strong{X} will refer to a species). Matrix
#' \strong{X} expresses both TCAP and TDAP (Pillar et al. 2009). By using
#' matrix correlation, we evaluate how the trait patterns in \strong{X} (TCAP
#' and TDAP) are associated to ecological gradients in \strong{E}. For relating
#' \strong{X} to \strong{E}, we define a distance matrix of the communities
#' (\strong{DX}) using \strong{X}, and another distance matrix of the community
#' sites (\strong{DE}) using \strong{E}. The matrix correlation ro(\strong{XE})
#' = ro(\strong{DX};\strong{DE}) between \strong{X} and \strong{E} is defined.
#' We then remove the trait-convergence component ro(\strong{TE}) from
#' ro(\strong{XE}) by computing the partial matrix correlation
#' ro(\strong{XE.T}), which measures the level of congruence between TDAP and
#' \strong{E}. Trait-divergence assembly patterns (TDAP, Pillar et al. 2009)
#' may result from community assembly processes related to biotic interactions
#' (Stubbs & Wilson 2004; Wilson 2007).
#'
#' \strong{ro(PE)}
#'
#' This matrix correlation refers to the phylogenetic structure related to the
#' ecological gradient comprising the metacommunity. The phylogenetic pairwise
#' dissimilarities in \strong{DF} are transformed into similarities and used to
#' define degrees of belonging qij to fuzzy sets. This is analogous to the
#' definition of functional fuzzy sets (Pillar & Orloci 1991; Pillar et al.
#' 2009). Based on the phylogenetic similarities, every species i among s
#' species in the metacommunity specifies a fuzzy set to which every species j
#' (j = 1 to s species, including species i) belongs with a certain degree of
#' belonging in the interval [0, 1]. In our definition, each row in matrix
#' \strong{Q} with the degrees of belonging must add to unit, i.e., the degrees
#' of belonging of a given species across the fuzzy sets are standardized to
#' unit total. By matrix multiplication \strong{P = WQ} will contain the
#' composition of the communities after fuzzy-weighting of species presences or
#' abundances by the species` phylogenetic similarities. Each column in matrix
#' \strong{P} holds the phylogenetic structure of a community. The
#' standardization of \strong{Q} is essential for the community totals in each
#' column in \strong{W} remaining the same in \strong{P}. Further, matrix
#' \strong{W} is adjusted to unit column totals prior to multiplication, so
#' that the total richness or abundance within each community in \strong{W}
#' will be standardized. Matrix correlation ro(\strong{PE}) =
#' ro(\strong{DP};\strong{DE}) measures the strength of the association between
#' community distances based on their phylogenetic structure in \strong{DP} and
#' distances based on their ecological conditions (\strong{DE}). Further,
#' \strong{P} can be explored for phylogenetic patterns at the metacommunity
#' level by using, e.g., ordination techniques.
#'
#' \strong{ro(PT) and ro(PX.T)}
#'
#' These matrix correlations measure phylogenetic signal at the metacommunity
#' level related to TCAP and to TDAP. We define phylogenetic signal at the
#' metacommunity level related to TCAP (PSMT) as the correlation between the
#' phylogenetic structure described in matrix \strong{P} and the
#' trait-convergence structure described in matrix \strong{T}. For this, a
#' proper distance matrix (e.g. Euclidean distances) of communities
#' (\strong{DP}) is computed using \strong{P} and another distance matrix of
#' the same communities (\strong{DT}) is computed using \strong{T}. Then matrix
#' correlation ro(\strong{PT}) = ro(\strong{DP};\strong{DT}) will measure the
#' level of congruence between variation in \strong{P} and \strong{T}, which is
#' a measure of PSMT. A strong phylogenetic signal at the metacommunity level
#' is expected when communities that are more similar in terms of phylogenetic
#' structure are also similar regarding their average trait values. We also
#' define phylogenetic signal at the metacommunity level related to TDAP
#' (PSMX.T) as the partial matrix correlation ro(\strong{PX.T}) =
#' ro(\strong{DP};\strong{DX.DT}) between community distances DP computed on
#' phylogenetic structure and community distances \strong{DX} computed on
#' species composition after fuzzy-weighting by the species, or trait
#' similarities, removing the effect of TCAP (\strong{DT}). This is analogous
#' to TDAP.
#'
#' \strong{ro(BF)}
#'
#' This matrix correlation measures phylogenetic signal at the species pool
#' level (PSS, Pillar & Duarte 2010). We define PSS as the matrix correlation
#' ro(\strong{FB}) = ro(\strong{DF};\strong{DB}) between species phylogenetic
#' dissimilarities (already defined as matrix \strong{DF}) and species trait
#' dissimilarities (derived from already defined matrix \strong{SB}) computed
#' on any number of traits from matrix \strong{B}. The species pool refers to
#' the species present in the metacommunity.
#'
#' \strong{Additional matrix correlations}
#'
#' The matrix correlations ro(\strong{TE.P}) and ro(\strong{XE.P}) are also
#' computed, which may be useful for evaluating causal models in path analysis.
#'
#' \strong{Mantel correlations}
#'
#' The Mantel and Partial Mantel statistics are calculated simply as the correlation
#' entries the dissimilarity matrices, using standard Mantel test (see
#' \code{\link[vegan]{mantel}} and \code{\link{cor.matrix}}). Partial Mantel
#' statistic use paired correlation between the three matrices and obtains the partial
#' correlation using the formula of first-order partial correlation coefficient. The
#' significances are obtained using a different procedure than standard Mantel test,
#' see section Testing against null models below.
#'
#' \strong{Procrustes correlations}
#'
#' The Procrustes correlation uses symmetric Procrustes as a measure of concordance
#' between the data matrices (see \code{\link[vegan]{procrustes}} and
#' \code{\link{procrustes.syncsa}}). Procrustes procedure use rotation, translation,
#' and rescaling for minimizing sum of squared differences between two data sets.
#' The correlation of Procrustes is calculated as the statistic derived from the
#' symmetric Procrustes sum of squares, representing the optimal fit between the two
#' data matrices. Partial Procrustes correlation is obtained by Procrustes correlation
#' between residuals matrices. Firstly one Principal Components Analysis (PCA,
#' see \code{\link{prcomp}}) is performed in the matrix Z for dimensionality reduction.
#' The max number of axis kept in the analysis is the number of sampling units divided
#' by 2, this axes of PCA represent the total variation in the Z matrix. After the
#' kept axes are used as predictor in one linear model for each variable of the
#' matrices X and Y. For this a linear model is build using as response one variable
#' of X (same via for Y matrix) and as predictor all remaining axes of PCA, after model
#' fitted and the residual are extracted with the aim of form the residual matrix. The linear
#' model is repeated in the other variables, only with the changed the response variable.
#' The same procedure is performed in the matrix
#' Y. Both residual matrices are submitted to Procrustes analysis and the statistic is
#' returned as a partial correlation, the Partial Procrustes statistic. The significances
#' are obtained using the same procedure than Mantel test, see section Testing against
#' null models below.
#'
#' \strong{Co-inertia correlations}
#'
#' Co-inertia Analysis uses RV coefficient as a measure of correlation between the data matrices.
#' The procedure is equal to Procrustes correlation, however using the RV coefficient.
#'
#' \strong{The ro.method "none" or "nullmodel"}
#'
#' When ro.method is equal to "none" only observed matrices are returned,
#' already when it is equal to "nullmodel" observed matrices and null matrices
#' generated according the null models are returned. This null matrices can be
#' used to apply any statistical method to test relation between matrices not
#' include in the package (e.x. linear models).
#'
#' \strong{Testing against null models}
#'
#' All the matrix correlations are tested against null models. The null model
#' is defined accoding to the correlation being tested. Usually in the SYNCSA package
#' the null models are based in permutation of species rather than permutation
#' of sample units. For ro(\strong{TE}),each permutation generates a random
#' matrix \strong{T} calculated after the
#' permutation among the species vectors in matrix \strong{B}. For
#' ro(\strong{XE}) and ro(\strong{XE.T}), each permutation generates a random
#' matrix \strong{X} after the permutation among species fuzzy sets (rows) in
#' matrix \strong{U}. For ro(\strong{PE}), ro(\strong{PT}), and
#' ro(\strong{PX.T}), each permutation generates a random matrix \strong{P}
#' after the permutation among species fuzzy sets (rows) in matrix \strong{Q}.
#' For ro(\strong{BF}), a conventional Mantel test is performed with
#' dissimilarity matrices \strong{DF} and \strong{DB}. Analogous null models
#' are used for testing the additional matrix correlations; that is, the same
#' null model for ro(\strong{TE}) is used for ro(\strong{TE.P}), the same model
#' for ro(\strong{XE}) is used for ro(\strong{XE.P}). The permutation can be restrict
#' within species groups specifying the strata argument.
#'
#' \strong{Traits types}
#'
#' Traits data can be numeric, factor or ordered factor. For this be considered in the
#' analyses traits data must be of data.frame class and containing each variable type
#' determined. Gower index is used to calculate the similarity between species, using
#' the function gowdis of package FD. For additional details and requirements of function
#' please see \code{\link[FD]{gowdis}}.
#'
#' \strong{Community data standardization}
#'
#' By default the community data is standardization to row totals will be 1. The options
#' are: "none" no transformation is applied; "standardized" the community data is standardized
#' to row totals will be 1; and "weights" community data is first "standardized" and when, individual
#' species weights are multiplied to each species entries. An additional method to weights
#' individual species is included only \code{\link{rao.diversity}} function. Current version of package
#' does not allow choose standardization/transformation method in the \code{\link{optimal}} function.
#'
#' \strong{Missing data}
#'
#' The functions ignore missing data when specified. In the case of direct
#' multiplication of matrices the missing data are replaced by 0, ignoring the cell with missing value. For the
#' matrix \strong{T = WB} an adjustment is done by divide each cell of the product
#' matrix (\strong{T}) by the sum of species proportion with trait data in \strong{B}. Result
#' matrices are shown without missing values. Where the matrices are calculated
#' using a dissimilarity index (matrix \strong{U} and correlations between
#' matrices) the missing data are ignored as in \code{\link[vegan]{vegdist}} function.
#' In some cases the dissimilarity matrices obtained by the function
#' \code{\link[vegan]{vegdist}} still contain some missing values. In these cases the
#' rest of the procedure will be affected. In these cases you can find
#' solutions in impute the missing values.
#'
#' \strong{Error messenger and options}
#'
#' The data pass by several ckeck points that can produce error messenger. The
#' matrices or data frames must be contain only numeric, binary or ordinal
#' variables, in the way that nominal variable should be expanded into binary
#' (see \code{\link{var.dummy}}). For enhance the code speed some functions use
#' by default matrix algebra, this option can produce error under certain circumstances. This
#' global option can be changed using options("SYNCSA.speed" = FALSE). If SYNCSA.speed = TRUE
#' for use matrix algebra and if SYNCSA.speed = FALSE use not another function of same procedure.
#'
#' @encoding UTF-8
#' @aliases SYNCSA syncsa print.syncsa
#' @importFrom stats cor as.dist
#' @importFrom vegan wcmdscale protest vegdist
#' @importFrom permute how
#' @importFrom parallel makeCluster stopCluster
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' Alternatively comm can be an object of class metacommunity.data, an alternative
#' way to set all data.frames/matrices. When you use the class metacommunity.data the arguments
#' traits, phylodist, envir and put.together must be null. See details.
#' @param traits Data frame or matrix data of species described by traits, with traits as
#' columns and species as rows (Default traits = NULL).
#' @param phylodist Matrix containing phylogenetic distance between species.
#' Must be a complete matrix, not a half diagonal matrix (Default phylodist = NULL).
#' @param envir Environmental variables for each community, with variables as
#' columns and sampling units as rows (Default envir = NULL).
#' @param checkdata Logical argument (TRUE or FALSE) to check if species
#' sequence in the community data follows the same order as the one in the
#' trait and in the phylodist matrices and if sampling units in the community data follows
#' the same order as the one in the environmental data (Default checkdata = TRUE).
#' @param ro.method Method to obtain the correlation, "mantel", "procrustes", "coinertia",
#' "none" or "nullmodel" (Default ro.method = "mantel").
#' @param method Mantel correlation method, as accepted by cor: "pearson",
#' "spearman" or "kendall" (Default method = "pearson").
#' @param dist Dissimilarity index used for Mantel correlation, as accepted by
#' vegdist: "manhattan", "euclidean", "canberra", "bray", "kulczynski",
#' "jaccard", "gower", "altGower", "morisita", "horn", "mountford", "raup" ,
#' "binomial" or "chao" (Default dist = "euclidean").
#' @param scale Logical argument (TRUE or FALSE) to specify if the traits are
#' measured on different scales (Default Scale = TRUE). When scale = TRUE traits
#' are measured on different scales the the matrix T is subjected to
#' standardization within each trait. When scale = FALSE traits are measured on
#' the same scale and the matrix T is not subjected to standardization.
#' Furthermore, if scale = TRUE the matrix of traits is subjected to
#' standardization within each trait, and Gower Index is used to calculate the
#' degree of belonging to the species, and if scale = FALSE the matrix of
#' traits is not subjected to standardization, and Euclidean distance is
#' calculated to determine the degree of belonging to the species.
#' @param scale.envir Logical argument (TRUE or FALSE) to specify if the
#' environmental variables are measured on different scales. If the
#' enviromental variables are measured on different scales, the scale.envir
#' = TRUE the matrix with enviromental variables is subjected to centralization
#' and standardization within each variable. (Default scale.envir = TRUE).
#' @param ranks Logical argument (TRUE or FALSE) to specify if ordinal variables are
#' convert to ranks (Default ranks = TRUE).
#' @param asym.bin Vector listing the asymmetric binary traits, see \code{\link[FD]{gowdis}} (Default asym.bin = NULL).
#' @param ord Method to be used for ordinal traits, see \code{\link[FD]{gowdis}} (Default ord = "metric").
#' @param put.together List to specify group of traits. Each group specify receive the
#' same weight that one trait outside any group, in the way each group is considered
#' as unique trait (Default put.together = NULL). This argument must be a list, see
#' examples.
#' @param na.rm Logical argument (TRUE or FALSE) to specify if pairwise
#' distances should be deleted in cases of missing observations (Default na.rm
#' = FALSE).
#' @param transformation Method to transformation, "none", "standardized" or "weights"
#' (Default transformation = "standardized").
#' @param spp.weights Vector with 0 or 1 to specify individual species weights (Default
#' spp.weights = NULL).
#' @param strata Strata named vector to specify restricting permutations within species
#' groups (Default strata = NULL).
#' @param permutations Number of permutations in assessing significance.
#' @param parallel Number of parallel processes. Tip: use parallel::detectCores() (Default parallel = NULL).
#' @param notification Logical argument (TRUE or FALSE) to specify if
#' notification of missing observations should to be shown (Default
#' notification = TRUE).
#' @param x An object of class syncsa.
#' @param ... Other parameters for the respective functions.
#' @return \item{call}{The arguments used.}
#' \item{notes}{Some notes about the statistics.}
#' \item{statistics}{Correlations roTE, roXE, roPE, roPT, roPX.T, roXE.T, roTE.P, roXE.P
#' and roBF, and their significance levels based on permutations.}
#' \item{matrices}{A list with observed matrices produced for the functions, see details.}
#' \item{matrices.null}{A list of list with null matrices generated according the null model,
#' only if ro.method is "nullmodel". The number of matrices returned depends of the number of permutations.}
#' \item{FunRao}{Rao quadratic entropy within each community, considering trait distance.}
#' \item{traits.weights}{Weight for each trait.}
#'
#' @note The function calculates the correlations despite the lack of one of
#' the matrices, provided that community data had been entered. Correlations
#' including unspecified matrices will appear with NA.
#'
#' \strong{IMPORTANT}: The sequence of species in the community data matrix
#' MUST be the same as that in the phylogenetic distance matrix and in traits
#' matrix. Similarly, the sequence of communities in the community data matrix
#' MUST be the same as that in the environmental data. See details and
#' \code{\link{organize.syncsa}}.
#'
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{organize.syncsa}}, \code{\link{matrix.t}},
#' \code{\link{matrix.x}}, \code{\link{matrix.p}}, \code{\link{optimal}},
#' \code{\link{rao.diversity}}, \code{\link{cor.matrix}}, \code{\link{var.type}},
#' \code{\link{var.dummy}}
#'
#' @references
#'
#' Debastiani, V.J & Pillar, V.D., (2012). SYNCSA-R tool for analysis of metacommunities
#' based on functional traits and phylogeny of the community components. Bioinformatics,
#' 28(15), 2067–2068.
#'
#' Pillar, V.D.; Duarte, L.d.S., (2010). A framework for metacommunity analysis
#' of phylogenetic structure. Ecology Letters, 13, 587:596.
#'
#' Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
#' Discriminating trait-convergence and trait-divergence assembly patterns in
#' ecological community gradients. Journal of Vegetation Science, 20, 334:348.
#'
#' Pillar, V.D. & Orloci, L., (1991). Fuzzy components in community level
#' comparisons. In: Computer Assisted Vegetation Analysis (eds Feoli, E. &
#' Orloci, L.). Kluwer, Dordrecht, 87:93.
#'
#' Stubbs, W.J. & Wilson, J.B., (2004). Evidence for limiting similarity in a
#' sand dune community. Journal of Ecology, 92, 557:567.
#'
#' Violle, C., Navas, M.L., Vile, D., Kazakou, E., Fortunel, C., Hummel, I. &
#' Garnier, E., (2007). Let the concept of trait be functional! Oikos, 116,
#' 882:892.
#'
#' Wilson, J.B., (2007). Trait-divergence assembly rules have been demonstrated:
#' limiting similarity lives! A reply to Grime. Journal of Vegetation Science,
#' 18, 451:452.
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' syncsa(ADRS$community, ADRS$traits, ADRS$phylo, ADRS$envir, permutations = 99)
#' data(flona)
#' put.together<-list(c("fol","sem"), c("tam", "red"))
#' put.together
#' res<-syncsa(flona$community, flona$traits, envir = flona$environment,
#'    put.together = put.together, permutations = 99)
#' res$spp.weights
#' @export
syncsa <- function (comm, traits = NULL, phylodist = NULL, envir = NULL, checkdata = TRUE, ro.method = "mantel",
                    method = "pearson", dist = "euclidean", scale = TRUE, scale.envir = TRUE, ranks = TRUE,
                    asym.bin = NULL, ord = "metric", put.together = NULL,
                    na.rm = FALSE, transformation = "standardized", spp.weights = NULL,
                    strata = NULL, permutations = 999, parallel = NULL, notification = TRUE)
# syncsa <- function (comm, traits = NULL, phylodist = NULL, envir = NULL, checkdata = TRUE, ro.method = "mantel",
#                     method = "pearson", dist = "euclidean", scale = TRUE, scale.envir = TRUE, ranks = TRUE, ord,
#                     put.together = NULL, na.rm = FALSE, transformation = "standardized", spp.weights = NULL,
#                     strata = NULL, permutations = 999, parallel = NULL, notification = TRUE)
{
  res <- list(call = match.call())
  res.matrices <- list()
  res.matrices.null <- list()
  list.warning <- list()
  roTE <- NA
  roXE <- NA
  roPE <- NA
  roPT <- NA
  roPX.T <- NA
  roXE.T <- NA
  roTE.P <- NA
  roXE.P <- NA
  roBF <- NA
  roRE <- NA
  roTY <- NA
  roXY <- NA
  roXY.T <- NA
  note.roTE <- paste("Trait-convergence assembly patterns (TCAP): roTE")
  note.roXE <- paste("Both trait-convergence assembly patterns and trait-divergence assembly patterns: roXE")
  note.roXE.T <- paste("Trait-divergence assembly patterns (TDAP): roXE.T")
  note.roBF <- paste("Phylogenetic signal at species level: roBF")
  note.roPE <- paste("Correlation of phylogenetically structured assembly patterns to ecological variables: roPE")
  note.roPT <- paste("Correlation of phylogenetically structured assembly patterns to trait-convergence assembly patterns: roPT")
  note.roPX.T <- paste("Correlation of phylogenetically structured assembly patterns to trait-divergence assembly patterns: roPX.T")
  note.roTE.P <- paste("Removing phylogeny from trait-convergence assembly patterns: roTE.P")
  note.roXE.P <- paste("Removing phylogeny from both trait-convergence assembly patterns and trait-divergence assembly patterns: roXE.P")
  note.roRE <- paste("Alpha divergence (correlation between environmental variables and Rao entropy): roRE")
  note.roTY <- paste("Correlations with Beals smoothed community matrix (Y) and matrix T: roTY")
  note.roXY <- paste("Correlations with Beals smoothed community matrix (Y) and matrix X: roXY")
  note.roXY.T <- paste("Correlations with Beals smoothed community matrix (Y) and matrix X removing T: roXY.T")
  note <- rbind(note.roTE, note.roXE, note.roXE.T, note.roBF, note.roPE, note.roPT, note.roPX.T, note.roTE.P, note.roXE.P,
                note.roRE, note.roTY, note.roXY, note.roXY.T)
  colnames(note) <- "Correlation meanings"
  N <- permutations
  roMETHOD <- c("mantel", "procrustes", "coinertia", "none", "nullmodel")
  romethod <- pmatch(ro.method, roMETHOD)
  if (length(romethod) > 1) {
    stop("\n Only one argument is accepted in ro.method \n")
  }
  if (is.na(romethod)) {
    stop("\n Invalid ro.method \n")
  }
  if(romethod == 1 | romethod == 2 | romethod == 3){
    res$notes <- note
  }
  TRANS <- c("none", "standardized", "weights", "beals")
  trans <- pmatch(transformation, TRANS)
  if (length(trans) > 1) {
    stop("\n Only one argument is accepted in transformation \n")
  }
  if (is.na(trans) | trans == 4) {
    stop("\n Invalid transformation \n")
  }
  if (inherits(comm, "metacommunity.data")) {
    if (!is.null(traits) | !is.null(phylodist) | !is.null(envir) | !is.null(put.together) | !is.null(spp.weights) | !is.null(strata) ) {
      stop("\n When you use an object of class metacommunity.data the arguments traits, phylodist, envir, spp.weights, put.together and strata must be null. \n")
    }
    traits <- comm$traits
    phylodist <- comm$phylodist
    envir <- comm$environmental
    put.together <- comm$put.together
    spp.weights <- comm$spp.weights
    strata <- comm$strata
    comm <- comm$community
  }
  if(checkdata){
    organize.temp <- organize.syncsa(comm, traits = traits, phylodist = phylodist, envir = envir,
                                     strata = strata, spp.weights = spp.weights, check.comm = TRUE)
    if(!is.null(organize.temp$stop)){
      organize.temp$call <- match.call()
      return(organize.temp)
    }
    list.warning <- organize.temp$list.warning
    comm <- organize.temp$community
    traits <- organize.temp$traits
    phylodist <- organize.temp$phylodist
    envir <- organize.temp$environmental
    strata <- organize.temp$strata
    spp.weights <- organize.temp$spp.weights
  }
  if(notification & !checkdata){
    if (!missing(comm)) {
      if(any(is.na(comm))){
        warning("Warning: NA in community data", call. = FALSE)
      }
    }
    if (!is.null(traits)) {
      if(any(is.na(traits))){
        warning("Warning: NA in traits matrix", call. = FALSE)
      }
    }
    if (!is.null(phylodist)) {
      if(any(is.na(phylodist))){
        warning("Warning: NA in phylodist data", call. = FALSE)
      }
    }
    if (!is.null(envir)) {
      if(any(is.na(envir))){
        warning("Warning: NA in environmental data", call. = FALSE)
      }
    }
  }
  if(!is.null(strata)){
    if(length(strata) != ncol(comm)){
      stop("\n strata must be the same length of number of species \n")
    }
  }
  if(!is.null(spp.weights)){
    if(length(spp.weights) != ncol(comm)){
      stop("\n spp.weights must be the same length of number of species \n")
    }
  }
  if(!checkdata){
    commvartype <- var.type(comm)
    if(any(commvartype == "n")){
      stop("\n comm must contain only numeric, binary or ordinal variables \n")
    }
    if (!is.null(traits)) {
      traitsvartype <- var.type(traits)
      if(any(traitsvartype == "n")){
        stop("\n trait must contain only numeric, binary or ordinal variables \n")
      }
    }
    if (!is.null(phylodist)) {
      phylodistvartype <- var.type(phylodist)
      if(any(phylodistvartype == "n")){
        stop("\n phylodist must contain only numeric, binary or ordinal variables \n")
      }
    }
    if (!is.null(envir)) {
      envirvartype <- var.type(envir)
      if(any(envirvartype == "n")){
        stop("\n envir must contain only numeric, binary or ordinal variables \n")
      }
    }
  }
  if (!is.null(envir)) {
    if(romethod == 1 & any(is.na(suppressWarnings(vegan::vegdist(envir, method = dist, na.rm = TRUE))))){
      stop("\n envir with too much NA \n")
    }
    if(romethod == 2 & any(is.na(envir))){
      stop("\n envir with NA \n")
    }
    if(romethod == 3 & any(is.na(envir))){
      stop("\n envir with NA \n")
    }
  }
  res.matrices <- syncsa.obs(comm, traits = traits, phylodist = phylodist, envir = envir,
                             scale = scale, scale.envir = scale.envir, ranks = ranks,
                             asym.bin = asym.bin, ord = ord, put.together = put.together,
                             na.rm = na.rm, transformation = transformation,
                             spp.weights = spp.weights, notification = notification)
  if(romethod != 4){
    if (notification & !is.null(traits)) {
      spp.all.trait.na <- apply(is.na(traits), 1, sum)==ncol(traits)
      if(any(spp.all.trait.na)){
        warning("Some species with NA in all traits, the use of strata vector is recommended", call. = FALSE)
      }
    }
    seqpermutation <- permut.vector(ncol(comm), strata = strata, nset = permutations)
  }
  if(romethod == 1 | romethod == 2 | romethod == 3){
    if(!is.null(parallel)){
      CL <- parallel::makeCluster(parallel, type = "PSOCK")
    } else {
      CL <- NULL
    }
  }
  permute.syncsa <- function(samp, mx1, mx2, permut.x, norm.x){
    res <- list()
    mx2 <- mx2[samp, ,drop = FALSE]
    res$mx2.null <- mx2
    if(permut.x){
      x.permut <- matmult.syncsa(mx1, mx2)
      if (norm.x) {
        matrix.permut <- apply(x.permut^2, 2, sum)
        x.permut <- sweep(x.permut, 2, sqrt(matrix.permut), "/")
      }
      res$mx.null <- x.permut
    }
    return(res)
  }
  if (!is.null(traits)) {
    res$traits.weights <- res.matrices$traits.weights
    res.matrices$traits.weights <- NULL
    W <- res.matrices$W
    B <- res.matrices$B
    T <- res.matrices$T
    U <- res.matrices$U
    X <- res.matrices$X
    Y <- res.matrices$Y
    res$FunRao <- cbind(rao.diversity(comm, traits = B, checkdata = FALSE, put.together = put.together, transformation = transformation, spp.weights = spp.weights)$FunRao)
    colnames(res$FunRao) <- "FunRao"
    if(romethod == 5){
      res.matrices.null$B.NULL <- vector("list", N)
      res.matrices.null$T.NULL <- vector("list", N)
      res.matrices.null$U.NULL <- vector("list", N)
      res.matrices.null$X.NULL <- vector("list", N)
      res.matrices.null$RAO.NULL <- vector("list", N)
      for (i in 1:N) {
        temp <- permute.syncsa(seqpermutation[i,], mx1 = W, mx2 = B, permut.x = TRUE, norm.x = scale)
        res.matrices.null$B.NULL[[i]] <- temp$mx2.null
        res.matrices.null$T.NULL[[i]] <- temp$mx.null
        temp <- permute.syncsa(seqpermutation[i,], mx1 = W, mx2 = U, permut.x = TRUE, norm.x = FALSE)
        res.matrices.null$U.NULL[[i]] <- temp$mx2.null
        res.matrices.null$X.NULL[[i]] <- temp$mx.null
        temp <- cbind(rao.diversity(comm, traits = permute.syncsa(seqpermutation[i,], mx2 = B, permut.x = FALSE, norm.x = FALSE), checkdata = FALSE, put.together = put.together, transformation = transformation, spp.weights = spp.weights)$FunRao)
        colnames(temp) <- "FunRao"
        res.matrices.null$RAO.NULL[[i]] <- temp
      }
    }
    if(romethod == 1){
      roTY <- cor.matrix(mx1 = W, mx2 = B, x = T, y = Y, method = method, dist = dist, permutations = N, norm = scale, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      roXY <- cor.matrix(mx1 = W, mx2 = U, x = X, y = Y, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      roXY.T <- cor.matrix.partial(mx1 = W, mx2 = U, x = X, y = Y, mz1 = W, mz2 = B, z = T, permute.my2 = FALSE, permute.mz2 = TRUE, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
    }
    if(romethod == 2){
      roTY <- pro.matrix(mx1 = W, mx2 = B, x = T, y = Y, permutations = N, norm = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      roXY <- pro.matrix(mx1 = W, mx2 = U, x = X, y = Y, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      roXY.T <- pro.matrix.partial(mx1 = W, mx2 = U, x = X, y = Y, mz1 = W, mz2 = B, z = T, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, strata = strata, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
    }
    if(romethod == 3){
      roTY <- rv.matrix(mx1 = W, mx2 = B, x = T, y = Y, permutations = N, norm = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      roXY <- rv.matrix(mx1 = W, mx2 = U, x = X, y = Y, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      roXY.T <- rv.matrix.partial(mx1 = W, mx2 = U, x = X, y = Y, mz1 = W, mz2 = B, z = T, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, strata = strata, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
    }
    if (!is.null(envir)) {
      E <- res.matrices$E
      if(romethod == 1){
        roTE <- cor.matrix(mx1 = W, mx2 = B, x = T, y = E, method = method, dist = dist, permutations = N, norm = scale, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roXE <- cor.matrix(mx1 = W, mx2 = U, x = X, y = E, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roXE.T <- cor.matrix.partial(mx1 = W, mx2 = U, x = X, y = E, mz1 = W, mz2 = B, z = T, permute.my2 = FALSE, permute.mz2 = TRUE, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roRE <- cor.matrix.rao(mx1 = comm, B, res$FunRao, E, method = method, dist = dist, put.together = put.together, permutations = N, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 2){
        roTE <- pro.matrix(mx1 = W, mx2 = B, x = T, y = E, permutations = N, norm = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roXE <- pro.matrix(mx1 = W, mx2 = U, x = X, y = E, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roXE.T <- pro.matrix.partial(mx1 = W, mx2 = U, x = X, y = E, mz1 = W, mz2 = B, z = T, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, strata = strata, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roRE <- pro.matrix.rao(mx1 = comm, B, res$FunRao, E, put.together = put.together, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 3){
        roTE <- rv.matrix(mx1 = W, mx2 = B, x = T, y = E, permutations = N, norm = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roXE <- rv.matrix(mx1 = W, mx2 = U, x = X, y = E, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roXE.T <- rv.matrix.partial(mx1 = W, mx2 = U, x = X, y = E, mz1 = W, mz2 = B, z = T, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, strata = strata, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roRE <- rv.matrix.rao(mx1 = comm, B, res$FunRao, E, put.together = put.together, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
    }
  }
  if (!is.null(phylodist)) {
    W <- res.matrices$W
    Q <- res.matrices$Q
    P <- res.matrices$P
    if(romethod == 5){
      res.matrices.null$Q.NULL <- vector("list", N)
      res.matrices.null$P.NULL <- vector("list", N)
      for (i in 1:N) {
        temp <- permute.syncsa(seqpermutation[i,], mx1 = W, mx2 = Q, permut.x = TRUE, norm.x = FALSE)
        res.matrices.null$Q.NULL[[i]] <- temp$mx2.null
        res.matrices.null$P.NULL[[i]] <- temp$mx.null
      }
    }
    if (!is.null(envir)) {
      E <- res.matrices$E
      if(romethod == 1){
        roPE <- cor.matrix(mx1 = W, mx2 = Q, x = P, y = E, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 2){
        roPE <- pro.matrix(mx1 = W, mx2 = Q, x = P, y = E, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 3){
        roPE <- rv.matrix(mx1 = W, mx2 = Q, x = P, y = E, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if (!is.null(traits)) {
        if(romethod == 1){
          roTE.P <- cor.matrix.partial(mx1 = W, mx2 = B, x = T, y = E, mz1 = W, mz2 = Q, z = P, permute.my2 = FALSE, permute.mz2 = TRUE, method = method, dist = dist, permutations = N, norm = scale, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
          roXE.P <- cor.matrix.partial(mx1 = W, mx2 = U, x = X, y = E, mz1 = W, mz2 = Q, z = P, permute.my2 = FALSE, permute.mz2 = TRUE, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        }
        if(romethod == 2){
          roTE.P <- pro.matrix.partial(mx1 = W, mx2 = B, x = T, y = E, mz1 = W, mz2 = Q, z = P, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, norm = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
          roXE.P <- pro.matrix.partial(mx1 = W, mx2 = U, x = X, y = E, mz1 = W, mz2 = Q, z = P, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        }
        if(romethod == 3){
          roTE.P <- rv.matrix.partial(mx1 = W, mx2 = B, x = T, y = E, mz1 = W, mz2 = Q, z = P, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, norm = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
          roXE.P <- rv.matrix.partial(mx1 = W, mx2 = U, x = X, y = E, mz1 = W, mz2 = Q, z = P, permute.my2 = FALSE, permute.mz2 = TRUE, permutations = N, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        }
      }
    }
    if (!is.null(traits)) {
      if(romethod == 1){
        roPT <- cor.matrix(mx1 = W, mx2 = Q, x = P, my1= W, my2 = B, y = T, permute.my2 = TRUE, method = method, dist = dist, permutations = N, norm.y = scale, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roPX.T <- cor.matrix.partial(mx1 = W, mx2 = Q, x = P, my1 = W, my2 = U, y = X, mz1 = W, mz2 = B, z = T, permute.my2 = TRUE, permute.mz2 = TRUE, method = method, dist = dist, permutations = N, strata = strata, na.rm = na.rm, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 2){
        roPT <- pro.matrix(mx1 = W, mx2 = Q, x = P, my1= W, my2 = B, y = T, permute.my2 = TRUE, permutations = N, norm.y = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roPX.T <- pro.matrix.partial(mx1 = W, mx2 = Q, x = P, my1 = W, my2 = U, y = X, mz1 = W, mz2 = B, z = T, permute.my2 = TRUE, permute.mz2 = TRUE, permutations = N, strata = strata, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 3){
        roPT <- rv.matrix(mx1 = W, mx2 = Q, x = P, my1= W, my2 = B, y = T, permute.my2 = TRUE, permutations = N, norm.y = scale, strata = strata, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
        roPX.T <- rv.matrix.partial(mx1 = W, mx2 = Q, x = P, my1 = W, my2 = U, y = X, mz1 = W, mz2 = B, z = T, permute.my2 = TRUE, permute.mz2 = TRUE, permutations = N, strata = strata, norm.z = scale, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 1){
        if (scale) {
          dist.traits <- vegan::vegdist(traits, method = "gower", diag = TRUE, upper = TRUE, na.rm = na.rm)
        } else{
          dist.traits <- vegan::vegdist(traits, method = "euclidean", diag = TRUE, upper = TRUE, na.rm = na.rm)
        }
        roBF <- cor.matrix.bf(dist.traits, stats::as.dist(phylodist), method = method, permutations = N, strata = strata, na.rm = na.rm, seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 2){
        vectors <- vegan::wcmdscale(phylodist/max(phylodist), eig = TRUE)$points
        traits.t <- sweep(B, 2, sqrt(apply(B^2, 2, sum, na.rm = na.rm)), "/")
        roBF <- pro.matrix.bf(vectors, traits.t, permutations = N, strata = strata, na.rm = na.rm,seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
      if(romethod == 3){
        vectors <- vegan::wcmdscale(phylodist/max(phylodist), eig = TRUE)$points
        traits.t <- sweep(B, 2, sqrt(apply(B^2, 2, sum, na.rm = na.rm)), "/")
        roBF <- rv.matrix.bf(vectors, traits.t, permutations = N, strata = strata, na.rm = na.rm,seqpermutation = seqpermutation, parallel = parallel, newClusters = FALSE, CL = CL)
      }
    }
  }
  if(!is.null(parallel)){
    parallel::stopCluster(CL)
  }
  if(length(list.warning)>0){
    res$list.warning <- list.warning
  }
  res$matrices <- res.matrices
  if(romethod == 5){
    res$matrices.null <- res.matrices.null
  }
  if(romethod == 1 | romethod == 2 | romethod == 3){
    res$statistics <- rbind(roTE, roXE, roPE, roPT, roPX.T, roXE.T, roTE.P, roXE.P, roBF, roRE, roTY, roXY, roXY.T)
    if(is.null(colnames(res$statistics))){
      colnames(res$statistics) <- "Obs"
    }
  }
  class(res) <- "syncsa"
  return(res)
}

#' @rdname pca
#' @encoding UTF-8
#' @export
plot.pcasyncsa <- function(x, show = c("variables", "individuals"), axis = c(1, 2), xlab = axis[1],
                         ylab = axis[2], arrows = TRUE, text = TRUE, points = FALSE, ...)
{
	if (length(show) > 1) {
		stop("\n Only one argument is accepted in show \n")
	}
	SHOW <- c("variables", "individuals")
	show <- pmatch(show, SHOW)
	if(show == 1){
		circle <- seq(0, 2*pi, length = 200)
		graphics::plot(cos(circle), sin(circle), type = 'l', col = "gray", xlab = xlab, ylab = ylab, xlim = c(-1, 1), ylim = c(-1, 1), asp = 1, ...)
		graphics::abline(h = 0, v = 0, lty = 3, col = "gray")
		if(arrows){
			graphics::arrows(0 ,0, x1 = x$variables[, axis[1]]*0.9, y1 = x$variables[, axis[2]]*0.9, length = 0.09, ...)
		}
		graphics::text(x$variables[,axis[1]], x$variables[,axis[2]], labels = rownames(x$variables), ...)
	}
	if(show == 2){
		graphics::plot(x$individuals[, axis[1]], x$individuals[, axis[2]], xlab = xlab, ylab = ylab, type = "n", asp = 1, ...)
		if(text){
			graphics::text(x$individuals[, axis[1]], x$individuals[, axis[2]], labels = rownames(x$individuals), ...)
		}
		if(points){
			graphics::points(x$individuals[, axis[1]], x$individuals[, axis[2]], ...)
		}
	}
}

#' @title Function for organize data for Package SYNCSA
#'
#' @description Package \strong{SYNCSA} requires that the species and community sequence in
#' the data.frame or matrix must be the same for all data.frame/matrices.
#'
#' @details The function organizes the data for the functions of the package
#' SYNCSA, placing the matrices of community, traits, phylogenetic
#' distance, environmental varibles, species weights and strata vectors in the same order.
#' The function use as reference the community data for organize all data.frame or matrices
#' in the same order that the sampling units names and species names found in community
#' data set. For this all data sets entered must be correctly named, with rows and columns
#' named. The matrices phylodist, traits, envir can be larger than community
#' data (more species and/or more sampling units) as long as it has at least
#' all species and/or sampling units that are in community data. The function
#' organizes the data despite the absence of one of the data.frames or matrices,
#' provided that the community data had been entered. Unspecified data will
#' appear as NULL.
#'
#' When trait is a data.frame with different types of variables correctly identified and
#' the argument convert.traits is TRUE factor traits are expanded in dummy traits and ordinal variables
#' are converted in numeric according to ranks argument.
#'
#' The individual species weights (spp.weights) and strata must be named vectors. The strata vector
#' is a vector for restrict permutation within species groups, insofar as the SYNCSA package the null
#' models are based in permutation of species rather than permutation of sample units.
#'
#' @encoding UTF-8
#' @param comm Community data, with species as columns and sampling units as
#' rows.
#' @param traits Matrix or data.frame of species described by traits, with traits as
#' columns and species as rows (Default traits = NULL).
#' @param phylodist Matrix containing phylogenetic distance between species.
#' Must be a complete matrix (not a half diagonal matrix).This matrix can be
#' larger than community data (more species) as long as it has at least all
#' species that are in community data (Default phylodist = NULL).
#' @param envir Environmental variables for each community, with variables as
#' columns and sampling units as rows (Default envir = NULL).
#' @param strata Strata named vector to specify restricting permutations within
#' species groups (Default strata = NULL).
#' @param spp.weights Named vector to specify individual species weights (Default spp.weights = NULL).
#' @param check.comm Logical argument (TRUE or FALSE) to remove sampling units and
#' species with total sums equal or less than zero (Default check.comm = TRUE).
#' @param convert.traits Logical argument (TRUE or FALSE) to convert factor traits in
#' dummy traits and/or convert ordinal variables in numeric (see ranks argument)
#' (Default convert.traits = FALSE).
#' @param ranks Logical argument (TRUE or FALSE) to specify if ordinal variables are
#' convert to ranks. If ranks = TRUE all ordered variable are replaced by their ranks
#' and if ranks = FALSE all ordinal variables are simply treated as continuous variables
#' (Default ranks = TRUE).
#' @return A object of class metacommunity.data (also of the class list) with the data.frames or matrices:
#' \item{call}{The arguments used.}
#' \item{community}{Community data.}
#' \item{traits}{Traits data.}
#' \item{phylodist}{Phylogenetic distance.}
#' \item{environmental}{Environmental variables.}
#' \item{community.var.type}{Type of each varible in community data,
#' where 'c' to continuous/numeric, 'o' to ordinal, 'b' to binary and 'f' to factor. Nominal are not allowed.}
#' \item{traits.var.type}{Type of each varible in traits data. See labels above.}
#' \item{phylodist.var.type}{Type of each varible in phylodist. See labels above.}
#' \item{environmental.var.type}{Type of each varible in environmental data. See labels above.}
#' \item{strata}{The strata vector for permutations.}
#' \item{put.together}{A list with suggestion to group of traits that are analyzed together,
#' only if convert.traits is TRUE and if some traits are of factor class.}
#' \item{spp.weights}{The individual species weights vector.}
#' \item{list.warning}{A list of warning.}
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{var.dummy}}, \code{\link{var.type}}
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' organize.syncsa(ADRS$community, ADRS$traits, ADRS$phylo, ADRS$envir)
#' @export
organize.syncsa <- function (comm, traits = NULL, phylodist = NULL, envir = NULL,
                             strata = NULL, spp.weights = NULL,
                             check.comm = TRUE, convert.traits = FALSE, ranks = TRUE)
{
  res <- list(call = match.call())
  res.temp <- res
  res.temp$stop <- FALSE
  if (missing(comm)){
    stop("\n comm not fount\n")
  }
  if(!inherits(comm, c("data.frame", "matrix"))){
    stop("comm must be a matrix or a data.frame")
  }
  if (is.null(colnames(comm))){
    stop("\n Column names of comm are null\n")
  }
  if (is.null(rownames(comm))){
    stop("\n Row names of comm are null\n")
  }
  commvartype <- var.type(comm)
  if(any(commvartype == "n") | any(commvartype == "f") | any(commvartype == "o")){
    stop("\n comm must contain only numeric or binary variables \n")
  }
  list.warning <- list()
  put.together <- NULL
  if(check.comm){
    col.rm <- colnames(comm)[!colSums(comm, na.rm = TRUE)>0]
    row.rm <- rownames(comm)[!rowSums(comm, na.rm = TRUE)>0]
    if(length(col.rm)>0){
      warning("Species removed from community data - Check list of warning in list.warning", call. = FALSE)
      list.warning$comm$spp <- data.frame(species.removed = col.rm)
    }
    if(length(row.rm)>0){
      warning("Communities removed from community data - Check list of warning in list.warning", call. = FALSE)
      list.warning$comm$comm <- data.frame(communities.removed = row.rm)
    }
    comm <- comm[, colSums(comm, na.rm = TRUE)>0, drop = FALSE]
    comm <- comm[rowSums(comm,na.rm=TRUE)>0, , drop = FALSE]
  }
  if(any(is.na(comm))){
    warning("Warning: NA in community data", call. = FALSE)
  }
  if (!is.null(traits)) {
    if(!inherits(traits, c("data.frame", "matrix"))){
      stop("traits must be a matrix or a data.frame")
    }
    if (is.null(colnames(traits))){
      stop("\n Column names of traits are null\n")
    }
    if (is.null(rownames(traits))){
      stop("\n Row names of traits are null\n")
    }
    traitsvartype <- var.type(traits)
    if(any(traitsvartype == "n")){
      stop("\n trait must contain only numeric, binary, factor or ordinal variables \n")
    }
    match.names <- match(colnames(comm), rownames(traits))
    if(sum(is.na(match.names))>0){
      list.warning$traits$spp <- data.frame(species.not.on.traits = setdiff(colnames(comm), rownames(traits)))
      warning("ERROR - Check list of warning in list.warning$traits", call. = FALSE)
      res.temp$stop <- TRUE
      res.temp$list.warning <- list.warning
      return(res.temp)
    }
    traits <- as.data.frame(traits[match.names, , drop = FALSE])
    if(convert.traits){
      if(any(traitsvartype == "f")){
        warning("Factor variables expanded in dummy variables", call. = FALSE)
      }
      traits.dummy.temp <- var.dummy(traits)
      traits <- traits.dummy.temp$data
      put.together <- traits.dummy.temp$together
      traitsvartype <- var.type(traits)
      if(any(traitsvartype == "o")){
        warning("Ordinal variables transformed in continual variables", call. = FALSE)
      }
      traits <- data.matrix(traits)
      for (i in 1:length(traitsvartype)){
        if (traitsvartype[i] == "o"){
          if (ranks){
            traits[,i] <- rank(traits[,i], na.last = "keep")
          } else {
            traits[,i] <- as.numeric(traits[, i])
          }
        }
      }
      traitsvartype <- var.type(traits)
    }
    if(any(is.na(traits))){
      warning("Warning: NA in traits matrix", call. = FALSE)
    }
  }
  if (!is.null(phylodist)) {
    if(!inherits(phylodist, c("data.frame", "matrix"))){
      stop("phylodist must be a matrix or a data.frame")
    }
    if (is.null(colnames(phylodist))){
      stop("\n Column names of phylodist are null\n")
    }
    if (is.null(rownames(phylodist))){
      stop("\n Row names of phylodist are null\n")
    }
    phylodistvartype <- var.type(phylodist)
    if(any(phylodistvartype == "n") | any(phylodistvartype == "f") | any(phylodistvartype == "o")){
      stop("\n phylodist must contain only numeric or binary variables \n")
    }
    match.names <- match(colnames(comm), colnames(phylodist))
    if(sum(is.na(match.names))>0){
      list.warning$phylodist$spp <- data.frame(species.not.on.phylodist = setdiff(colnames(comm), colnames(phylodist)))
      warning("ERROR - Check list of warning in list.warning$phylodist", call. = FALSE)
      res.temp$stop <- TRUE
      res.temp$list.warning <- list.warning
      return(res.temp)
    }
    phylodist <- phylodist[match.names, match.names, drop = FALSE]
    if(any(is.na(phylodist))){
      warning("Warning: NA in phylogenetic distance matrix",call.=FALSE)
    }
  }
  if (!is.null(envir)) {
    if(!inherits(envir, c("data.frame", "matrix"))){
      stop("envir must be a matrix or a data.frame")
    }
    if (is.null(colnames(envir))){
      stop("\n Column names of envir are null\n")
    }
    if (is.null(rownames(envir))){
      stop("\n Row names of envir are null\n")
    }
    envirvartype <- var.type(envir)
    if(any(envirvartype == "n") | any(envirvartype == "f") | any(envirvartype == "o")){
      stop("\n envir must contain only numeric or binary variables \n")
    }
    match.names <- match(rownames(comm), rownames(envir))
    if(sum(is.na(match.names))>0){
      list.warning$envir$comm <- data.frame(comm.not.on.envir = setdiff(rownames(comm), rownames(envir)))
      warning("ERROR - Check list of warning in list.warning$envir", call. = FALSE)
      res.temp$stop <- TRUE
      res.temp$list.warning <- list.warning
      return(res.temp)
    }
    envir <- envir[match.names,,drop=FALSE]
    if(any(is.na(envir))){
      warning("Warning: NA in environmental data", call. = FALSE)
    }
  }
  if (!is.null(strata)) {
    if (is.null(names(strata))){
      stop("\n Names of strata factor are null\n")
    }
    match.names <- match(colnames(comm), names(strata))
    if(sum(is.na(match.names))>0){
      list.warning$strata <- data.frame(species.not.on.strata = setdiff(colnames(comm), names(strata)))
      warning("ERROR - Check list of warning in list.warning$strata", call. = FALSE)
      res.temp$stop <- TRUE
      res.temp$list.warning <- list.warning
      return(res.temp)
    }
    strata <- strata[match.names]
  }
  if (!is.null(spp.weights)) {
    if (is.null(names(spp.weights))){
      stop("\n Names of spp.weights are null\n")
    }
    match.names <- match(colnames(comm), names(spp.weights))
    if(sum(is.na(match.names))>0){
      list.warning$spp.weights <- data.frame(species.not.on.spp.weights = setdiff(colnames(comm), names(spp.weights)))
      warning("ERROR - Check list of warning in list.warning$spp.weights", call. = FALSE)
      res.temp$stop <- TRUE
      res.temp$list.warning <- list.warning
      return(res.temp)
    }
    spp.weights <- spp.weights[match.names]
  }
  if (!is.null(traits)) {
    spp.all.trait.na <- apply(is.na(traits), 1, sum)==ncol(traits)
    if(any(spp.all.trait.na)){
      if (is.null(strata)) {
        strata <- rep(1, nrow(traits))
        names(strata) <- colnames(comm)
      }
      strata[spp.all.trait.na] <- max(strata)+seq_len(sum(spp.all.trait.na))
      warning("The strata vector was generated or modified", call. = FALSE)
    }
  }
  if (is.null(traits)){
    traits <- NULL
    traitsvartype <- NULL
  }
  if (is.null(phylodist)){
    phylodist <- NULL
    phylodistvartype <- NULL
  }
  if (is.null(envir)){
    envir <- NULL
    envirvartype <- NULL
  }
  if (is.null(strata)){
    strata <- NULL
  }
  if (is.null(spp.weights)){
    spp.weights <- NULL
  }
  if(length(list.warning)>0){
    res$list.warning <- list.warning
  }
  res$community <- comm
  res$traits <- traits
  res$phylodist <- phylodist
  res$environmental <- envir
  res$community.var.type <- commvartype
  res$traits.var.type <- traitsvartype
  res$phylodist.var.type <- phylodistvartype
  res$environmental.var.type <- envirvartype
  res$strata <- strata
  res$put.together <- put.together
  res$spp.weights <- spp.weights
  class(res) <- c("list", "metacommunity.data")
  return(res)
}

#' @rdname syncsa
#' @encoding UTF-8
#' @export
syncsa.obs <- function (comm, traits = NULL, phylodist = NULL, envir = NULL,
                        scale = TRUE, scale.envir = TRUE, ranks = TRUE,
                        asym.bin = NULL, ord = "metric", put.together = NULL,
                        na.rm = FALSE, transformation = "standardized",
                        spp.weights = NULL, notification = TRUE)
  # syncsa.obs <- function (comm, traits = NULL, phylodist = NULL, envir = NULL,
  #                         scale = TRUE, scale.envir = TRUE, ranks = TRUE, ord,
  #                         put.together = NULL, na.rm = FALSE, transformation = "standardized",
  #                         spp.weights = NULL, notification = TRUE)
{
  res <- list(W = NULL, Y = NULL,
              B = NULL, T = NULL,
              U = NULL, X = NULL,
              Q = NULL, P = NULL, traits.weights = NULL)
  if (is.null(traits) & is.null(phylodist)){
    res$W <- matrix.w.transformation(comm, transformation = transformation,
                                     spp.weights = spp.weights, notification = FALSE)
  }
  if (!is.null(traits)) {
    m <- ncol(traits)
    traits.weights <- rep(1, m)
    make.names <- is.null(colnames(traits))
    colnames(traits) <- colnames(traits, do.NULL = FALSE, prefix = "T")
    names(traits.weights) <- colnames(traits)
    if(!is.null(put.together)){
      if(!inherits(put.together, "list")){
        stop("\n put.together must be a object of class list\n")
      }
      if(make.names){
        for(k in 1:length(put.together)){
          put.together[[k]] <- paste("T", put.together[[k]], sep = "")
        }
      }
      if(max(table(unlist(put.together)))>1){
        stop("\n The same trait appears more than once in put.together\n")
      }
      if(length(setdiff(unlist(put.together),colnames(traits)))>0){
        stop("\n Check traits names in put.together\n")
      }
      for(k in 1:length(put.together)){
        traits.weights[put.together[[k]]] <- 1/length(put.together[[k]])
      }
    }
    matrixT <- matrix.t(comm, traits, scale = scale, ranks = ranks,
                        transformation = transformation, spp.weights = spp.weights, notification = FALSE)
    check.U <- function(traits, scale, ranks, ...){
      # check.U <- function(traits, scale, ranks, ord, ...){
      vartype <- var.type(traits)
      # if(missing(ord)){
      #   for(i in 1:length(vartype)){
      #     if(ranks & vartype[i] == "o"){
      #       traits[, i] <- rank(traits[, i], na.last = "keep")
      #     }
      #     traits[, i] <- as.numeric(traits[, i])
      #   }
      #   traits <- as.matrix(traits)
      # }
      for(i in 1:length(vartype)){
        if(ranks & vartype[i] == "o"){
          traits[, i] <- rank(traits[, i], na.last = "keep")
        }
      }
      if (scale) {
        # dist.traits <- FD::gowdis(traits, asym.bin = NULL, ...)
        dist.traits <- FD::gowdis(traits, ...)
      }
      else{
        dist.traits <- as.matrix(vegan::vegdist(traits, method = "euclidean", diag = TRUE, upper = TRUE, na.rm = TRUE))
      }
      res <- any(is.na(dist.traits))
      return(res)
    }
    if(notification){
      # if(check.U(traits, scale = scale, ranks = ranks, ord, w = weights)){
      if(check.U(traits, scale = scale, ranks = ranks, asym.bin = asym.bin, ord = ord, w = traits.weights)){
        warning("Warning: NA in distance matrix between species based in traits", call. = FALSE)
      }
    }
    matrixX <- matrix.x(comm, traits, scale = scale, ranks = ranks, notification = FALSE,
                        asym.bin = asym.bin, ord = ord, w = traits.weights,
                        transformation = transformation, spp.weights = spp.weights)
    res$W <- matrixT$matrix.w
    res$B <- matrixT$matrix.b
    res$T <- matrixT$matrix.T
    res$U <- matrixX$matrix.u
    res$X <- matrixX$matrix.X
    res$traits.weights <- traits.weights
  }
  if (!is.null(phylodist)) {
    matrixP <- matrix.p(comm, phylodist, transformation = transformation, spp.weights = spp.weights, notification = FALSE)
    res$W <- matrixP$matrix.w
    res$Q <- matrixP$matrix.q
    res$P <- matrixP$matrix.P
  }
  if (!is.null(envir)) {
    E <- envir
    if (scale.envir) {
      E <- cent.norm(envir, na.rm = na.rm)
    }
    res$E <- E
  }
  res$Y <- matrix.w.transformation(res$W, transformation = "beals", reference = NULL,
                                   type = 3, include = TRUE, notification = FALSE)
  return(res)
}

#' @title Degree of belonging of species
#'
#' @description Function to obtain a matrix containing the degrees of belongings of each
#' and every species. The degree of belonging of each species is defined based on
#' its ecological or phylogenetic resemblance to every other
#' species in the community. For more details, see \code{\link{matrix.p}},
#' \code{\link{matrix.x}} and \code{\link{syncsa}}.
#'
#' @encoding UTF-8
#' @param dis Matrix containing distance between species.
#' @param standardize Logical argument (TRUE or FALSE) to specify if dis must
#' be standardized in values ranging from 0 to 1 (Default standardize = TRUE).
#' @return Standardized matrix containing the degree of belonging of species in
#' relation to each other. Row totals (species) = 1.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{matrix.p}}, \code{\link{matrix.x}},
#' \code{\link{syncsa}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' belonging(ADRS$phylo)
#' @export
belonging <- function (dis, standardize = TRUE)
{
  distance <- as.matrix(dis)
  if(standardize){
    distance <- distance/max(distance)
  }
  similarity <- 1-distance
  stats <- 1/colSums(similarity)
  res <- sweep(similarity, 1, stats, "*")
  return(res)
}

#' @title Text Progress Bar
#'
#' @description Function to display a progress bar in the R console. See \code{\link{txtProgressBar}}.
#'
#' @encoding UTF-8
#' @importFrom utils setTxtProgressBar txtProgressBar
#' @param n Number of the current progress.
#' @param N Total number of cases.
#' @param ... Other parameters for the txtProgressBar function.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @keywords Auxiliary
#' @export
ProgressBAR<-function(n, N, ...)
{
  n <- n/N
  for(i in c(0, n)){
    A <- utils::txtProgressBar(...)
    utils::setTxtProgressBar(A, i)
  }
}

#' @title Matrix X
#'
#' @description Function to obtain a matrix containing trait-weighted species composition.
#' For more details, see \code{\link{syncsa}}.
#'
#' @encoding UTF-8
#' @importFrom vegan vegdist
#' @importFrom FD gowdis
#' @param comm Community data, with species as columns and sampling units as
#' rows. This matrix can contain either presence/absence or abundance data.
#' @param traits Matrix or data frame of species described by traits, with traits as
#' columns and species as rows.
#' @param scale Logical argument (TRUE or FALSE) to specify if the traits are
#' measured on different scales (Default scale = TRUE). When scale = TRUE traits
#' are measured on different scales the matrix of traits is subjected to
#' standardization within each trait, and Gower Index is used to calculate the
#' degree of belonging to the species. When scale = FALSE traits are measured on
#' the same scale the matrix of traits is not subjected to standardization,
#' and Euclidean distance is calculated to determine the degree of belonging to
#' the species.
#' @param ranks Logical argument (TRUE or FALSE) to specify if ordinal variables are
#' convert to ranks (Default ranks = TRUE).
#' @param transformation Method to community data transformation, "none", "standardized"
#' or "weights" (Default transformation = "standardized").
#' @param spp.weights Vector with 0 or 1 to specify individual species weights (Default
#' spp.weights = NULL).
#' @param notification Logical argument (TRUE or FALSE) to specify if
#' notifications of missing observations are shown (Default notification =
#' TRUE).
#' @param ... Parameters for \code{\link[FD]{gowdis}} function.
#' @return \item{matriz.w}{Standardized community matrix, where rows are
#' communities and columns species. If default transformation, row totals (communities) = 1.}
#' \item{matriz.u}{Standardized matrix containing the degree of belonging of
#' each species in relation to each other species. Row totals (species) = 1.}
#' \item{matriz.X}{Trait-weighted species composition matrix. If default transformation,
#' row totals (communities) = 1.}
#' @note \strong{IMPORTANT}: The sequence species show up in community data
#' matrix MUST be the same as they show up in traits matrix or in the spp.weights vector. See
#' \code{\link{organize.syncsa}}.
#' @author Vanderlei Julio Debastiani <vanderleidebastiani@@yahoo.com.br>
#' @seealso \code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{belonging}},
#' \code{\link{matrix.t}}, \code{\link{matrix.p}}, \code{\link[FD]{gowdis}}
#' @references Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
#' metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
#' 587-596.
#'
#' Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
#' Discriminating trait-convergence and trait-divergence assembly patterns in
#' ecological community gradients. Journal of Vegetation Science, 20, 334:348.
#' @keywords SYNCSA
#' @examples
#' data(ADRS)
#' matrix.x(ADRS$community, ADRS$traits)
#' @export
matrix.x <- function (comm, traits, scale = TRUE, ranks = TRUE,
                      transformation = "standardized", spp.weights = NULL, notification = TRUE, ...)
{
  vartype <- var.type(traits)
  # if(any(vartype == "n")){
  #   stop("\n trait must contain only numeric, binary or ordinal variables \n")
  # }
  # if(missing(ord)){
  #   for(i in 1:length(vartype)){
  #     if(ranks & vartype[i] == "o"){
  #       traits[, i] <- rank(traits[, i], na.last = "keep")
  #     }
  #     traits[, i] <- as.numeric(traits[, i])
  #   }
  #   traits <- as.matrix(traits)
  # }
  # if(missing(ord)){
  for(i in 1:length(vartype)){
    if(ranks & vartype[i] == "o"){
      traits[, i] <- rank(traits[, i], na.last = "keep")
    }
    # traits[, i] <- as.numeric(traits[, i])
  }
  # traits <- as.matrix(traits)
  # }
  TRANS <- c("none", "standardized", "weights", "beals")
  trans <- pmatch(transformation, TRANS)
  if (length(trans) > 1) {
    stop("\n Only one argument is accepted in transformation \n")
  }
  if (is.na(trans) | trans == 4) {
    stop("\n Invalid transformation \n")
  }
  matrix.w <- matrix.w.transformation(comm, transformation = transformation,
                                      spp.weights = spp.weights, notification = notification)
  x.NA <- apply(traits, 2, is.na)
  if(notification){
    if(any(x.NA)){
      warning("Warning: NA in traits matrix", call. = FALSE)
    }
  }
  if (scale) {
    dist.traits <- as.matrix(FD::gowdis(traits, ...))
    ## In case a species has NO trait info, replace the diag with NA
    diag(dist.traits)[which(rowSums(is.na(dist.traits))==(ncol(dist.traits)-1))] <- NA
    similar.traits <- 1 - dist.traits
    matrix.traits <- 1/rowSums(similar.traits, na.rm = TRUE)
    matrix.u <- sweep(similar.traits, 1, matrix.traits, "*")
  }
  else{
    dist.traits <- as.matrix(vegan::vegdist(traits, method = "euclidean", diag = TRUE, upper = TRUE, na.rm = TRUE))
    similar.traits <- 1 - (dist.traits/max(dist.traits, na.rm = TRUE))
    matrix.traits <- 1/rowSums(similar.traits, na.rm = TRUE)
    matrix.u <- sweep(similar.traits, 1, matrix.traits, "*")
  }
  u.NA <- apply(matrix.u, 2, is.na)
  if(notification){
    if(any(u.NA)){
      warning("Warning: NA in matrix U", call. = FALSE)
    }
  }
  # matrix.u[u.NA] <- 0
  # matrix.X <- matrix.w %*% matrix.u
  matrix.X <- matmult.syncsa(matrix.w, matrix.u)
  matrix.u[u.NA] <- 0
  # if(any(u.NA)){
  #   matrix.X <- sweep(matrix.X, 1, rowSums(matrix.X), "/")
  # }
  return(list(matrix.w = matrix.w, matrix.u = matrix.u, matrix.X = matrix.X))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permut.row.matrix.R
\encoding{UTF-8}
\name{permut.row.matrix}
\alias{permut.row.matrix}
\title{Permutate rows in a matrix}
\usage{
permut.row.matrix(data, strata = NULL, seqpermutation = NULL)
}
\arguments{
\item{data}{A matrix.}

\item{strata}{Strata vector to specify restricting permutations in rows, must be the same length of number of rows in data matrix (Default strata = NULL).}

\item{seqpermutation}{A set of predefined permutation vector (Default seqpermutation = NULL).}
}
\value{
\item{permut.matrix}{The matrix permuted.} \item{samp}{The sequence of permutation.}
}
\description{
Function to permutate rows in a matrix.
}
\seealso{
\code{\link{syncsa}}, \code{\link{permut.vector}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix.w.transformation.R
\encoding{UTF-8}
\name{matrix.w.transformation}
\alias{matrix.w.transformation}
\title{Community data transformation}
\usage{
matrix.w.transformation(
  comm,
  transformation = "standardized",
  spp.weights = NULL,
  reference = NULL,
  type = 0,
  include = TRUE,
  notification = TRUE
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.}

\item{transformation}{Method to community data transformation, "none", "standardized", "weights"
or "beals" (Default transformation = "standardized").}

\item{spp.weights}{Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).}

\item{reference}{Data to compute joint occurrences in \code{\link[vegan]{beals}} function. If NULL, comm is
used as reference to compute the joint occurrences (Default reference = NULL).}

\item{type}{Method to specifies how abundance values are used in \code{\link[vegan]{beals}} function, type = 0
presence/absence, type = 1 abundances are used to compute conditioned probabilities, type = 2 abundances
are used to compute weighted averages of conditioned probabilities or type = 3 abundances are used to
compute both conditioned probabilities and weighted averages (Default type = 0).}

\item{include}{Logical argument (TRUE or FALSE) to specify if target species are included when computing
the mean of the conditioned probabilities in \code{\link[vegan]{beals}} (Default include = TRUE).}

\item{notification}{Logical argument (TRUE or FALSE) to specify if notifications for missing observations
are to be shown (Default notification = TRUE).}
}
\value{
Transformed community matrix, where rows are communities and columns species.
}
\description{
Function to transformation community data and replace missing data. See details.
}
\details{
The function applies standardization/transformation for community data. The options
are: "none" no transformation is applied; "standardized" the community data is standardized
to row totals will be 1; "weights" community data is first "standardized" and when, individual
species weights are multiplied to each species entries; and "beals" option applies Beals smoothing
using the function \code{\link[vegan]{beals}}. The arguments "reference", "type" and "include" are the same
used in \code{\link[vegan]{beals}} function. Missing data are replaced by 0 after the standardization/transformation.
}
\note{
\strong{IMPORTANT}: Species sequence in the community data
matrix MUST be the same as the one in the reference matrix or in spp.weights vector. See
\code{\link{organize.syncsa}}.
}
\examples{
data(ADRS)
matrix.w.transformation(ADRS$community)
}
\references{
Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
587-596.
De Cáceres, M.; Legendre, P. 2008. Beals smoothing revisited. Oecologia 156: 657–669.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{matrix.p}},
\code{\link{matrix.t}}, \code{\link{matrix.x}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

\encoding{UTF-8}
\name{flona}
\alias{flona}
\docType{data}
\title{Hypothetical data for SYNCSA}
\usage{
data(flona)
}
\description{Hypothetical data for running examples.}
\format{
  A list with 4 matrices.
  \describe{
    \item{commnunity}{Community data, observations of 59 species in 39 sites.}
    \item{traits}{Matrix data of species described by five traits.}
    \item{phylo}{Matrix containing phylogenetic distance between species.}
    \item{environment}{Three environmental variables for each community.}
    }
}

\examples{
data(flona)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var.type.R
\encoding{UTF-8}
\name{var.type}
\alias{var.type}
\title{Check the type of variables}
\usage{
var.type(data)
}
\arguments{
\item{data}{A data.frame or matrix.}
}
\value{
A vector with the variable types, where 'c' is continuous/numeric, 'o' is
ordinal, 'b' is binary, 'n' is nominal and 'f' is factor.
}
\description{
Function to check the type of variables in a data.frame or matrix. This function was extracted
and slightly modified of the function \code{\link[FD]{gowdis}}.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{var.dummy}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coinertia.partial.syncsa.R,
%   R/coinertia.syncsa.R
\encoding{UTF-8}
\name{coinertia.partial.syncsa}
\alias{coinertia.partial.syncsa}
\alias{coinertia.syncsa}
\title{Co-inertia and Partial Co-inertia correlations.}
\usage{
coinertia.partial.syncsa(x, y, z, scale = FALSE)

coinertia.syncsa(x, y, scale = FALSE)
}
\arguments{
\item{x, y}{Matrix that will be correlated.}

\item{z}{Matrix whose effect will be removed from the correlation between x and y.}

\item{scale}{Standardized variables to unit variance.}
}
\value{
RV coefficient (generalization of the Pearson correlation coefficient) between matrices.
}
\description{
Function to obtain the RV coefficient between two matrices and
partial RV coefficient between three matrices.
}
\references{
Legendre, P. and Legendre L. (2012). Numerical Ecology. 3nd English edition. Elsevier.
}
\seealso{
\code{\link{syncsa}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimal.R, R/print.optimal.R
\encoding{UTF-8}
\name{optimal}
\alias{optimal}
\alias{print.optimal}
\title{Searching for optimal traits}
\usage{
optimal(
  comm,
  traits = NULL,
  envir = NULL,
  checkdata = TRUE,
  subset.min = 1,
  subset.max = ncol(traits),
  pattern = NULL,
  ro.method = "mantel",
  dist = "euclidean",
  method = "pearson",
  scale = TRUE,
  scale.envir = TRUE,
  ranks = TRUE,
  asym.bin = NULL,
  ord = "metric",
  put.together = NULL,
  na.rm = FALSE,
  notification = TRUE,
  progressbar = FALSE
)

\method{print}{optimal}(x, ...)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.
Alternatively comm can be an object of class metacommunity.data, an alternative
way to set all data.frames/matrices. When you use the class metacommunity.data the arguments
traits, envir and put.together must be null. See details.}

\item{traits}{Matrix data of species described by traits, with traits as
columns and species as rows (Default traits = NULL).}

\item{envir}{Environmental variables for each community, with variables as
columns and sampling units as rows (Default envir = NULL).}

\item{checkdata}{Logical argument (TRUE or FALSE) to check if species
sequence in the community data follows the same order as the one in the
trait and if sampling units in the community data follows the same order as the one in the
environmental matrices (Default checkdata = TRUE).}

\item{subset.min}{Minimum of traits in each subset (Default subset.min = 1).}

\item{subset.max}{Maximum of traits in each subset (Default subset.max = ncol(traits)).}

\item{pattern}{Patterns for maximize correlation, "tcap", "tdap",
"tcap.tdap" or "rao" (Default pattern = NULL).}

\item{ro.method}{Method to obtain the correlation, "mantel" or "procrustes"
(Default ro.method = "mantel").}

\item{dist}{Dissimilarity index, as accepted by vegdist: "manhattan",
"euclidean", "canberra", "bray", "kulczynski", "jaccard", "gower",
"altGower", "morisita", "horn", "mountford", "raup" , "binomial" or "chao".}

\item{method}{Correlation method, as accepted by cor: "pearson", "spearman"
or "kendall".}

\item{scale}{Logical argument (TRUE or FALSE) to specify if the traits are
measured on different scales (Default Scale = TRUE). When scale = TRUE traits
are measured on different scales and the matrix T is subjected to
standardization within each trait. When scale = FALSE if traits are measured on
the same scale and the matrix T is not subjected to standardization.
Furthermore, if scale = TRUE the matrix of traits is subjected to
standardization within each trait, and Gower Index is used to calculate the
degree of belonging to the species, and if scale = FALSE the matrix of
traits is not subjected to standardization, and Euclidean distance is
calculated to determine the degree of belonging to the species.}

\item{scale.envir}{Logical argument (TRUE or FALSE) to specify if the
environmental variables are measured on different scales (Default scale =
TRUE). If the enviromental variables are measured on different scales, the
matrix is subjected to centralization and standardization within each
variable.}

\item{ranks}{Logical argument (TRUE or FALSE) to specify if ordinal variables are
convert to ranks (Default ranks = TRUE).}

\item{asym.bin}{Vector listing the asymmetric binary traits, see \code{\link[FD]{gowdis}} (Default asym.bin = NULL).}

\item{ord}{Method to be used for ordinal traits, see \code{\link[FD]{gowdis}} (Default ord = "metric").}

\item{put.together}{List to specify group traits that are added or removed
together (Default put.together = NULL). This argument must be a list, see
examples.}

\item{na.rm}{Logical argument (TRUE or FALSE) to specify if pairwise
deletion of missing observations when computing dissimilarities (Default
na.rm = FALSE).}

\item{notification}{Logical argument (TRUE or FALSE) to specify if
notifications of missing observations are shown (Default notification =
TRUE).}

\item{progressbar}{Logical argument (TRUE or FALSE) to specify if display a
progress bar on the R console (Default progressbar = FALSE).}

\item{x}{An object of class optimal.}

\item{...}{Other parameters for the respective functions.}
}
\value{
\item{Subset}{Subset of traits that maximizes the correlation.}
\item{ro}{Correlation for the subset of traits.}
}
\description{
Maximize trait-convergence assembly patterns (TCAP = roTE), trait-divergence
assembly patterns (TDAP = roXE.T), maximize both trait-divergence assembly
patterns and trait-convergence assembly patterns (TCAP.TDAP = roXE) or
alpha divergence (roRE) For more details, see \code{\link{syncsa}}.
}
\details{
Package \strong{SYNCSA} requires that the species and community sequence in
the data.frame or matrix must be the same for all dataframe/matrices.
The function \code{\link{organize.syncsa}} organizes the data for the functions
of the package, placing the matrices of community, traits and
environmental varibles in the same order. The function
use of function organize.syncsa is not requered for run the functions, but
is recommended. In this way the arguments comm, traits, envir, as well as the argument
put.together, can be specified them as normal arguments or by passing them
with the object returned by the function \code{\link{organize.syncsa}} using, in this
case only the argument comm. Using the object returned by organize.syncsa, the comm argument
is used as an alternative way of entering to set all data.frames/matrices, and therefore
the other arguments (traits, envir, and put.together) must be null.
}
\note{
\strong{IMPORTANT}: The sequence species show up in community data
matrix MUST be the same as they show up in traits matrix. See details and
\code{\link{organize.syncsa}}.
}
\examples{
data(flona)
optimal(flona$community, flona$traits, flona$environment, subset.min = 3,
   subset.max = 5, pattern = "tcap")
optimal(flona$community, flona$traits, flona$environment, subset.min = 3,
   subset.max = 5, pattern = "tdap")
optimal(flona$community, flona$traits, flona$environment, subset.min = 3,
   subset.max = 5, pattern = "tcap.tdap")
put.together <- list(c("fol", "sem"), c("tam", "red"))
put.together
optimal(flona$community, flona$traits, flona$environment, subset.min = 1,
   subset.max = 3, pattern = "tcap", put.together = put.together)
}
\references{
Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
587-596.

Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
Discriminating trait-convergence and trait-divergence assembly patterns in
ecological community gradients. Journal of Vegetation Science, 20, 334:348.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

\encoding{UTF-8}
\name{ADRS}
\alias{ADRS}
\docType{data}
\title{Artificial Data for Run SYNCSA}
\usage{
data(ADRS)
}
\description{Artificial data for run SYNCSA.}
\format{
  A list with:.
  \describe{
    \item{commnunity}{Community data, observations of six species in ten sites.}
    \item{traits}{Matrix data of species described by two traits.}
    \item{phylo}{Matrix containing phylogenetic distance between species.}
    \item{envir}{One environmental variable for each community.}
  }
}
\examples{
data(ADRS)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cent.norm.R
\encoding{UTF-8}
\name{cent.norm}
\alias{cent.norm}
\title{Matrix centralization and standardization}
\usage{
cent.norm(x, na.rm = FALSE)
}
\arguments{
\item{x}{A matrix}

\item{na.rm}{Logical argument (TRUE or FALSE) to specify if missing
observations are removed (Default na.rm = FALSE).}
}
\description{
Function to perform centralization and standardization in a matrix.
}
\seealso{
\code{\link{syncsa}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CollectNames.R
\encoding{UTF-8}
\name{CollectNames}
\alias{CollectNames}
\title{Collect names an entire list}
\usage{
CollectNames(l, prefix = NULL)
}
\arguments{
\item{l}{A list.}

\item{prefix}{A prefix to nomes.}
}
\value{
The names.
}
\description{
Function to collect names an entire list.
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var.dummy.R
\encoding{UTF-8}
\name{var.dummy}
\alias{var.dummy}
\title{Generate dummy variable}
\usage{
var.dummy(data)
}
\arguments{
\item{data}{A data.frame.}
}
\value{
\item{data}{The data with all variables of class factor expanded.} \item{together}{A list
with sugestion to group of traits that are analysed together.}
}
\description{
Function to expand factor variables in dummy variables in a data.frame. See details.
}
\details{
Variables in the data.frame of class factor is expanded in dummy variables. Each
level of factor produce a new column in the dataframe, with presence (1) or
absense (0) of level. The name of columns is a combination of orginal variable name
plus the level separate by underscore ( _ ). Ordered factor and character class are
not expanded.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{var.type}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix.t.R
\encoding{UTF-8}
\name{matrix.t}
\alias{matrix.t}
\title{Matrix T}
\usage{
matrix.t(
  comm,
  traits,
  scale = TRUE,
  ranks = TRUE,
  transformation = "standardized",
  spp.weights = NULL,
  notification = TRUE
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.}

\item{traits}{Matrix or data frame of species described by traits, with traits as
columns and species as rows.}

\item{scale}{Logical argument (TRUE or FALSE) to specify if the traits are
measured on different scales (Default scale = TRUE). When scale = TRUE traits
are measured on different scales and the matrix T is subjected to
standardization within each trait. When scale = FALSE traits are measured on
the same scale the matrix T is not subjected to standardization.}

\item{ranks}{Logical argument (TRUE or FALSE) to specify if ordinal variables are
convert to ranks (Default ranks = TRUE).}

\item{transformation}{Method to community data transformation, "none", "standardized" or "weights" (Default transformation = "standardized").}

\item{spp.weights}{Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).}

\item{notification}{Logical argument (TRUE or FALSE) to specify if
notifications of missing observations are shown (Default notification =
TRUE).}
}
\value{
\item{matriz.w}{Standardized community matrix, where rows are
communities and columns species. If default transformation, row totals (communities) = 1.}
\item{matriz.b}{Matrix of traits, exactly the same data input.}
\item{matriz.T}{Matrix containing trait averages at community level. If
Scale = TRUE the matrix T is standardized within the traits.}
}
\description{
Function to obtain a matrix containing trait averages at community level.
For more details, see \code{\link{syncsa}}.
}
\note{
\strong{IMPORTANT}: The sequence species show up in community data
matrix MUST be the same as they show up in traits matrix or in the spp.weights vector. See
\code{\link{organize.syncsa}}.
}
\examples{
data(ADRS)
matrix.t(ADRS$community, ADRS$traits)
}
\references{
Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
587-596.

Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
Discriminating trait-convergence and trait-divergence assembly patterns in
ecological community gradients. Journal of Vegetation Science, 20, 334:348.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}},
\code{\link{matrix.p}}, \code{\link{matrix.x}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R, R/plot.pcasyncsa.R
\encoding{UTF-8}
\name{pca}
\alias{pca}
\alias{print.pcasyncsa}
\alias{plot.pcasyncsa}
\title{Principal Components Analysis (PCA) with NA (missing data)}
\usage{
pca(data)

\method{plot}{pcasyncsa}(
  x,
  show = c("variables", "individuals"),
  axis = c(1, 2),
  xlab = axis[1],
  ylab = axis[2],
  arrows = TRUE,
  text = TRUE,
  points = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame  or matrix with individuals in rows and variables in columns.}

\item{x}{A object of class pcasyncsa.}

\item{show}{Draw "variables" or "individuals".}

\item{axis}{Axis for draw, must have length equal to two (Default axis = c(1, 2)).}

\item{xlab}{Text for x label (Default xlab = axis[1]).}

\item{ylab}{Text for y label (Default ylab = axis[2]).}

\item{arrows}{Logical argument (TRUE or FALSE) to specify if arrows are showed for variables (Default arrows = TRUE).}

\item{text}{Logical argument (TRUE or FALSE) to specify if text are showed for individuals (Default text = TRUE).}

\item{points}{Logical argument (TRUE or FALSE) to specify if points are showed for individuals (Default points = FALSE).}

\item{...}{Parameters for \code{\link{plot}} function.}
}
\value{
\item{decomposition}{list with the results of decomposition of correlation matrix.}
\item{eigenvalues}{Data frame containing all the eigenvalues, the
percentage of inertia and the cumulative percentage of inertia.}
\item{individuals}{Coordinates for the individuals.}
\item{variables}{Correlation between original variables and axes.}
}
\description{
The function use the option "pairwise.complete.obs" (in function \code{\link{cor}}) for
calculate the correlation. The correlation between each pair of variables is computed
using all complete pairs of observations on those variables.
}
\examples{
data(ADRS)
traits<-ADRS$traits
# Some NA
traits[c(1,5),1]<-NA
traits[3,2]<-NA
traits
res<-pca(traits)
res
plot(res, show = "variables", arrows = TRUE)
plot(res, show = "individuals", axis = c(1, 2), text = TRUE)
plot(res, show = "individuals", text = FALSE, points = TRUE)
}
\seealso{
\code{\link{syncsa}}
\code{\link{syncsa}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/belonging.R
\encoding{UTF-8}
\name{belonging}
\alias{belonging}
\title{Degree of belonging of species}
\usage{
belonging(dis, standardize = TRUE)
}
\arguments{
\item{dis}{Matrix containing distance between species.}

\item{standardize}{Logical argument (TRUE or FALSE) to specify if dis must
be standardized in values ranging from 0 to 1 (Default standardize = TRUE).}
}
\value{
Standardized matrix containing the degree of belonging of species in
relation to each other. Row totals (species) = 1.
}
\description{
Function to obtain a matrix containing the degrees of belongings of each
and every species. The degree of belonging of each species is defined based on
its ecological or phylogenetic resemblance to every other
species in the community. For more details, see \code{\link{matrix.p}},
\code{\link{matrix.x}} and \code{\link{syncsa}}.
}
\examples{
data(ADRS)
belonging(ADRS$phylo)
}
\references{
Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
587-596.
}
\seealso{
\code{\link{matrix.p}}, \code{\link{matrix.x}},
\code{\link{syncsa}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rao.diversity.R
\encoding{UTF-8}
\name{rao.diversity}
\alias{rao.diversity}
\title{Rao's quadratic entropy}
\usage{
rao.diversity(
  comm,
  traits = NULL,
  phylodist = NULL,
  checkdata = TRUE,
  ord = "metric",
  put.together = NULL,
  standardize = TRUE,
  transformation = "standardized",
  spp.weights = NULL,
  ...
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.
Alternatively comm can be an object of class metacommunity.data, an alternative
way to set all data.frames/matrices. When you use the class metacommunity.data the arguments
traits, phylodist and put.together must be null. See details.}

\item{traits}{Data frame or matrix data of species described by traits, with traits as
columns and species as rows (Default traits = NULL).}

\item{phylodist}{Matrix containing phylogenetic distance between species
(Default phylodist = NULL).}

\item{checkdata}{Logical argument (TRUE or FALSE) to check if species
sequence in the community data follows the same order as the one in the
trait and in the phylodist matrices (Default checkdata = TRUE).}

\item{ord}{Method to be used for ordinal variables, see \code{\link[FD]{gowdis}}
(Default ord = "metric").}

\item{put.together}{List to specify group of traits. Each group specify receive the
same weight that one trait outside any group, in the way each group is considered
as unique trait (Default put.together = NULL). This argument must be a list, see
examples in \code{\link{syncsa}}.}

\item{standardize}{Logical argument (TRUE or FALSE) to specify if standardize phylogenetic
distance to range into range 0 to 1. (Default standardize = TRUE).}

\item{transformation}{Method to transformation, "none", "standardized", "weights"
or "max.weights" (Default transformation = "standardized").}

\item{spp.weights}{Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).}

\item{...}{Parameters for \code{\link[FD]{gowdis}} function.}
}
\value{
\item{Simpson}{Gini-Simpson index within each community (equivalent
to Rao quadratic entropy with null, crisp, similarities).}
\item{FunRao}{Rao quadratic entropy within each community, considering trait distance.}
\item{FunRedundancy}{Functional redundancy in each community.}
\item{PhyRao}{Rao quadratic entropy within each community, considering phylogenetic distance.}
\item{PhyRedundancy}{Phylogenetic redundancy in each community.}
}
\description{
Calculates Rao's quadratic entropy, functional and phylogenetic redundancy.
}
\details{
Rao's quadratic entropy is a measure of diversity of ecological communities
defined by Rao (1982) and is based on the proportion of the abundance of
species present in a community and some measure of dissimilarity among them.
The dissimilarity range from 0 to 1 and is based on a set of specified
functional traits or in the phylogenetic dissimilarity.

For the trait data, the function calculates the square root of the
one-complement of Gower`s similarity index, in order to have a dissimilarity
matrix with Euclidean metric properties. Gower`s index ranges from 0 to 1
and can handle traits measured indifferent scales. When the species are
completely different in terms of their traits, Rao quadratic entropy is
equivalent to the Gini-Simpson index. Traits data can be numeric, factor or
ordered factor. For this be considered traits data must be of data.frame
class and containing each variable type determined. For additional details and
requirements of function please see \code{\link[FD]{gowdis}}.

Functional redundancy is defined purely as the difference between species
diversity and Rao`s quadratic entropy based on their functional
dissimilarity (de Bello et al. 2007). The same definition is used for
phylogenetic redundancy.

By default, the community data is standardization to row totals will be 1.
The options are: "none" no transformation is applied; "standardized" the
community data is standardized to row totals will be 1; and "weights" community
data is first "standardized" and when, individual species weights are multiplied
to each species entries. The argument transformation also allow an additional
method to weights individual species, called "max.weights". In this method
community data is standardization as default, however dissimilarity matrix is
weighted indeed. The argument spp.weights specify the weights to target species
and the dissimilarity weights are 1 if at least one species in the pair belongs
to the target community, otherwise weights to pair are 0.

Package \strong{SYNCSA} requires that the species and community sequence in
the data.frame or matrix must be the same for all dataframe/matrices.
The function \code{\link{organize.syncsa}} organizes the data for the functions
of the package, placing the matrices of community, traits, phylogenetic distance
in the same order. The function use of function organize.syncsa is not requered
for run the functions, but is recommended. In this way the arguments comm, traits,
phylodist, as well as the argument put.together and spp.weights, can be specified them as normal
arguments or by passing them with the object returned by the function
\code{\link{organize.syncsa}} using, in this case only the argument comm.
Using the object returned by organize.syncsa, the comm argument is used as an alternative
way of entering to set all data.frames/matrices, and therefore the other arguments
(traits, phylodist, put.together and spp.weights) must be null.
}
\note{
\strong{IMPORTANT}: The sequence species show up in community data
matrix MUST be the same as they show up in traits and phylodist matrices as well as
in the species weights vector. See details and \code{\link{organize.syncsa}}.
}
\examples{
data(ADRS)
rao.diversity(ADRS$community)
rao.diversity(ADRS$community, traits = ADRS$traits)
}
\references{
de Bello, F.; Leps, J.; Lavorel, S. & Moretti, M. (2007).
Importance of species abundance for assessment of trait composition: an
example based on pollinator communities. Community Ecology, 8, 163:170.

Pillar, V.D.; Blanco, C.C.; Muler, S.C.; Sosinski, E.E.; Joner, F. & Duarte,
L.d.S. (2013). Functional redundancy and stability in plant communities.
Journal of Vegetation Science, 24, 963:974.

Rao, C.R. (1982). Diversity and dissimilarity coefficients: a unified
approach. Theoretical Population Biology, 21, 24:43.
}
\seealso{
\code{\link{organize.syncsa}}, \code{\link[FD]{gowdis}},
\code{\link{syncsa}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permut.vector.R
\encoding{UTF-8}
\name{permut.vector}
\alias{permut.vector}
\title{Permutate a vector}
\usage{
permut.vector(n, strata = NULL, nset = 999)
}
\arguments{
\item{n}{The length of vector.}

\item{strata}{A vector to specify restricting permutations.}

\item{nset}{The number of permutations to generate for the set (Default strata = NULL).}
}
\value{
A matrix of permutations, where each row is a separate permutation.
}
\description{
Function to permutate a vector of size n using the function \code{\link[permute]{shuffleSet}}.
}
\seealso{
\code{\link{syncsa}}, \code{\link{permut.row.matrix}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matmult.syncsa.R
\encoding{UTF-8}
\name{matmult.syncsa}
\alias{matmult.syncsa}
\title{Matrix multiplication with missing data}
\usage{
matmult.syncsa(X, Y)
}
\arguments{
\item{X}{A matrix, typically the standardized community matrix (W).}

\item{Y}{A matrix.}
}
\value{
The matrix product.
}
\description{
Function to get the matrix product when missing data (NA) are found in matrix Y.
The matrix X must be standardized. See details.
}
\details{
The function ignore missing data when found in matrix Y. Before multiplication of matrices
the missing data in Y are replaced by 0 and multiplication is performed, and then, an adjustment is
performed. This adjustment is done by divide each cell of the product matrix by the sum of
proportions of nonzero at X with complete data in Y. In SYNCSA context this adjustment is done by
divide each cell of the product matrix by the sum of species proportion with trait data in Y.
The matrix X must be standardized, in other words, row totals must be equal to 1.
}
\seealso{
\code{\link{syncsa}}, \code{\link{matmult}},
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organize.syncsa.R
\encoding{UTF-8}
\name{organize.syncsa}
\alias{organize.syncsa}
\title{Function for organize data for Package SYNCSA}
\usage{
organize.syncsa(
  comm,
  traits = NULL,
  phylodist = NULL,
  envir = NULL,
  strata = NULL,
  spp.weights = NULL,
  check.comm = TRUE,
  convert.traits = FALSE,
  ranks = TRUE
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows.}

\item{traits}{Matrix or data.frame of species described by traits, with traits as
columns and species as rows (Default traits = NULL).}

\item{phylodist}{Matrix containing phylogenetic distance between species.
Must be a complete matrix (not a half diagonal matrix).This matrix can be
larger than community data (more species) as long as it has at least all
species that are in community data (Default phylodist = NULL).}

\item{envir}{Environmental variables for each community, with variables as
columns and sampling units as rows (Default envir = NULL).}

\item{strata}{Strata named vector to specify restricting permutations within
species groups (Default strata = NULL).}

\item{spp.weights}{Named vector to specify individual species weights (Default spp.weights = NULL).}

\item{check.comm}{Logical argument (TRUE or FALSE) to remove sampling units and
species with total sums equal or less than zero (Default check.comm = TRUE).}

\item{convert.traits}{Logical argument (TRUE or FALSE) to convert factor traits in
dummy traits and/or convert ordinal variables in numeric (see ranks argument)
(Default convert.traits = FALSE).}

\item{ranks}{Logical argument (TRUE or FALSE) to specify if ordinal variables are
convert to ranks. If ranks = TRUE all ordered variable are replaced by their ranks
and if ranks = FALSE all ordinal variables are simply treated as continuous variables
(Default ranks = TRUE).}
}
\value{
A object of class metacommunity.data (also of the class list) with the data.frames or matrices:
\item{call}{The arguments used.}
\item{community}{Community data.}
\item{traits}{Traits data.}
\item{phylodist}{Phylogenetic distance.}
\item{environmental}{Environmental variables.}
\item{community.var.type}{Type of each varible in community data,
where 'c' to continuous/numeric, 'o' to ordinal, 'b' to binary and 'f' to factor. Nominal are not allowed.}
\item{traits.var.type}{Type of each varible in traits data. See labels above.}
\item{phylodist.var.type}{Type of each varible in phylodist. See labels above.}
\item{environmental.var.type}{Type of each varible in environmental data. See labels above.}
\item{strata}{The strata vector for permutations.}
\item{put.together}{A list with suggestion to group of traits that are analyzed together,
only if convert.traits is TRUE and if some traits are of factor class.}
\item{spp.weights}{The individual species weights vector.}
\item{list.warning}{A list of warning.}
}
\description{
Package \strong{SYNCSA} requires that the species and community sequence in
the data.frame or matrix must be the same for all data.frame/matrices.
}
\details{
The function organizes the data for the functions of the package
SYNCSA, placing the matrices of community, traits, phylogenetic
distance, environmental varibles, species weights and strata vectors in the same order.
The function use as reference the community data for organize all data.frame or matrices
in the same order that the sampling units names and species names found in community
data set. For this all data sets entered must be correctly named, with rows and columns
named. The matrices phylodist, traits, envir can be larger than community
data (more species and/or more sampling units) as long as it has at least
all species and/or sampling units that are in community data. The function
organizes the data despite the absence of one of the data.frames or matrices,
provided that the community data had been entered. Unspecified data will
appear as NULL.

When trait is a data.frame with different types of variables correctly identified and
the argument convert.traits is TRUE factor traits are expanded in dummy traits and ordinal variables
are converted in numeric according to ranks argument.

The individual species weights (spp.weights) and strata must be named vectors. The strata vector
is a vector for restrict permutation within species groups, insofar as the SYNCSA package the null
models are based in permutation of species rather than permutation of sample units.
}
\examples{
data(ADRS)
organize.syncsa(ADRS$community, ADRS$traits, ADRS$phylo, ADRS$envir)
}
\seealso{
\code{\link{syncsa}}, \code{\link{var.dummy}}, \code{\link{var.type}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix.p.R
\encoding{UTF-8}
\name{matrix.p}
\alias{matrix.p}
\title{Matrix P}
\usage{
matrix.p(
  comm,
  phylodist,
  transformation = "standardized",
  spp.weights = NULL,
  notification = TRUE
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.}

\item{phylodist}{Matrix containing phylogenetic distance between species.
Must be a complete matrix (not a diagonal resemblance matrix).}

\item{transformation}{Method to community data transformation, "none", "standardized" or "weights" (Default transformation = "standardized").}

\item{spp.weights}{Vector with 0 or 1 to specify individual species weights (Default spp.weights = NULL).}

\item{notification}{Logical argument (TRUE or FALSE) to specify if
notifications for missing observations are to be shown (Default notification =
TRUE).}
}
\value{
\item{matrix.w}{Standardized community matrix, where rows are
communities and columns species. If default transformation, row totals (communities) = 1.}
\item{matrix.q}{Standardized matrix containing the degree of belonging of
species in relation to each other. Row totals (species) = 1.}
\item{matrix.P}{Phylogeny-weighted species composition matrix. If default transformation, row totals (communities) = 1.}
}
\description{
Function to obtain a matrix containing phylogeny-weighted species
composition. For more details, see \code{\link{syncsa}}.
}
\note{
\strong{IMPORTANT}: Species sequence in the community data
matrix MUST be the same as the one in the phylogenetic distance matrix or in the spp.weights vector. See
\code{\link{organize.syncsa}}.
}
\examples{
data(ADRS)
matrix.p(ADRS$community, ADRS$phylo)
}
\references{
Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
587-596.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{belonging}},
\code{\link{matrix.t}}, \code{\link{matrix.x}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/procrustes.syncsa.R,
%   R/procrustes.partial.syncsa.R
\encoding{UTF-8}
\name{procrustes.syncsa}
\alias{procrustes.syncsa}
\alias{procrustes.partial.syncsa}
\title{Procrustes and Partial Procrustes correlations.}
\usage{
procrustes.syncsa(x, y)

procrustes.partial.syncsa(x, y, z)
}
\arguments{
\item{x, y}{Matrix that will be correlated.}

\item{z}{Matrix whose effect will be removed from the correlation between x
and y.}
}
\value{
Procrustes correlation between matrices.
}
\description{
Function to obtain the Procrustes correlation between two matrices and
partial Procrustes correlation between three matrices.
}
\details{
The function procrustes.syncsa is a small change in the function
\code{\link[vegan]{procrustes}}. The function uses a correlation-like statistic
derived from the symmetric Procrustes sum of squares. Partial Procrustes
correlations is obtained by Procrustes correlation of residuals between
each variable in z being used as linear predictor of all variables in x and y.
For more details, see \code{\link{syncsa}}.
}
\references{
Peres-Neto, P.R. and Jackson, D.A. (2001). How well do
multivariate data sets match? The advantages of a Procrustean
superimposition approach over the Mantel test. Oecologia 129: 169-178.
}
\seealso{
\code{\link{syncsa}}, \code{\link{cor.matrix}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/part.cor.R
\encoding{UTF-8}
\name{part.cor}
\alias{part.cor}
\title{First-order partial correlation coefficient}
\usage{
part.cor(rxy, rxz, ryz)
}
\arguments{
\item{rxy}{Correlation between xy}

\item{rxz}{Correlation between xz}

\item{ryz}{Correlation between yz}
}
\value{
The first-order partial correlation coefficient.
}
\description{
Function to obtain the first-order partial correlation coefficient.
}
\seealso{
\code{\link{syncsa}}, \code{\link{cor.matrix}},
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix.x.R
\encoding{UTF-8}
\name{matrix.x}
\alias{matrix.x}
\title{Matrix X}
\usage{
matrix.x(
  comm,
  traits,
  scale = TRUE,
  ranks = TRUE,
  transformation = "standardized",
  spp.weights = NULL,
  notification = TRUE,
  ...
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.}

\item{traits}{Matrix or data frame of species described by traits, with traits as
columns and species as rows.}

\item{scale}{Logical argument (TRUE or FALSE) to specify if the traits are
measured on different scales (Default scale = TRUE). When scale = TRUE traits
are measured on different scales the matrix of traits is subjected to
standardization within each trait, and Gower Index is used to calculate the
degree of belonging to the species. When scale = FALSE traits are measured on
the same scale the matrix of traits is not subjected to standardization,
and Euclidean distance is calculated to determine the degree of belonging to
the species.}

\item{ranks}{Logical argument (TRUE or FALSE) to specify if ordinal variables are
convert to ranks (Default ranks = TRUE).}

\item{transformation}{Method to community data transformation, "none", "standardized"
or "weights" (Default transformation = "standardized").}

\item{spp.weights}{Vector with 0 or 1 to specify individual species weights (Default
spp.weights = NULL).}

\item{notification}{Logical argument (TRUE or FALSE) to specify if
notifications of missing observations are shown (Default notification =
TRUE).}

\item{...}{Parameters for \code{\link[FD]{gowdis}} function.}
}
\value{
\item{matriz.w}{Standardized community matrix, where rows are
communities and columns species. If default transformation, row totals (communities) = 1.}
\item{matriz.u}{Standardized matrix containing the degree of belonging of
each species in relation to each other species. Row totals (species) = 1.}
\item{matriz.X}{Trait-weighted species composition matrix. If default transformation,
row totals (communities) = 1.}
}
\description{
Function to obtain a matrix containing trait-weighted species composition.
For more details, see \code{\link{syncsa}}.
}
\note{
\strong{IMPORTANT}: The sequence species show up in community data
matrix MUST be the same as they show up in traits matrix or in the spp.weights vector. See
\code{\link{organize.syncsa}}.
}
\examples{
data(ADRS)
matrix.x(ADRS$community, ADRS$traits)
}
\references{
Pillar, V.D.; Duarte, L.d.S. (2010). A framework for
metacommunity analysis of phylogenetic structure. Ecology Letters, 13,
587-596.

Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
Discriminating trait-convergence and trait-divergence assembly patterns in
ecological community gradients. Journal of Vegetation Science, 20, 334:348.
}
\seealso{
\code{\link{syncsa}}, \code{\link{organize.syncsa}}, \code{\link{belonging}},
\code{\link{matrix.t}}, \code{\link{matrix.p}}, \code{\link[FD]{gowdis}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ProgressBAR.R
\encoding{UTF-8}
\name{ProgressBAR}
\alias{ProgressBAR}
\title{Text Progress Bar}
\usage{
ProgressBAR(n, N, ...)
}
\arguments{
\item{n}{Number of the current progress.}

\item{N}{Total number of cases.}

\item{...}{Other parameters for the txtProgressBar function.}
}
\description{
Function to display a progress bar in the R console. See \code{\link{txtProgressBar}}.
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{Auxiliary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/syncsa.R, R/print.syncsa.R, R/syncsa.obs.R
\encoding{UTF-8}
\name{syncsa}
\alias{syncsa}
\alias{SYNCSA}
\alias{print.syncsa}
\alias{syncsa.obs}
\title{SYNCSA}
\usage{
syncsa(
  comm,
  traits = NULL,
  phylodist = NULL,
  envir = NULL,
  checkdata = TRUE,
  ro.method = "mantel",
  method = "pearson",
  dist = "euclidean",
  scale = TRUE,
  scale.envir = TRUE,
  ranks = TRUE,
  asym.bin = NULL,
  ord = "metric",
  put.together = NULL,
  na.rm = FALSE,
  transformation = "standardized",
  spp.weights = NULL,
  strata = NULL,
  permutations = 999,
  parallel = NULL,
  notification = TRUE
)

\method{print}{syncsa}(x, ...)

syncsa.obs(
  comm,
  traits = NULL,
  phylodist = NULL,
  envir = NULL,
  scale = TRUE,
  scale.envir = TRUE,
  ranks = TRUE,
  asym.bin = NULL,
  ord = "metric",
  put.together = NULL,
  na.rm = FALSE,
  transformation = "standardized",
  spp.weights = NULL,
  notification = TRUE
)
}
\arguments{
\item{comm}{Community data, with species as columns and sampling units as
rows. This matrix can contain either presence/absence or abundance data.
Alternatively comm can be an object of class metacommunity.data, an alternative
way to set all data.frames/matrices. When you use the class metacommunity.data the arguments
traits, phylodist, envir and put.together must be null. See details.}

\item{traits}{Data frame or matrix data of species described by traits, with traits as
columns and species as rows (Default traits = NULL).}

\item{phylodist}{Matrix containing phylogenetic distance between species.
Must be a complete matrix, not a half diagonal matrix (Default phylodist = NULL).}

\item{envir}{Environmental variables for each community, with variables as
columns and sampling units as rows (Default envir = NULL).}

\item{checkdata}{Logical argument (TRUE or FALSE) to check if species
sequence in the community data follows the same order as the one in the
trait and in the phylodist matrices and if sampling units in the community data follows
the same order as the one in the environmental data (Default checkdata = TRUE).}

\item{ro.method}{Method to obtain the correlation, "mantel", "procrustes", "coinertia",
"none" or "nullmodel" (Default ro.method = "mantel").}

\item{method}{Mantel correlation method, as accepted by cor: "pearson",
"spearman" or "kendall" (Default method = "pearson").}

\item{dist}{Dissimilarity index used for Mantel correlation, as accepted by
vegdist: "manhattan", "euclidean", "canberra", "bray", "kulczynski",
"jaccard", "gower", "altGower", "morisita", "horn", "mountford", "raup" ,
"binomial" or "chao" (Default dist = "euclidean").}

\item{scale}{Logical argument (TRUE or FALSE) to specify if the traits are
measured on different scales (Default Scale = TRUE). When scale = TRUE traits
are measured on different scales the the matrix T is subjected to
standardization within each trait. When scale = FALSE traits are measured on
the same scale and the matrix T is not subjected to standardization.
Furthermore, if scale = TRUE the matrix of traits is subjected to
standardization within each trait, and Gower Index is used to calculate the
degree of belonging to the species, and if scale = FALSE the matrix of
traits is not subjected to standardization, and Euclidean distance is
calculated to determine the degree of belonging to the species.}

\item{scale.envir}{Logical argument (TRUE or FALSE) to specify if the
environmental variables are measured on different scales. If the
enviromental variables are measured on different scales, the scale.envir
= TRUE the matrix with enviromental variables is subjected to centralization
and standardization within each variable. (Default scale.envir = TRUE).}

\item{ranks}{Logical argument (TRUE or FALSE) to specify if ordinal variables are
convert to ranks (Default ranks = TRUE).}

\item{asym.bin}{Vector listing the asymmetric binary traits, see \code{\link[FD]{gowdis}} (Default asym.bin = NULL).}

\item{ord}{Method to be used for ordinal traits, see \code{\link[FD]{gowdis}} (Default ord = "metric").}

\item{put.together}{List to specify group of traits. Each group specify receive the
same weight that one trait outside any group, in the way each group is considered
as unique trait (Default put.together = NULL). This argument must be a list, see
examples.}

\item{na.rm}{Logical argument (TRUE or FALSE) to specify if pairwise
distances should be deleted in cases of missing observations (Default na.rm
= FALSE).}

\item{transformation}{Method to transformation, "none", "standardized" or "weights"
(Default transformation = "standardized").}

\item{spp.weights}{Vector with 0 or 1 to specify individual species weights (Default
spp.weights = NULL).}

\item{strata}{Strata named vector to specify restricting permutations within species
groups (Default strata = NULL).}

\item{permutations}{Number of permutations in assessing significance.}

\item{parallel}{Number of parallel processes. Tip: use parallel::detectCores() (Default parallel = NULL).}

\item{notification}{Logical argument (TRUE or FALSE) to specify if
notification of missing observations should to be shown (Default
notification = TRUE).}

\item{x}{An object of class syncsa.}

\item{...}{Other parameters for the respective functions.}
}
\value{
\item{call}{The arguments used.}
\item{notes}{Some notes about the statistics.}
\item{statistics}{Correlations roTE, roXE, roPE, roPT, roPX.T, roXE.T, roTE.P, roXE.P
and roBF, and their significance levels based on permutations.}
\item{matrices}{A list with observed matrices produced for the functions, see details.}
\item{matrices.null}{A list of list with null matrices generated according the null model,
only if ro.method is "nullmodel". The number of matrices returned depends of the number of permutations.}
\item{FunRao}{Rao quadratic entropy within each community, considering trait distance.}
\item{traits.weights}{Weight for each trait.}
}
\description{
This function integrates several steps for the analysis of phylogenetic
assembly patterns and their links to traits and ecological processes in a
metacommunity (Pillar et al. 2009, Pillar & Duarte 2010, Debastiani & Pillar 2012). The
function implement methods that have been available in the SYNCSA
application written in C++ (by Valerio Pillar, available at
http://ecoqua.ecologia.ufrgs.br/SYNCSA.html). See details.
}
\details{
Package \strong{SYNCSA} requires that the species and community sequence in
the data.frame or matrix must be the same for all dataframe/matrices.
The function \code{\link{organize.syncsa}} organizes the data for the functions
of the package, placing the matrices of community, traits, phylogenetic distance,
environmental varibles and strata vector in the same order. The function
use of function organize.syncsa is not requered for run the functions, but
is recommended. It requires data organized into the following matrices: (1) the
presences or abundances of species in a set of communities (\strong{W}); (2) the
phylogenetic pairwise dissimilarities of these species (\strong{DF}); (3) a set of
functional traits describing the species (\strong{B}), which
may be a mixture of binary and quantitative traits (continual and ordinal),
but not nominal ones (these should be expanded into binary traits); and (4)
the ecological gradient of interest, which may be one or more factors to
which the communities respond or ecosystem effects of the communities
(\strong{E}). In this way the arguments comm, traits, phylodist, envir,
as well as the arguments put.together and strata, can be specified them as normal
arguments or by passing them with the object returned by the function
\code{\link{organize.syncsa}} using, in this case only the argument comm. Using the
object returned by organize.syncsa, the comm argument is used as an alternative way
of entering to set all data.frames/matrices, and therefore the other arguments
(traits, phylodist, envir, put.together and strata) must be null.


\strong{Correlations}

The function computes several correlations (Mantel or
Procrustes) that express trait-convergence assembly patterns (TCAP),
trait-divergence assembly patterns (TDAP), and phylogenetic signal in
functional traits at the species pool level and at the metacomunity level.
This function also generates P-values by permutation testing based on null
models (Pillar et al. 2009, Pillar & Duarte 2010).

\strong{ro(TE)}

This correlation refers to trait-convergence assembly patterns related to
the ecological gradient (TCAP, Pillar et al. 2009). For evaluating TCAP, by
matrix multiplication we define \strong{T = WB}, which with previous
standardization of \strong{W} to unit column totals will contain the trait
averages in each community. The elements in \strong{T} are community
weighted mean values or community functional parameters (Violle et al.
2007). Standardization of the traits (rows) in \strong{T} is needed if the
trait set contains traits measured with different scales. By using matrix
correlation, we evaluate how the trait patterns in \strong{T} are associated
to ecological gradients in \strong{E}. For relating \strong{T} to
\strong{E}, using Mantel correlation we define a distance matrix of the
communities (\strong{DT}) using \strong{T}, and another distance matrix of
the community sites (\strong{DE}) using \strong{E}. The matrix correlation
ro(\strong{TE}) = ro(\strong{DT};\strong{DE}) measures the level of
congruence between TCAP and \strong{E}. A strong correlation ro(\strong{TE})
indicates the factors directly or indirectly represented in \strong{E} are
involved in ecological filtering of species that, at least for the traits
considered in the analysis, consistently produce trait-convergence assembly
patterns along the gradient comprising the metacommunity.

\strong{ro(XE) and ro(XE.T)}

These matrix correlations refer to trait-divergence assembly patterns
related to the ecological gradient (TDAP, Pillar et al. 2009). For the
identification of TDAP, in a first step the species pairwise similarities
(in the range 0 to 1) in matrix \strong{SB} based on traits in \strong{B}
are used to define matrix \strong{U} with degrees of belonging of species to
fuzzy sets. By matrix multiplication \strong{X = WU} will contain the
species composition of the communities after fuzzy-weighting by their trait
similarities (each row in \strong{X} will refer to a species). Matrix
\strong{X} expresses both TCAP and TDAP (Pillar et al. 2009). By using
matrix correlation, we evaluate how the trait patterns in \strong{X} (TCAP
and TDAP) are associated to ecological gradients in \strong{E}. For relating
\strong{X} to \strong{E}, we define a distance matrix of the communities
(\strong{DX}) using \strong{X}, and another distance matrix of the community
sites (\strong{DE}) using \strong{E}. The matrix correlation ro(\strong{XE})
= ro(\strong{DX};\strong{DE}) between \strong{X} and \strong{E} is defined.
We then remove the trait-convergence component ro(\strong{TE}) from
ro(\strong{XE}) by computing the partial matrix correlation
ro(\strong{XE.T}), which measures the level of congruence between TDAP and
\strong{E}. Trait-divergence assembly patterns (TDAP, Pillar et al. 2009)
may result from community assembly processes related to biotic interactions
(Stubbs & Wilson 2004; Wilson 2007).

\strong{ro(PE)}

This matrix correlation refers to the phylogenetic structure related to the
ecological gradient comprising the metacommunity. The phylogenetic pairwise
dissimilarities in \strong{DF} are transformed into similarities and used to
define degrees of belonging qij to fuzzy sets. This is analogous to the
definition of functional fuzzy sets (Pillar & Orloci 1991; Pillar et al.
2009). Based on the phylogenetic similarities, every species i among s
species in the metacommunity specifies a fuzzy set to which every species j
(j = 1 to s species, including species i) belongs with a certain degree of
belonging in the interval [0, 1]. In our definition, each row in matrix
\strong{Q} with the degrees of belonging must add to unit, i.e., the degrees
of belonging of a given species across the fuzzy sets are standardized to
unit total. By matrix multiplication \strong{P = WQ} will contain the
composition of the communities after fuzzy-weighting of species presences or
abundances by the species` phylogenetic similarities. Each column in matrix
\strong{P} holds the phylogenetic structure of a community. The
standardization of \strong{Q} is essential for the community totals in each
column in \strong{W} remaining the same in \strong{P}. Further, matrix
\strong{W} is adjusted to unit column totals prior to multiplication, so
that the total richness or abundance within each community in \strong{W}
will be standardized. Matrix correlation ro(\strong{PE}) =
ro(\strong{DP};\strong{DE}) measures the strength of the association between
community distances based on their phylogenetic structure in \strong{DP} and
distances based on their ecological conditions (\strong{DE}). Further,
\strong{P} can be explored for phylogenetic patterns at the metacommunity
level by using, e.g., ordination techniques.

\strong{ro(PT) and ro(PX.T)}

These matrix correlations measure phylogenetic signal at the metacommunity
level related to TCAP and to TDAP. We define phylogenetic signal at the
metacommunity level related to TCAP (PSMT) as the correlation between the
phylogenetic structure described in matrix \strong{P} and the
trait-convergence structure described in matrix \strong{T}. For this, a
proper distance matrix (e.g. Euclidean distances) of communities
(\strong{DP}) is computed using \strong{P} and another distance matrix of
the same communities (\strong{DT}) is computed using \strong{T}. Then matrix
correlation ro(\strong{PT}) = ro(\strong{DP};\strong{DT}) will measure the
level of congruence between variation in \strong{P} and \strong{T}, which is
a measure of PSMT. A strong phylogenetic signal at the metacommunity level
is expected when communities that are more similar in terms of phylogenetic
structure are also similar regarding their average trait values. We also
define phylogenetic signal at the metacommunity level related to TDAP
(PSMX.T) as the partial matrix correlation ro(\strong{PX.T}) =
ro(\strong{DP};\strong{DX.DT}) between community distances DP computed on
phylogenetic structure and community distances \strong{DX} computed on
species composition after fuzzy-weighting by the species, or trait
similarities, removing the effect of TCAP (\strong{DT}). This is analogous
to TDAP.

\strong{ro(BF)}

This matrix correlation measures phylogenetic signal at the species pool
level (PSS, Pillar & Duarte 2010). We define PSS as the matrix correlation
ro(\strong{FB}) = ro(\strong{DF};\strong{DB}) between species phylogenetic
dissimilarities (already defined as matrix \strong{DF}) and species trait
dissimilarities (derived from already defined matrix \strong{SB}) computed
on any number of traits from matrix \strong{B}. The species pool refers to
the species present in the metacommunity.

\strong{Additional matrix correlations}

The matrix correlations ro(\strong{TE.P}) and ro(\strong{XE.P}) are also
computed, which may be useful for evaluating causal models in path analysis.

\strong{Mantel correlations}

The Mantel and Partial Mantel statistics are calculated simply as the correlation
entries the dissimilarity matrices, using standard Mantel test (see
\code{\link[vegan]{mantel}} and \code{\link{cor.matrix}}). Partial Mantel
statistic use paired correlation between the three matrices and obtains the partial
correlation using the formula of first-order partial correlation coefficient. The
significances are obtained using a different procedure than standard Mantel test,
see section Testing against null models below.

\strong{Procrustes correlations}

The Procrustes correlation uses symmetric Procrustes as a measure of concordance
between the data matrices (see \code{\link[vegan]{procrustes}} and
\code{\link{procrustes.syncsa}}). Procrustes procedure use rotation, translation,
and rescaling for minimizing sum of squared differences between two data sets.
The correlation of Procrustes is calculated as the statistic derived from the
symmetric Procrustes sum of squares, representing the optimal fit between the two
data matrices. Partial Procrustes correlation is obtained by Procrustes correlation
between residuals matrices. Firstly one Principal Components Analysis (PCA,
see \code{\link{prcomp}}) is performed in the matrix Z for dimensionality reduction.
The max number of axis kept in the analysis is the number of sampling units divided
by 2, this axes of PCA represent the total variation in the Z matrix. After the
kept axes are used as predictor in one linear model for each variable of the
matrices X and Y. For this a linear model is build using as response one variable
of X (same via for Y matrix) and as predictor all remaining axes of PCA, after model
fitted and the residual are extracted with the aim of form the residual matrix. The linear
model is repeated in the other variables, only with the changed the response variable.
The same procedure is performed in the matrix
Y. Both residual matrices are submitted to Procrustes analysis and the statistic is
returned as a partial correlation, the Partial Procrustes statistic. The significances
are obtained using the same procedure than Mantel test, see section Testing against
null models below.

\strong{Co-inertia correlations}

Co-inertia Analysis uses RV coefficient as a measure of correlation between the data matrices.
The procedure is equal to Procrustes correlation, however using the RV coefficient.

\strong{The ro.method "none" or "nullmodel"}

When ro.method is equal to "none" only observed matrices are returned,
already when it is equal to "nullmodel" observed matrices and null matrices
generated according the null models are returned. This null matrices can be
used to apply any statistical method to test relation between matrices not
include in the package (e.x. linear models).

\strong{Testing against null models}

All the matrix correlations are tested against null models. The null model
is defined accoding to the correlation being tested. Usually in the SYNCSA package
the null models are based in permutation of species rather than permutation
of sample units. For ro(\strong{TE}),each permutation generates a random
matrix \strong{T} calculated after the
permutation among the species vectors in matrix \strong{B}. For
ro(\strong{XE}) and ro(\strong{XE.T}), each permutation generates a random
matrix \strong{X} after the permutation among species fuzzy sets (rows) in
matrix \strong{U}. For ro(\strong{PE}), ro(\strong{PT}), and
ro(\strong{PX.T}), each permutation generates a random matrix \strong{P}
after the permutation among species fuzzy sets (rows) in matrix \strong{Q}.
For ro(\strong{BF}), a conventional Mantel test is performed with
dissimilarity matrices \strong{DF} and \strong{DB}. Analogous null models
are used for testing the additional matrix correlations; that is, the same
null model for ro(\strong{TE}) is used for ro(\strong{TE.P}), the same model
for ro(\strong{XE}) is used for ro(\strong{XE.P}). The permutation can be restrict
within species groups specifying the strata argument.

\strong{Traits types}

Traits data can be numeric, factor or ordered factor. For this be considered in the
analyses traits data must be of data.frame class and containing each variable type
determined. Gower index is used to calculate the similarity between species, using
the function gowdis of package FD. For additional details and requirements of function
please see \code{\link[FD]{gowdis}}.

\strong{Community data standardization}

By default the community data is standardization to row totals will be 1. The options
are: "none" no transformation is applied; "standardized" the community data is standardized
to row totals will be 1; and "weights" community data is first "standardized" and when, individual
species weights are multiplied to each species entries. An additional method to weights
individual species is included only \code{\link{rao.diversity}} function. Current version of package
does not allow choose standardization/transformation method in the \code{\link{optimal}} function.

\strong{Missing data}

The functions ignore missing data when specified. In the case of direct
multiplication of matrices the missing data are replaced by 0, ignoring the cell with missing value. For the
matrix \strong{T = WB} an adjustment is done by divide each cell of the product
matrix (\strong{T}) by the sum of species proportion with trait data in \strong{B}. Result
matrices are shown without missing values. Where the matrices are calculated
using a dissimilarity index (matrix \strong{U} and correlations between
matrices) the missing data are ignored as in \code{\link[vegan]{vegdist}} function.
In some cases the dissimilarity matrices obtained by the function
\code{\link[vegan]{vegdist}} still contain some missing values. In these cases the
rest of the procedure will be affected. In these cases you can find
solutions in impute the missing values.

\strong{Error messenger and options}

The data pass by several ckeck points that can produce error messenger. The
matrices or data frames must be contain only numeric, binary or ordinal
variables, in the way that nominal variable should be expanded into binary
(see \code{\link{var.dummy}}). For enhance the code speed some functions use
by default matrix algebra, this option can produce error under certain circumstances. This
global option can be changed using options("SYNCSA.speed" = FALSE). If SYNCSA.speed = TRUE
for use matrix algebra and if SYNCSA.speed = FALSE use not another function of same procedure.
}
\note{
The function calculates the correlations despite the lack of one of
the matrices, provided that community data had been entered. Correlations
including unspecified matrices will appear with NA.

\strong{IMPORTANT}: The sequence of species in the community data matrix
MUST be the same as that in the phylogenetic distance matrix and in traits
matrix. Similarly, the sequence of communities in the community data matrix
MUST be the same as that in the environmental data. See details and
\code{\link{organize.syncsa}}.
}
\examples{
data(ADRS)
syncsa(ADRS$community, ADRS$traits, ADRS$phylo, ADRS$envir, permutations = 99)
data(flona)
put.together<-list(c("fol","sem"), c("tam", "red"))
put.together
res<-syncsa(flona$community, flona$traits, envir = flona$environment,
   put.together = put.together, permutations = 99)
res$spp.weights
}
\references{
Debastiani, V.J & Pillar, V.D., (2012). SYNCSA-R tool for analysis of metacommunities
based on functional traits and phylogeny of the community components. Bioinformatics,
28(15), 2067–2068.

Pillar, V.D.; Duarte, L.d.S., (2010). A framework for metacommunity analysis
of phylogenetic structure. Ecology Letters, 13, 587:596.

Pillar, V.D., Duarte, L.d.S., Sosinski, E.E. & Joner, F. (2009).
Discriminating trait-convergence and trait-divergence assembly patterns in
ecological community gradients. Journal of Vegetation Science, 20, 334:348.

Pillar, V.D. & Orloci, L., (1991). Fuzzy components in community level
comparisons. In: Computer Assisted Vegetation Analysis (eds Feoli, E. &
Orloci, L.). Kluwer, Dordrecht, 87:93.

Stubbs, W.J. & Wilson, J.B., (2004). Evidence for limiting similarity in a
sand dune community. Journal of Ecology, 92, 557:567.

Violle, C., Navas, M.L., Vile, D., Kazakou, E., Fortunel, C., Hummel, I. &
Garnier, E., (2007). Let the concept of trait be functional! Oikos, 116,
882:892.

Wilson, J.B., (2007). Trait-divergence assembly rules have been demonstrated:
limiting similarity lives! A reply to Grime. Journal of Vegetation Science,
18, 451:452.
}
\seealso{
\code{\link{organize.syncsa}}, \code{\link{matrix.t}},
\code{\link{matrix.x}}, \code{\link{matrix.p}}, \code{\link{optimal}},
\code{\link{rao.diversity}}, \code{\link{cor.matrix}}, \code{\link{var.type}},
\code{\link{var.dummy}}
}
\author{
Vanderlei Julio Debastiani <vanderleidebastiani@yahoo.com.br>
}
\keyword{SYNCSA}
